@@ -61,8 +61,14 @@ AppWindow::AppWindow( BaseObjectType* cobject, const Glib::RefPtr<Gtk::Builder>&
     // DRAW UI
     try
     {
-        Lifeograph::get_builder()->get_widget( "Hb_main", m_Hb_main );
-        Lifeograph::get_builder()->get_widget( "St_main", m_St_main );
+        auto builder{ Lifeograph::get_builder() };
+        builder->get_widget( "Bx_main", m_Bx_main );
+        builder->get_widget( "Hb_main", m_Hb_main );
+        builder->get_widget( "St_main", m_St_main );
+        builder->get_widget( "IB_main", m_IB_info );
+        builder->get_widget( "L_infobar", m_L_info );
+        builder->get_widget( "B_infobar", m_B_info );
+
         m_view_login = new ViewLogin;
     }
     catch( ... )
@@ -70,13 +76,33 @@ AppWindow::AppWindow( BaseObjectType* cobject, const Glib::RefPtr<Gtk::Builder>&
         throw LIFEO::Error( "Failed to create the login view" );
     }
 
+#ifdef _WIN32
+    m_Hb_main->set_decoration_layout( ":minimize,maximize,close" );
+#endif
+
     m_A_logout_wo_saving = Lifeograph::p->add_action( "logout_wo_saving",
             sigc::bind( sigc::mem_fun( *this, &AppWindow::logout ), false ) );
 
     m_A_toggle_auto_logout = Lifeograph::p->add_action_bool( "toggle_auto_logout",
             sigc::mem_fun( *this, &AppWindow::handle_auto_logout_toggled ), false );
 
+    m_A_show_preferences = Lifeograph::p->add_action( "preferences",
+            sigc::ptr_fun( &DialogPreferences::create ) );
+
+    m_A_enable_editing = Lifeograph::p->add_action( "enable_editing",
+            [ this ]() { panel_diary->enable_editing(); } );
+    Lifeograph::p->set_accel_for_action( "app.enable_editing", "F2" );
+
+    m_IB_info->signal_response().connect(
+            sigc::mem_fun( this, &AppWindow::handle_infobar_response ) );
+
+    m_Bx_main->remove( *m_IB_info ); // necessary due to GNOME Bugzilla bug 710888
+
+    m_A_show_preferences->set_enabled( ! Lifeograph::p->m_flag_read_only );
+
     m_view_login->handle_start();
+
+    update_title();
 }
 
 AppWindow::~AppWindow()
@@ -114,7 +140,7 @@ AppWindow::on_delete_event( GdkEventAny* )
 {
     PRINT_DEBUG( "AppWindow::on_delete_event()" );
 
-    return( Lifeograph::quit() != true );
+    return( Lifeograph::p->quit() != true );
 }
 
 bool
@@ -131,16 +157,65 @@ AppWindow::write_backup()
 void
 AppWindow::show_about()
 {
-    if( m_about_dialog == nullptr )
+    static Gtk::AboutDialog* s_about_dialog{ nullptr };
+
+    if( s_about_dialog == nullptr )
     {
-        Lifeograph::get_builder2()->get_widget( "aboutdialog", m_about_dialog );
-        m_about_dialog->set_name( PROGRAM_NAME );
-        m_about_dialog->set_version( Lifeograph::PROGRAM_VERSION_STRING );
-        m_about_dialog->set_transient_for( *this );
+        Lifeograph::get_builder2()->get_widget( "aboutdialog", s_about_dialog );
+        s_about_dialog->set_name( PROGRAM_NAME );
+        s_about_dialog->set_version( Lifeograph::PROGRAM_VERSION_STRING );
+        s_about_dialog->set_transient_for( *AppWindow::p );
+#ifdef _WIN32
+        s_about_dialog->set_logo_default();
+#endif
     }
 
-    m_about_dialog->run();
-    m_about_dialog->hide();
+    s_about_dialog->run();
+    s_about_dialog->hide();
+}
+
+void
+AppWindow::show_info( Gtk::MessageType type, const Glib::ustring& text,
+                      const Glib::ustring &button_text, InfoResponse response )
+{
+    m_L_info->set_markup( text );
+    m_B_info->set_visible( !button_text.empty() );
+    m_B_info->set_label( button_text );
+    m_resp_cur = response;
+    m_IB_info->set_message_type( type );
+    if( m_IB_info->get_parent() == nullptr )
+        m_Bx_main->pack_start( *m_IB_info, Gtk::PACK_SHRINK ); // necessary due to GNOME Bugzilla bug 710888
+    m_IB_info->show();
+    m_flag_info_is_visible = true;
+}
+
+void
+AppWindow::handle_infobar_response( int response )
+{
+    if( response != Gtk::RESPONSE_CLOSE )
+    {
+        switch( m_resp_cur )
+        {
+            case RESP_UNDO_REMOVE_DIARY:
+                m_view_login->undo_remove_selected_diary();
+                break;
+            default:
+                break;
+        }
+    }
+
+    hide_infobar();
+}
+
+inline void
+AppWindow::hide_infobar()
+{
+    if( m_flag_info_is_visible )
+    {
+        m_IB_info->hide();
+        m_Bx_main->remove( *m_IB_info ); // necessary due to GNOME Bugzilla bug 710888
+        m_flag_info_is_visible = false;
+    }
 }
 
 void
@@ -162,13 +237,33 @@ AppWindow::handle_login()
     m_auto_logout_status = ( Lifeograph::settings.autologout && Diary::d->is_encrypted() ) ? 0 : 1;
 
     Lifeograph::m_actions_read->set_sensitive( true );
-    Lifeograph::m_actions_edit->set_visible( Diary::d->is_read_only() == false );
-    Lifeograph::m_actions_edit->set_sensitive( Diary::d->is_read_only() == false );
+    Lifeograph::m_actions_edit->set_visible( false );
+    Lifeograph::m_actions_edit->set_sensitive( false );
 
     m_St_main->set_visible_child( *m_paned_main );
 
     update_toggle_auto_logout_gui( true );
-    update_title();
+}
+
+void
+AppWindow::handle_edit_enabled()
+{
+    Lifeograph::m_actions_edit->set_visible( true );
+    Lifeograph::m_actions_edit->set_sensitive( true );
+
+    m_connection_backup = Glib::signal_timeout().connect_seconds(
+            sigc::mem_fun( this, &AppWindow::write_backup ), BACKUP_INTERVAL );
+
+    update_toggle_auto_logout_gui( true );  // must come after m_flag_editing_enabled is set
+}
+
+void
+AppWindow::handle_logout()
+{
+    if( Lifeograph::loginstatus == Lifeograph::LOGGED_TIME_OUT )
+        show_info( Gtk::MESSAGE_INFO, _( STRING::ENTER_PASSWORD_TIMEOUT ) );
+    else if( m_flag_info_is_visible )
+        hide_infobar();
 }
 
 // AUTO LOGOUT SYSTEM
@@ -271,7 +366,7 @@ AppWindow::update_toggle_auto_logout_gui( bool flag_availabilities_too )
     {
         m_A_toggle_auto_logout->set_enabled( Diary::d->is_encrypted() &&
                                              Lifeograph::settings.autologout );
-        m_A_logout_wo_saving->set_enabled( ! Diary::d->is_read_only() );
+        m_A_logout_wo_saving->set_enabled( Diary::d->is_editing_enabled() );
     }
 
     m_A_toggle_auto_logout->change_state( ! is_auto_logout_in_use() );
@@ -287,11 +382,9 @@ AppWindow::finish_editing( bool opt_save )
 
     // files added to recent list here if not already there
     if( ! Diary::d->get_path().empty() )
-        if( Lifeograph::stock_diaries.find( Diary::d->get_path() ) ==
-                Lifeograph::stock_diaries.end() )
-            Lifeograph::settings.recentfiles.insert( Diary::d->get_path() );
+        Lifeograph::settings.recentfiles.insert( Diary::d->get_path() );
 
-    if( ! Diary::d->is_read_only() )
+    if( Diary::d->is_editing_enabled() )
     {
         m_entry_view->sync();
         Diary::d->set_last_elem( panel_main->get_cur_elem() );
@@ -329,16 +422,16 @@ AppWindow::finish_editing( bool opt_save )
                 "<b>" + Diary::d->get_path() + ".~unsaved~</b>" ), true );
             messagedialog.add_button( _( "_Log out without Saving" ), Gtk::RESPONSE_ACCEPT );
 
-            if( messagedialog.run() !=  Gtk::RESPONSE_ACCEPT )
+            if( messagedialog.run() != Gtk::RESPONSE_ACCEPT )
                 return false;
             // back up changes
             Diary::d->write( Diary::d->get_path() + ".~unsaved~" );
         }
+
+        m_connection_backup.disconnect();
     }
 
     m_connection_timeout.disconnect();
-    if( ! Diary::d->is_read_only() )
-        m_connection_backup.disconnect();
 
     // CLEARING
     // TODO: m_loginstatus = LOGGING_OUT_IN_PROGRESS;
@@ -355,6 +448,7 @@ AppWindow::finish_editing( bool opt_save )
 
     if( Lifeograph::loginstatus == Lifeograph::LOGGED_IN )
         Lifeograph::loginstatus = Lifeograph::LOGGED_OUT;
+    Diary::d->remove_lock_if_necessary();
     Diary::d->clear();
     Lifeograph::s_internaloperation--;
 
@@ -369,11 +463,14 @@ AppWindow::logout( bool opt_save )
     Lifeograph::p->m_flag_open_directly = false;   // should be reset to prevent logging in again
     if( finish_editing( opt_save ) )
     {
+        this->handle_logout();
         m_view_login->handle_logout();
         m_St_main->set_visible_child( "login" );
     }
 
     m_auto_logout_status = 1;
+
+    update_title();
 }
 
 bool
@@ -403,15 +500,14 @@ AppWindow::confirm_dismiss_element( const DiaryElement* elem, Gtk::Widget* extra
 void
 AppWindow::update_title()
 {
-    Glib::ustring title( PROGRAM_NAME );
+    set_title( Lifeograph::loginstatus == Lifeograph::LOGGED_IN ?
+            Diary::d->get_name() : PROGRAM_NAME );
 
-    if( Lifeograph::loginstatus == Lifeograph::LOGGED_IN )
-    {
-        title += " - ";
-        title += Diary::d->get_name();
-    }
-
-    set_title( title );
+    static Gtk::Label label;
+    if( Lifeograph::loginstatus == Lifeograph::LOGGED_IN ) // just hide the title
+        m_Hb_main->set_custom_title( label );
+    else // remove custom widget to show the title
+        m_Hb_main->property_custom_title() = nullptr;
 }
 
 void
@@ -429,58 +525,36 @@ AppWindow::login()
             Lifeograph::get_builder()->get_widget_derived( "TV_all_tags", panel_extra );
 
             // ICONS
-            std::string icon_dir( Lifeograph::ICONDIR );
-            if( ! Lifeograph::settings.icon_theme.empty() )
-            {
-                icon_dir += "/";
-                icon_dir += Lifeograph::settings.icon_theme;
-            }
-
             Glib::RefPtr< Gtk::IconTheme > theme = Gtk::IconTheme::get_default();
-            theme->append_search_path( icon_dir );
+            theme->append_search_path( Lifeograph::get_icon_dir() );
 
-            Lifeograph::icons->diary_16 =
-                    Gdk::Pixbuf::create_from_file( icon_dir + "/diary-16.png" );
-            Lifeograph::icons->diary_32 =
-                    Gdk::Pixbuf::create_from_file( icon_dir + "/diary-32.png" );
-            Lifeograph::icons->entry_16 =
-                    Gdk::Pixbuf::create_from_file( icon_dir + "/entry-16.png" );
-            Lifeograph::icons->entry_32 =
-                    Gdk::Pixbuf::create_from_file( icon_dir + "/entry-32.png" );
-            Lifeograph::icons->favorite_16 =
-                    Gdk::Pixbuf::create_from_file( icon_dir + "/favorite-16.png" );
+            Lifeograph::icons->entry_16 = Lifeograph::load_icon( "entry-16.png" );
+            Lifeograph::icons->entry_32 = Lifeograph::load_icon( "entry-32.png" );
+            Lifeograph::icons->favorite_16 = Lifeograph::load_icon( "favorite-16.png" );
 
-            Lifeograph::icons->tag_16 =
-                    Gdk::Pixbuf::create_from_file( icon_dir + "/tag-16.png" );
-            Lifeograph::icons->tag_32 =
-                    Gdk::Pixbuf::create_from_file( icon_dir + "/tag-32.png" );
-            Lifeograph::icons->tag_theme_16 =
-                    Gdk::Pixbuf::create_from_file( icon_dir + "/tag_theme-16.png" );
-            Lifeograph::icons->tag_theme_32 =
-                    Gdk::Pixbuf::create_from_file( icon_dir + "/tag_theme-32.png" );
-            Lifeograph::icons->untagged_16 =
-                    Gdk::Pixbuf::create_from_file( icon_dir + "/untagged-16.png" );
-            Lifeograph::icons->untagged_32 =
-                    Gdk::Pixbuf::create_from_file( icon_dir + "/untagged-32.png" );
-            Lifeograph::icons->tag_category_16 =
-                    Gdk::Pixbuf::create_from_file( icon_dir + "/tag_category-16.png" );
-            Lifeograph::icons->tag_category_32 =
-                    Gdk::Pixbuf::create_from_file( icon_dir + "/tag_category-32.png" );
+            Lifeograph::icons->tag_16 = Lifeograph::load_icon( "tag-16.png" );
+            Lifeograph::icons->tag_32 = Lifeograph::load_icon( "tag-32.png" );
+            Lifeograph::icons->tag_theme_16 = Lifeograph::load_icon( "tag_theme-16.png" );
+            Lifeograph::icons->tag_theme_32 = Lifeograph::load_icon( "tag_theme-32.png" );
 
-            Lifeograph::icons->chapter_free_16 =
-                    Gdk::Pixbuf::create_from_file( icon_dir + "/chapter_free-16.png" );
-            Lifeograph::icons->chapter_free_32 =
-                    Gdk::Pixbuf::create_from_file( icon_dir + "/chapter_free-32.png" );
+            Lifeograph::icons->tag_p_16 = Lifeograph::load_icon( "tag_p-16.png" );
+            Lifeograph::icons->tag_p_32 = Lifeograph::load_icon( "tag_p-32.png" );
+            Lifeograph::icons->tag_p_theme_16 = Lifeograph::load_icon( "tag_p_theme-16.png" );
+            Lifeograph::icons->tag_p_theme_32 = Lifeograph::load_icon( "tag_p_theme-32.png" );
 
-            Lifeograph::icons->chapter_temp_16 =
-                    Gdk::Pixbuf::create_from_file( icon_dir + "/chapter_temp-16.png" );
-            Lifeograph::icons->chapter_temp_32 =
-                    Gdk::Pixbuf::create_from_file( icon_dir + "/chapter_temp-32.png" );
+            Lifeograph::icons->untagged_16 = Lifeograph::load_icon( "untagged-16.png" );
+            Lifeograph::icons->untagged_32 = Lifeograph::load_icon( "untagged-32.png" );
+            Lifeograph::icons->tag_category_16 = Lifeograph::load_icon( "tag_category-16.png" );
+            Lifeograph::icons->tag_category_32 = Lifeograph::load_icon( "tag_category-32.png" );
 
-            Lifeograph::icons->chapter_ordr_16 =
-                    Gdk::Pixbuf::create_from_file( icon_dir + "/chapter_ordr-16.png" );
-            Lifeograph::icons->chapter_ordr_32 =
-                    Gdk::Pixbuf::create_from_file( icon_dir + "/chapter_ordr-32.png" );
+            Lifeograph::icons->chapter_free_16 = Lifeograph::load_icon( "chapter_free-16.png" );
+            Lifeograph::icons->chapter_free_32 = Lifeograph::load_icon( "chapter_free-32.png" );
+
+            Lifeograph::icons->chapter_temp_16 = Lifeograph::load_icon( "chapter_temp-16.png" );
+            Lifeograph::icons->chapter_temp_32 = Lifeograph::load_icon( "chapter_temp-32.png" );
+
+            Lifeograph::icons->chapter_ordr_16 = Lifeograph::load_icon( "chapter_ordr-16.png" );
+            Lifeograph::icons->chapter_ordr_32 = Lifeograph::load_icon( "chapter_ordr-32.png" );
 
             Gtk::IconInfo ii = theme->lookup_icon( "filter-16-symbolic", 16,
                                                    Gtk::IconLookupFlags( 0 ) );
@@ -488,26 +562,22 @@ AppWindow::login()
             Lifeograph::icons->filter_16 = ii.load_symbolic( m_paned_main->get_style_context(),
                                                              res );
 
-            Lifeograph::icons->bullet_16 =
-                    Gdk::Pixbuf::create_from_file( icon_dir + "/bullet-16.png" );
-            Lifeograph::icons->todo_auto_32 =
-                    Gdk::Pixbuf::create_from_file( icon_dir + "/todo_auto-32.png" );
-            Lifeograph::icons->todo_open_16 =
-                    Gdk::Pixbuf::create_from_file( icon_dir + "/todo_open-16.png" );
-            Lifeograph::icons->todo_open_32 =
-                    Gdk::Pixbuf::create_from_file( icon_dir + "/todo_open-32.png" );
+            ii = theme->lookup_icon( "go-next-symbolic", 16, Gtk::IconLookupFlags( 0 ) );
+            Lifeograph::icons->next_16 = ii.load_symbolic( m_paned_main->get_style_context(),
+                                                                         res );
+
+            Lifeograph::icons->bullet_16 = Lifeograph::load_icon( "bullet-16.png" );
+            Lifeograph::icons->todo_auto_32 = Lifeograph::load_icon( "todo_auto-32.png" );
+            Lifeograph::icons->todo_open_16 = Lifeograph::load_icon( "todo_open-16.png" );
+            Lifeograph::icons->todo_open_32 = Lifeograph::load_icon( "todo_open-32.png" );
             Lifeograph::icons->todo_progressed_16 =
-                    Gdk::Pixbuf::create_from_file( icon_dir + "/todo_progressed-16.png" );
+                    Lifeograph::load_icon( "todo_progressed-16.png" );
             Lifeograph::icons->todo_progressed_32 =
-                    Gdk::Pixbuf::create_from_file( icon_dir + "/todo_progressed-32.png" );
-            Lifeograph::icons->todo_done_16 =
-                    Gdk::Pixbuf::create_from_file( icon_dir + "/todo_done-16.png" );
-            Lifeograph::icons->todo_done_32 =
-                    Gdk::Pixbuf::create_from_file( icon_dir + "/todo_done-32.png" );
-            Lifeograph::icons->todo_canceled_16 =
-                    Gdk::Pixbuf::create_from_file( icon_dir + "/todo_canceled-16.png" );
-            Lifeograph::icons->todo_canceled_32 =
-                    Gdk::Pixbuf::create_from_file( icon_dir + "/todo_canceled-32.png" );
+                    Lifeograph::load_icon( "todo_progressed-32.png" );
+            Lifeograph::icons->todo_done_16 = Lifeograph::load_icon( "todo_done-16.png" );
+            Lifeograph::icons->todo_done_32 = Lifeograph::load_icon( "todo_done-32.png" );
+            Lifeograph::icons->todo_canceled_16 = Lifeograph::load_icon( "todo_canceled-16.png" );
+            Lifeograph::icons->todo_canceled_32 = Lifeograph::load_icon( "todo_canceled-32.png" );
 
             // PANELS & VIEWS
             m_filter_view = new FilterView; // this is needed by PanelDiary
@@ -555,10 +625,6 @@ AppWindow::login()
                        sigc::mem_fun( *this, &AppWindow::handle_redo ) );
     }
 
-    if( ! Diary::d->is_read_only() )
-        m_connection_backup = Glib::signal_timeout().connect_seconds(
-                sigc::mem_fun( this, &AppWindow::write_backup ), BACKUP_INTERVAL );
-
     // LOGIN HANDLERS
     Lifeograph::s_internaloperation++;
 
@@ -576,4 +642,6 @@ AppWindow::login()
     Lifeograph::s_internaloperation--;
 
     Lifeograph::loginstatus = Lifeograph::LOGGED_IN;
+
+    update_title();
 }
