@@ -45,6 +45,8 @@ namespace LIFEO
 class AppWindow : public Gtk::ApplicationWindow
 {
     public:
+        enum InfoResponse { RESP_OK = 10, RESP_UNDO_REMOVE_DIARY = 20 };
+
                                     AppWindow( BaseObjectType*,
                                                const Glib::RefPtr<Gtk::Builder>& );
                                     ~AppWindow();
@@ -60,13 +62,22 @@ class AppWindow : public Gtk::ApplicationWindow
         bool                        finish_editing( bool = true );
         void                        logout( bool );
 
-        void                        show_about();
+        static void                 show_about();
+
+        void                        show_info( Gtk::MessageType, const Glib::ustring&,
+                                               const Glib::ustring& = "",
+                                               InfoResponse = RESP_OK );
+        void                        handle_infobar_response( int );
+        void                        hide_infobar();
+
 
         bool                        write_backup();
 
         void                        handle_undo();
         void                        handle_redo();
         void                        handle_login();
+        void                        handle_edit_enabled();
+        void                        handle_logout();
 
         // AUTO LOGOUT FUNCTIONALITY
         bool                        handle_idle();
@@ -101,8 +112,11 @@ class AppWindow : public Gtk::ApplicationWindow
         bool                        on_delete_event( GdkEventAny* );
 
         // WIDGETS
+        Gtk::Box*                   m_Bx_main;
         Gtk::Stack*                 m_St_main{ nullptr };
-        Gtk::AboutDialog*           m_about_dialog{ nullptr };
+        Gtk::InfoBar*               m_IB_info;
+        Gtk::Label*                 m_L_info;
+        Gtk::Button*                m_B_info;
 
         Gtk::Paned*                 m_paned_entry{ nullptr };
         Gtk::Paned*                 m_paned_main{ nullptr };
@@ -112,6 +126,8 @@ class AppWindow : public Gtk::ApplicationWindow
         int                         m_seconds_remaining{ LOGOUT_COUNTDOWN + 1 };
         sigc::connection            m_connection_timeout;
         int                         m_auto_logout_status{ 1 }; // reflects the result of all factors
+        InfoResponse                m_resp_cur; // FIXME: a most unfortunate solution!!
+        bool                        m_flag_info_is_visible{ false };
 
         // ACTIONS
         Glib::RefPtr< Gtk::Action > m_action_undo;
@@ -119,8 +135,10 @@ class AppWindow : public Gtk::ApplicationWindow
 
         Glib::RefPtr< Gio::SimpleAction > m_A_toggle_auto_logout;
         Glib::RefPtr< Gio::SimpleAction > m_A_logout_wo_saving;
+        Glib::RefPtr< Gio::SimpleAction > m_A_show_preferences;
+        Glib::RefPtr< Gio::SimpleAction > m_A_enable_editing;
 };
 
 } // end of namespace LIFEO
 
-#endif
\ No newline at end of file
+#endif
