@@ -1,6 +1,6 @@
 /***********************************************************************************
 
-    Copyright (C) 2007-2018 Ahmet Öztürk (aoz_2@yahoo.com)
+    Copyright (C) 2007-2019 Ahmet Öztürk (aoz_2@yahoo.com)
 
     This file is part of Lifeograph.
 
@@ -19,11 +19,8 @@
 
 ***********************************************************************************/
 
-#include <gtkmm/table.h>
-
 #include "lifeograph.hpp"
 #include "dialog_export.hpp"
-#include "dialog_password.hpp"
 #include "app_window.hpp"
 
 using namespace LIFEO;
@@ -145,8 +142,7 @@ DialogOpenDiary::DialogOpenDiary()
 {
     // Gtk gets confused if cancel is not stock:
     add_button( Gtk::Stock::CANCEL, Gtk::RESPONSE_CANCEL );
-    add_button( _( "Edit" ), RESPONSE_EDIT );
-    add_button( _( "Read" ), RESPONSE_READ );
+    add_button( Gtk::Stock::OPEN, Gtk::RESPONSE_OK );
 
     init_filters();
     add_filter( filter_any );
@@ -158,7 +154,7 @@ DialogOpenDiary::DialogOpenDiary()
 
     set_transient_for( * AppWindow::p );
 
-    set_default_response( RESPONSE_EDIT );
+    set_default_response( Gtk::RESPONSE_OK );
 }
 
 void
@@ -189,198 +185,7 @@ DialogOpenDiary::handle_selection_changed()
     if( flag_enable )
         flag_enable = ( Glib::file_test( get_filename(), Glib::FILE_TEST_IS_DIR ) == false );
 
-    set_response_sensitive( RESPONSE_READ, flag_enable );
-    set_response_sensitive( RESPONSE_EDIT, flag_enable );
-}
-
-// DIALOG IMPORT ===================================================================================
-DialogImport::DialogImport( BaseObjectType* cobject,
-                            const Glib::RefPtr< Gtk::Builder >& refbuilder )
-:   DialogEvent( cobject, refbuilder ), m_dialog_password( nullptr ), m_diary( nullptr )
-{
-    refbuilder->get_widget( "B_import_import", m_B_import );
-    refbuilder->get_widget( "B_import_cancel", m_B_cancel );
-    refbuilder->get_widget( "FB_import_diary", m_file_chooser_button );
-    refbuilder->get_widget( "L_import_info", m_label_info );
-    refbuilder->get_widget( "L_import_entries", m_label_entries );
-    refbuilder->get_widget( "L_import_tags", m_label_tags );
-    refbuilder->get_widget( "L_import_chapters", m_label_chapters );
-    refbuilder->get_widget_derived( "E_import_add_tag", m_tag_widget );
-
-    refbuilder->get_widget( "CB_import_entries", m_checkbutton_import_entries );
-    refbuilder->get_widget( "CB_import_tags", m_checkbutton_import_tags );
-    refbuilder->get_widget( "CB_import_chapters", m_checkbutton_import_chapters );
-
-    // FILTERING
-    DialogOpenDiary::init_filters();
-    m_file_chooser_button->add_filter( DialogOpenDiary::filter_any );
-    m_file_chooser_button->add_filter( DialogOpenDiary::filter_diary );
-    m_file_chooser_button->set_filter( DialogOpenDiary::filter_diary );
-
-    // SIGNALS
-    m_file_chooser_button->signal_file_set().connect(
-            sigc::mem_fun( this, &DialogImport::handle_diary_changed ) );
-
-    m_checkbutton_import_entries->signal_toggled().connect(
-            sigc::mem_fun( this, &DialogImport::handle_parts_changed ) );
-    m_checkbutton_import_tags->signal_toggled().connect(
-            sigc::mem_fun( this, &DialogImport::handle_parts_changed ) );
-    m_checkbutton_import_chapters->signal_toggled().connect(
-            sigc::mem_fun( this, &DialogImport::handle_parts_changed ) );
-
-    m_B_import->signal_clicked().connect(
-            sigc::bind( sigc::mem_fun( this, &DialogEvent::response ), RESPONSE_GO ));
-    m_B_cancel->signal_clicked().connect(
-            sigc::bind( sigc::mem_fun( this, &DialogEvent::response ), RESPONSE_CANCEL ));
-}
-
-inline bool
-DialogImport::is_options_ready()
-{
-    return( m_option_import_entries || m_option_import_tags || m_option_import_chapters );
-}
-
-void
-DialogImport::on_show()
-{
-    Gtk::Dialog::on_show();
-    set_not_ready( _( "Select a diary file for importing" ) );
-
-    m_file_chooser_button->unselect_all();
-    m_tag_widget->clear();
-
-    m_checkbutton_import_entries->set_active( true );
-    m_checkbutton_import_tags->set_active( true );
-    m_checkbutton_import_chapters->set_active( true );
-
-    m_tag_widget->populate();
-
-    m_file_chooser_button->grab_focus();
-}
-
-void
-DialogImport::handle_diary_changed()
-{
-    if( m_diary != nullptr )
-        delete m_diary;
-
-    m_diary = new Diary;
-
-    if( m_diary->set_path( m_file_chooser_button->get_filename(), Diary::SPT_READ_ONLY )
-            == SUCCESS )
-    {
-        if( m_diary->read_header() == SUCCESS )
-        {
-            if( m_diary->is_encrypted() )
-            {
-                DialogPassword::launch( DialogPassword::OT_OPEN, m_diary );
-                // TODO: do something if result != RESPONSE_GO
-            }
-
-            Result result = m_diary->read_body();
-            if( result == SUCCESS )
-            {
-                set_ready();
-            }
-            else if( result == WRONG_PASSWORD )
-            {
-                set_not_ready( _( "Entered password is wrong; please try again" ) );
-            }
-            else
-            {
-                set_not_ready( _( "Selected file cannot be read" ) );
-            }
-        }
-        else
-        {
-            set_not_ready( _( "Selected file does not seem to be a valid diary" ) );
-        }
-    }
-    else
-        set_not_ready( _( "Selected file cannot be read" ) );
-}
-
-void
-DialogImport::handle_parts_changed()
-{
-    m_option_import_entries = m_checkbutton_import_entries->get_active();
-    m_option_import_tags = m_checkbutton_import_tags->get_active();
-    m_option_import_chapters = m_checkbutton_import_chapters->get_active();
-
-    m_tag_widget->set_sensitive( m_option_import_entries );
-    m_B_import->set_sensitive( m_flag_diary_is_ready && is_options_ready() );
-}
-
-void
-DialogImport::set_ready()
-{
-    m_B_import->set_sensitive( is_options_ready() );
-
-    int n_chapters = 0;
-    for( auto& kv_ctg : m_diary->m_chapter_categories )
-    {
-        n_chapters += kv_ctg.second->size();
-    }
-
-    m_label_entries->set_text(
-            Glib::ustring::compose( "%1", m_diary->get_size() ) );
-    m_label_tags->set_text(
-            Glib::ustring::compose( "%1", m_diary->get_tags()->size() ) );
-    m_label_chapters->set_text(
-            Glib::ustring::compose( "%1/%2", n_chapters, m_diary->m_chapter_categories.size() ) );
-
-    m_label_info->set_text( _( "Diary is ready for importing" ) );
-    m_flag_diary_is_ready = true;
-}
-
-void
-DialogImport::set_not_ready( const Glib::ustring& message )
-{
-    m_B_import->set_sensitive( false );
-
-    m_label_entries->set_text( "-" );
-    m_label_tags->set_text( "-" );
-    m_label_chapters->set_text( "-" );
-
-    if( m_diary )
-    {
-        delete m_diary;
-        m_diary = nullptr;
-    }
-
-    m_label_info->set_text( message );
-    m_flag_diary_is_ready = false;
-}
-
-void
-DialogImport::on_response( int response )
-{
-    if( response == RESPONSE_GO )
-    {
-        if( m_option_import_tags )  // tags need to come before entries
-        {
-            for( auto& kv_tag : m_diary->m_tags )
-            {
-                Diary::d->import_tag( kv_tag.second );
-            }
-        }
-
-        if( m_option_import_entries )
-        {
-            Diary::d->import_entries( *m_diary, m_option_import_tags,
-                                      m_tag_widget->get_nav() );
-        }
-
-        if( m_option_import_chapters )
-        {
-            Diary::d->import_chapters( *m_diary );
-        }
-    }
-
-    delete m_diary;
-    m_diary = nullptr;
-
-    Gtk::Dialog::on_response( response );
+    set_response_sensitive( Gtk::RESPONSE_OK, flag_enable );
 }
 
 // DIALOG EXPORT ===================================================================================
