@@ -1,6 +1,6 @@
 /***********************************************************************************
 
-    Copyright (C) 2007-2018 Ahmet Öztürk (aoz_2@yahoo.com)
+    Copyright (C) 2007-2019 Ahmet Öztürk (aoz_2@yahoo.com)
 
     This file is part of Lifeograph.
 
@@ -29,8 +29,6 @@
 #include <gtkmm.h>
 
 #include "diary.hpp"
-#include "dialog_password.hpp"
-#include "widget_entrytags.hpp"
 
 
 namespace LIFEO
@@ -72,9 +70,6 @@ class DialogSaveDiary : public DialogSaveGeneric
 class DialogOpenDiary : public Gtk::FileChooserDialog
 {
     public:
-        static const int            RESPONSE_READ = 1000;
-        static const int            RESPONSE_EDIT = 2000;
-
                                     DialogOpenDiary();
 
         // FILTERING
@@ -91,50 +86,6 @@ class DialogOpenDiary : public Gtk::FileChooserDialog
         static bool                 flag_filters_created;
 };
 
-// IMPORT DIALOG ===================================================================================
-class DialogImport : public DialogEvent
-{
-	public:
-        DialogImport( BaseObjectType*,
-                      const Glib::RefPtr< Gtk::Builder >& );
-
-        void                        on_show();
-        void                        on_response( int );
-
-        void                        handle_diary_changed();
-        void                        handle_parts_changed();
-
-        void                        set_ready();
-        void                        set_not_ready( const Glib::ustring& );
-
-        bool                        is_options_ready();
-
-    private:
-        Gtk::Button*                m_B_import;
-        Gtk::Button*                m_B_cancel;
-        Gtk::FileChooserButton*     m_file_chooser_button;
-        Gtk::Label*                 m_label_info;
-        Gtk::Label*                 m_label_entries;
-        Gtk::Label*                 m_label_tags;
-        Gtk::Label*                 m_label_chapters;
-        WidgetTagEdit*              m_tag_widget;
-        Gtk::Entry*                 m_entry_password;
-
-        Gtk::CheckButton*           m_checkbutton_import_entries;
-        Gtk::CheckButton*           m_checkbutton_import_tags;
-        Gtk::CheckButton*           m_checkbutton_import_chapters;
-
-        DialogPassword*             m_dialog_password;
-
-        Diary*                      m_diary;
-
-        bool                        m_option_import_entries;
-        bool                        m_option_import_tags;
-        bool                        m_option_import_chapters;
-        bool                        m_flag_diary_is_ready;
-
-};
-
 // EXPORTERS =======================================================================================
 class Diarysaver
 {
