@@ -1,6 +1,6 @@
 /***********************************************************************************
 
-    Copyright (C) 2007-2018 Ahmet Öztürk (aoz_2@yahoo.com)
+    Copyright (C) 2007-2019 Ahmet Öztürk (aoz_2@yahoo.com)
 
     This file is part of Lifeograph.
 
@@ -33,27 +33,20 @@
 #include <cerrno>
 #include <cassert>
 #include <cfloat>
+#include <random>
 
 #include "diary.hpp"
 #include "helpers.hpp"
 #include "lifeograph.hpp"
 
-#ifdef LIFEO_WINDOZE
-#include <unistd.h>
-#include "strings.hpp"
-#endif
 
 using namespace LIFEO;
 
 
 // STATIC MEMBERS
 Diary*                  Diary::d;
-#if LIFEOGRAPH_DEBUG_BUILD
-bool                    Diary::s_flag_ignore_locks( true );
-#else
-bool                    Diary::s_flag_ignore_locks( false );
-#endif
-ElementView< Diary >* Diary::shower( nullptr );
+bool                    Diary::s_flag_ignore_locks{ false };
+ElementView< Diary >*   Diary::shower{ nullptr };
 
 // PARSING HELPERS
 Date
@@ -96,7 +89,7 @@ Diary::Diary()
 
 Diary::~Diary()
 {
-    remove_lock();
+    remove_lock_if_necessary();
 
     delete m_topics;
     delete m_groups;
@@ -119,7 +112,7 @@ Diary::init_new( const std::string& path )
 
     add_today(); // must come after m_ptr2chapter_ctg_cur is set
 
-    return LIFEO::SUCCESS;
+    return write();
 }
 
 void
@@ -127,12 +120,10 @@ Diary::clear()
 {
     close_file();
 
-    if( remove_lock() )
-        m_path.clear();
+    m_path.clear();
 
-    m_read_version = 0;
+    m_read_version = DB_FILE_VERSION_INT;
 
-    m_current_id = DEID_FIRST;
     m_force_id = DEID_UNSET;
     m_ids.clear();
     m_ids[ DEID_DIARY ] = this; // add diary back to IDs pool
@@ -162,9 +153,11 @@ Diary::clear()
     m_language.clear();
     m_option_sorting_criteria = SC_DATE;
     m_option_sorting_dir = SD_DESCENDING;
+
+    m_flag_read_only = false;
+    m_flag_editing_enabled = false;
 }
 
-#ifndef LIFEO_WINDOZE
 const Icon&
 Diary::get_icon() const
 {
@@ -175,28 +168,19 @@ Diary::get_icon32() const
 {
     return Lifeograph::icons->diary_32;
 }
-#else
-int
-Diary::get_icon() const
-{
-    return ICON_NO_DIARY;
-}
-#endif
 
 LIFEO::Result
 Diary::set_path( const std::string& path0, SetPathType type )
 {
     std::string path( path0 );
 
-#ifndef LIFEO_WINDOZE
     // RESOLVE SYMBOLIC LINK
-    if( g_file_test( path0.c_str(), G_FILE_TEST_IS_SYMLINK ) )
+    if( g_file_test( PATH( path0 ).c_str(), G_FILE_TEST_IS_SYMLINK ) )
     {
         GError* err = nullptr;
-        path = g_file_read_link( path0.c_str(), &err );
+        path = g_file_read_link( PATH( path0 ).c_str(), &err );
         print_info( "Symbolic link resolved to path: ", path );
     }
-#endif
 
     // CHECK FILE SYSTEM PERMISSIONS
     if( access( PATH( path ).c_str(), F_OK ) != 0 ) // check existence
@@ -217,47 +201,20 @@ Diary::set_path( const std::string& path0, SetPathType type )
         PRINT_DEBUG( "File is not readable" );
         return LIFEO::FILE_NOT_READABLE;
     }
-    else if( type != SPT_READ_ONLY && access( PATH( path ).c_str(), W_OK ) != 0 ) // check write access
+    else if( type == SPT_NEW && access( PATH( path ).c_str(), W_OK ) != 0 ) // check write access
     {
-        if( type == SPT_NEW )
-        {
-            PRINT_DEBUG( "File is not writable" );
-            return LIFEO::FILE_NOT_WRITABLE;
-        }
-        // errno == EACCES or errno == EROFS but no need to bother
-        PRINT_DEBUG( "File is not writable, opening read-only..." );
-        type = SPT_READ_ONLY;
+        PRINT_DEBUG( "File is not writable" );
+        return LIFEO::FILE_NOT_WRITABLE;
     }
 
-    // CHECK AND "TOUCH" THE NEW LOCK
-    if( type != SPT_READ_ONLY )
-    {
-        std::string path_lock( path + LOCK_SUFFIX );
-        if( access( PATH( path_lock ).c_str(), F_OK ) == 0 )
-        {
-            if( s_flag_ignore_locks )
-                print_info( "Ignored file lock" );
-            else
-                return LIFEO::FILE_LOCKED;
-        }
-
-        if( type == SPT_NORMAL )
-        {
-            FILE* fp = fopen( PATH( path_lock ).c_str(), "a+" );
-            if( fp )
-                fclose( fp );
-            else
-                print_error( "Could not create lock file" );
-        }
-    }
-
-    // REMOVE PREVIOUS LOCK IF ANY
-    remove_lock();
+    // REMOVE PREVIOUS PATH'S LOCK IF ANY
+    remove_lock_if_necessary();
 
     // ACCEPT PATH
     m_path = path;
     m_name = get_filename_base( path );
     m_flag_read_only = ( type == SPT_READ_ONLY );
+    m_flag_editing_enabled = false;
 
     return LIFEO::SUCCESS;
 }
@@ -268,6 +225,46 @@ Diary::get_path() const
     return m_path;
 }
 
+LIFEO::Result
+Diary::enable_editing()
+{
+    if( m_flag_read_only )
+    {
+        PRINT_DEBUG( "Diary: editing cannot be enabled. Diary is read-only" );
+        return LIFEO::FILE_LOCKED;
+    }
+
+    if( access( PATH( m_path ).c_str(), W_OK ) != 0 ) // check write access
+    {
+        PRINT_DEBUG( "File is not writable" );
+        return LIFEO::FILE_NOT_WRITABLE;
+        // errno == EACCES or errno == EROFS but no need to bother
+    }
+
+    // CHECK AND "TOUCH" THE NEW LOCK
+    std::string path_lock{ m_path + LOCK_SUFFIX };
+    if( access( PATH( path_lock ).c_str(), F_OK ) == 0 )
+    {
+        if( s_flag_ignore_locks )
+            print_info( "Ignored file lock" );
+        else
+            return LIFEO::FILE_LOCKED;
+    }
+
+    FILE* fp = fopen( PATH( path_lock ).c_str(), "a+" );
+    if( fp )
+        fclose( fp );
+    else
+    {
+        print_error( "Could not create lock file" );
+        return LIFEO::FILE_LOCKED;
+    }
+
+    m_flag_editing_enabled = true;
+
+    return LIFEO::SUCCESS;
+}
+
 bool
 Diary::is_path_set() const
 {
@@ -495,7 +492,7 @@ Diary::read_encrypted()
     return parse_db_body_text( stream );
 }
 
- // PARSING HELPERS
+// PARSING HELPERS
 inline void
 parse_todo_status( DiaryElement* elem, char c )
 {
@@ -569,7 +566,6 @@ parse_theme( Theme* ptr2theme, const std::string& line )
 {
     switch( line[ 1 ] )
     {
-#ifndef LIFEO_WINDOZE
         case 'f':   // font
             ptr2theme->font = Pango::FontDescription( line.substr( 2 ) );
             break;
@@ -588,26 +584,6 @@ parse_theme( Theme* ptr2theme, const std::string& line )
         case 'l':   // highlight color
             ptr2theme->color_highlight.set( line.substr( 2 ) );
             break;
-#else
-        case 'f':   // font
-            ptr2theme->font = line.substr( 2 );
-            break;
-        case 'b':   // base color
-            ptr2theme->color_base = line.substr( 2 );
-            break;
-        case 't':   // text color
-            ptr2theme->color_text = line.substr( 2 );
-            break;
-        case 'h':   // heading color
-            ptr2theme->color_heading = line.substr( 2 );
-            break;
-        case 's':   // subheading color
-            ptr2theme->color_subheading = line.substr( 2 );
-            break;
-        case 'l':   // highlight color
-            ptr2theme->color_highlight = line.substr( 2 );
-            break;
-#endif
     }
 }
 
@@ -1413,7 +1389,7 @@ Diary::upgrade_entries_to_1050()
         Entry* entry{ ( *iter ).second };
 
         if( entry->get_todo_status() == ES::NOT_TODO )
-            entry->set_todo_status( entry->calculate_todo_status_internal() );
+            entry->set_todo_status( Entry::calculate_todo_status( entry->get_text() ) );
     }
 }
 
@@ -1452,21 +1428,12 @@ create_db_tag_text( const Tag* tag, std::stringstream& output )
     {
         Theme* theme( tag->get_theme() );
 
-#ifndef LIFEO_WINDOZE
         output << type << 'f' << theme->font.to_string() << '\n';
         output << type << 'b' << convert_gdkrgba_to_string( theme->color_base ) << '\n';
         output << type << 't' << convert_gdkrgba_to_string( theme->color_text ) << '\n';
         output << type << 'h' << convert_gdkrgba_to_string( theme->color_heading ) << '\n';
         output << type << 's' << convert_gdkrgba_to_string( theme->color_subheading ) << '\n';
         output << type << 'l' << convert_gdkrgba_to_string( theme->color_highlight ) << '\n';
-#else
-        output << type << 'f' << theme->font << '\n';
-        output << type << 'b' << theme->color_base << '\n';
-        output << type << 't' << theme->color_text << '\n';
-        output << type << 'h' << theme->color_heading << '\n';
-        output << type << 's' << theme->color_subheading << '\n';
-        output << type << 'l' << theme->color_highlight << '\n';
-#endif
     }
 }
 
@@ -1525,11 +1492,7 @@ create_db_chapterctg_text( char type, const CategoryChapters* ctg, std::stringst
 
         if( chapter->get_color() != Color( "White" ) )
             output << "Cc" <<
-#ifndef LIFEO_WINDOZE
             convert_gdkrgba_to_string( chapter->get_color() )
-#else
-            chapter->get_color()
-#endif
             << '\n';
     }
 }
@@ -1551,7 +1514,7 @@ Diary::create_db_body_text( std::stringstream& output )
     // ROOT TAGS
     for( auto& kv_tag : m_tags )
     {
-        if( kv_tag.second->get_category() == nullptr )
+        if( kv_tag.second->get_ctg() == nullptr )
             create_db_tag_text( kv_tag.second, output );
     }
     // CATEGORIZED TAGS
@@ -1660,6 +1623,7 @@ Diary::create_db_body_text( std::stringstream& output )
             // for now we convert entry text to std::string here:
             std::string             content( entry->m_text );
             std::string::size_type  pt_start( 0 ), pt_end( 0 );
+            bool flag_terminate_loop{ false };
 
             while( true )
             {
@@ -1667,17 +1631,19 @@ Diary::create_db_body_text( std::stringstream& output )
                 if( pt_end == std::string::npos )
                 {
                     pt_end = content.size();
-                    output << "P " << content.substr( pt_start, content.size() - pt_start )
-                           << "\n\n";
-                    break; // end of while( true )
+                    flag_terminate_loop = true;
                 }
                 else
-                {
                     pt_end++;
-                    output << "P " << content.substr( pt_start, pt_end - pt_start );
-                    pt_start = pt_end;
-                }
+
+                output << "P " << content.substr( pt_start, pt_end - pt_start );
+
+                if( flag_terminate_loop )
+                    break; // end of while( true )
+
+                pt_start = pt_end;
             }
+            output << "\n\n";
         }
     }
 
@@ -1696,8 +1662,10 @@ Diary::write()
     // BACKUP THE PREVIOUS VERSION
     if( access( PATH( m_path ).c_str(), F_OK ) == 0 )
     {
-        std::string path_old( m_path + ".~previousversion~" );
-        rename( PATH( m_path ).c_str(), PATH( path_old ).c_str() );
+        auto path_prev{ PATH( m_path + ".~previousversion~" ) };
+        if( access( path_prev.c_str(), F_OK ) == 0 )
+            remove( path_prev.c_str() );
+        rename( PATH( m_path ).c_str(), path_prev.c_str() );
     }
 
     // WRITE THE FILE
@@ -1924,6 +1892,40 @@ Diary::write_encrypted( const std::string& path )
     return LIFEO::SUCCESS;
 }
 
+DEID
+Diary::create_new_id( DiaryElement* element )
+{
+    DEID retval;
+    if( m_force_id == DEID_UNSET )
+    {
+        do
+        {
+#ifndef _WIN32
+            std::random_device rd;  //seed
+            std::mt19937 gen( rd() ); // generator
+#else
+            std::mt19937 gen( time( NULL) ); // generator
+#endif
+            std::uniform_int_distribution<> dis0(0, 9);
+            std::uniform_int_distribution<> dis1(1, 9);
+
+            retval = ( dis1( gen ) * pow( 10, 6 ) );
+
+            for( int i = 0; i < 6; i++ )
+                retval += ( dis0( gen ) * pow( 10, i ) );
+        }
+        while( m_ids.find( retval ) != m_ids.end() );
+    }
+    else
+    {
+        retval = m_force_id;
+        m_force_id = DEID_UNSET;
+    }
+    m_ids[ retval ] = element;
+
+    return retval;
+}
+
 DiaryElement*
 Diary::get_element( DEID id ) const
 {
@@ -2007,10 +2009,10 @@ Diary::set_last_elem( const DiaryElement* elem )
 }
 
 // LOCKS ===========================================================================================
-inline bool
-Diary::remove_lock()
+bool
+Diary::remove_lock_if_necessary()
 {
-    if( m_path.empty() )
+    if( not( m_flag_editing_enabled ) || m_flag_read_only || m_path.empty() )
         return false;
 
     std::string path_lock( m_path + LOCK_SUFFIX );
@@ -2027,9 +2029,9 @@ Diary::add_today()
 }
 
 Entry*
-Diary::get_entry( const Date::date_t date, bool filtered_too )
+Diary::get_entry( const Date::date_t date, bool filtered_too ) const
 {
-    EntryIter iter( m_entries.find( date ) );
+    EntryIterConst iter( m_entries.find( date ) );
     if( iter != m_entries.end() )
         if( filtered_too || iter->second->get_filtered_out() == false )
             return iter->second;
@@ -2292,11 +2294,8 @@ Diary::replace_text( const Ustring& newtext )
         entry = kv_entry.second;
         if( entry->get_filtered_out() )
             continue;
-#ifndef LIFEO_WINDOZE
+
         Ustring entrytext = entry->get_text().lowercase();
-#else
-        Ustring entrytext = entry->get_text();
-#endif
         iter_str = 0;
         int count = 0;
         while( ( iter_str = entrytext.find( m_search_text, iter_str ) ) != std::string::npos )
@@ -2348,8 +2347,8 @@ Diary::dismiss_tag( Tag* tag, bool flag_dismiss_associated )
     }
 
     // remove from category if any
-    if( tag->get_category() != nullptr )
-        tag->get_category()->erase( tag );
+    if( tag->get_ctg() != nullptr )
+        tag->get_ctg()->erase( tag );
 
     // clear filters if necessary
     if( tag == m_filter_active->get_tag() )
@@ -2361,6 +2360,14 @@ Diary::dismiss_tag( Tag* tag, bool flag_dismiss_associated )
     delete tag;
 }
 
+void
+Diary::rename_tag( Tag* tag, const Ustring& new_name )
+{
+    m_tags.erase( tag->get_name() );
+    tag->set_name( create_unique_name_for_map( m_tags, new_name ) );
+    m_tags[ tag->get_name() ] = tag;
+}
+
 CategoryTags*
 Diary::create_tag_ctg()
 {
@@ -2399,7 +2406,7 @@ Diary::dismiss_tag_ctg( CategoryTags* ctg, bool flag_dismiss_contained )
     else
     {
         for( Tag* tag : *ctg )
-            tag->set_category( nullptr );
+            tag->set_ctg( nullptr );
     }
 
     // remove from the list and delete
@@ -2407,6 +2414,14 @@ Diary::dismiss_tag_ctg( CategoryTags* ctg, bool flag_dismiss_contained )
     delete ctg;
 }
 
+void
+Diary::rename_tag_ctg( CategoryTags* ctg, const Ustring& new_name )
+{
+    m_tag_categories.erase( ctg->get_name() );
+    ctg->set_name( create_unique_name_for_map( m_tag_categories, new_name ) );
+    m_tag_categories[ ctg->get_name() ] = ctg;
+}
+
 // CHAPTERS ========================================================================================
 CategoryChapters*
 Diary::create_chapter_ctg()
@@ -2448,18 +2463,12 @@ Diary::dismiss_chapter_ctg( CategoryChapters* category )
     delete category;
 }
 
-bool
-Diary::rename_chapter_ctg( CategoryChapters* category, const Ustring& new_name )
+void
+Diary::rename_chapter_ctg( CategoryChapters* ctg, const Ustring& new_name )
 {
-    if( m_chapter_categories.count( new_name ) > 0 )
-        return false;
-
-    m_chapter_categories.erase( category->get_name() );
-    category->set_name( new_name );
-    m_chapter_categories.insert(
-            PoolCategoriesChapters::value_type( new_name, category ) );
-
-    return true;
+    m_chapter_categories.erase( ctg->get_name() );
+    ctg->set_name( create_unique_name_for_map( m_chapter_categories, new_name ) );
+    m_chapter_categories[ ctg->get_name() ] = ctg;
 }
 
 void
@@ -2637,14 +2646,14 @@ Diary::add_entry_to_related_chapter( Entry* entry )
     // NOTE: works as per the current listing options needs to be updated when something
     // changes the arrangement such as a change in the current chapter category
 
-    CategoryChapters* ptr2ctg;
+    CategoryChapters* cc;
 
     if( entry->get_date().is_ordinal() ) // in groups or topics
-        ptr2ctg = ( entry->get_date().is_hidden() ? m_groups : m_topics );
+        cc = ( entry->get_date().is_hidden() ? m_groups : m_topics );
     else // in chapters
-        ptr2ctg = m_ptr2chapter_ctg_cur;
+        cc = m_ptr2chapter_ctg_cur;
 
-    for( auto& kv_chapter : *ptr2ctg )
+    for( auto& kv_chapter : *cc )
     {
         Chapter* chapter( kv_chapter.second );
 
@@ -2812,100 +2821,218 @@ Diary::show()
 }
 
 // IMPORTING =======================================================================================
-bool
-Diary::import_tag( Tag* tag )
+void
+Diary::import_tag_ctg( const CategoryTags* ctg_r, bool flag_add )
 {
-    Tag* new_tag( create_tag( tag->get_name(), nullptr ) );
-    if( new_tag == nullptr )
+    if( flag_add && !get_element( ctg_r->get_id() ) )
+        m_force_id = ctg_r->get_id();
+
+    CategoryTags* ctg_l{ flag_add ?
+            create_tag_ctg( create_unique_name_for_map( m_tag_categories, ctg_r->get_name() ) ) :
+            dynamic_cast< CategoryTags* >( get_element( ctg_r->get_id() ) ) };
+
+    if( ctg_l == nullptr )
+        return;
+
+    if( !flag_add )
+        rename_tag_ctg( ctg_l, ctg_r->get_name() );
+}
+
+bool
+Diary::import_tag( const Tag* tag_r, bool flag_add )
+{
+    DiaryElement* elem_ctg{ nullptr };
+    CategoryTags* ctg{ tag_r->get_ctg() };
+    if( ctg )
+        elem_ctg = get_element( ctg->get_id() );
+
+    if( elem_ctg && elem_ctg->get_type() == ET_TAG_CTG )
+        ctg = dynamic_cast< CategoryTags* >( elem_ctg );
+    else
+        ctg = nullptr;
+
+    if( flag_add && !get_element( tag_r->get_id() ) ) // if id is available
+        m_force_id = tag_r->get_id();
+
+    Tag* tag_l{ flag_add ?
+            create_tag( create_unique_name_for_map( m_tags, tag_r->get_name() ), ctg ) :
+            dynamic_cast< Tag* >( get_element( tag_r->get_id() ) ) };
+
+    if( tag_l == nullptr )
         return false;
 
-    if( tag->get_has_own_theme() )
-        new_tag->create_own_theme_duplicating( tag->get_theme() );
+    if( !flag_add )
+    {
+        rename_tag( tag_l, tag_r->get_name() );
+        tag_l->set_ctg( ctg );
+    }
+    tag_l->set_chart_type( tag_r->get_chart_type() );
+    tag_l->set_unit( tag_r->get_unit() );
+
+    if( tag_r->get_has_own_theme() )
+        tag_l->create_own_theme_duplicating( tag_r->get_theme() );
 
     return true;
 }
 
 bool
-Diary::import_entries( const Diary& diary,
-                       bool flag_import_tags,
-                       const NameAndValue& nav )
+Diary::import_entry( const Entry* entry_r, Tag* tag_all, const Value value, bool flag_add )
 {
-    Entry* entry_new;
-    Entry* entry_ext;
-    Tag* tag_all = nullptr;
+    if( flag_add && !get_element( entry_r->get_id() ) ) // if id is available
+        m_force_id = entry_r->get_id();
 
-    if( ! nav.name.empty() )
-        tag_all = ( nav.value == 1 ? create_tag( nav.name, nullptr ) :
-                create_tag( nav.name, nullptr, ChartPoints::MONTHLY|ChartPoints::CUMULATIVE ) );
+    Entry* entry_l{ flag_add ?
+            create_entry( entry_r->m_date.m_date, entry_r->m_text, false ) :
+            dynamic_cast< Entry* >( get_element( entry_r->get_id() ) )  };
 
-    for( EntryIterConstRev iter = diary.m_entries.rbegin();
-         iter != diary.m_entries.rend();
-         ++iter )
+    if( entry_l == nullptr )
+        return false;
+
+    if( !flag_add )
     {
-        entry_ext = iter->second;
-        entry_new = new Entry( this,
-                               entry_ext->m_date.m_date,
-                               entry_ext->m_text,
-                               entry_ext->is_favored() );
-
-        // fix order:
-        entry_new->m_date.reset_order_1();
-        while( m_entries.find( entry_new->m_date.m_date ) != m_entries.end() )
-            entry_new->m_date.m_date++;
-
-        // copy other data:
-        entry_new->m_date_created = entry_ext->m_date_created;
-        entry_new->m_date_edited = entry_ext->m_date_edited;
-        entry_new->m_date_status = entry_ext->m_date_status;
-
-        // insert it into the diary:
-        m_entries[ entry_new->m_date.m_date ] = entry_new;
-        add_entry_to_related_chapter( entry_new );
-
-        if( flag_import_tags )
-        {
-            Tag* tag;
-
-            for( Tag* tag_ext : entry_ext->m_tags )
-            {
-                tag = ( m_tags.find( tag_ext->get_name() ) )->second;
-                entry_new->add_tag( tag );
-            }
-
-            // preserve the theme:
-            if( entry_ext->get_theme_is_set() )
-            {
-                tag = ( m_tags.find( entry_ext->get_theme_tag()->get_name() ) )->second;
-                entry_new->set_theme_tag( tag );
-            }
-        }
-        if( tag_all )
-            entry_new->add_tag( tag_all, nav.value );
-
-        // add to untagged if that is the case:
-        if( entry_new->get_tags().empty() )
-            m_untagged.add_entry( entry_new );
+        entry_l->m_text = entry_r->m_text;
+        set_entry_date( entry_l, entry_r->get_date() );
+        entry_l->clear_tags();
     }
 
+    // copy over the tags:
+    for( Tag* tag_r : entry_r->m_tags )
+    {
+        DiaryElement* elem_tag{ get_element( tag_r->get_id() ) };
+        if( elem_tag && elem_tag->get_type() == ET_TAG )
+        {
+            Tag* tag{ dynamic_cast< Tag* >( elem_tag ) };
+            entry_l->add_tag( tag, tag_r->get_value( entry_r ) );
+        }
+    }
+
+    // preserve the theme:
+    if( entry_r->get_theme_is_set() )
+    {
+        DiaryElement* elem_tag{ get_element( entry_r->get_theme_tag()->get_id() ) };
+        if( elem_tag && elem_tag->get_type() == ET_TAG )
+            entry_l->set_theme_tag( dynamic_cast< Tag* >( elem_tag ) );
+    }
+
+    if( tag_all )
+        entry_l->add_tag( tag_all, value );
+
+    // add to untagged if that is the case:
+    if( entry_l->get_tags().empty() )
+        m_untagged.add_entry( entry_l );
+
+    // copy other data:
+    entry_l->m_date_created = entry_r->m_date_created;
+    entry_l->m_date_edited = entry_r->m_date_edited;
+    entry_l->m_date_status = entry_r->m_date_status;
+    entry_l->m_option_lang = entry_r->m_option_lang;
+    entry_l->m_location = entry_r->m_location;
+    entry_l->m_status = entry_r->m_status;
+
     return true;
 }
 
+void
+Diary::import_chapter_ctg( const CategoryChapters* cc_r, bool flag_add )
+{
+    if( flag_add && !get_element( cc_r->get_id() ) )
+        m_force_id = cc_r->get_id();
+
+    CategoryChapters* cc_l{ flag_add ?
+            create_chapter_ctg(
+                    create_unique_name_for_map( m_chapter_categories, cc_r->get_name() ) ) :
+            dynamic_cast< CategoryChapters* >( get_element( cc_r->get_id() ) ) };
+
+    if( cc_l == nullptr )
+        return;
+
+    if( !flag_add )
+        rename_chapter_ctg( cc_l, cc_r->get_name() );
+}
+
 bool
-Diary::import_chapters( const Diary& diary )
+Diary::import_chapter( const Chapter* chapter_r, bool flag_add )
 {
-    for( auto& kv_cc : diary.m_chapter_categories )
-    {
-        CategoryChapters* cc( kv_cc.second );
-        CategoryChapters* cc_new = create_chapter_ctg(
-                create_unique_name_for_map( m_chapter_categories, cc->get_name() ) );
+    CategoryChapters* ctg{ nullptr };
+    Date::date_t date{ chapter_r->get_date().m_date };
 
-        for( auto& kv_chapter : *cc )
+    switch( chapter_r->get_type() )
+    {
+        case ET_CHAPTER:
         {
-            Chapter* chapter( kv_chapter.second );
-            cc_new->create_chapter( chapter->get_name(), chapter->get_date().m_date );
+            DiaryElement* elem_ctg{ nullptr };
+
+            if( chapter_r->get_ctg() )
+                elem_ctg = get_element( chapter_r->get_ctg()->get_id() );
+
+            if( elem_ctg && elem_ctg->get_type() == ET_CHAPTER_CTG )
+                ctg = dynamic_cast< CategoryChapters* >( elem_ctg );
+            else
+                ctg = m_ptr2chapter_ctg_cur;
+
+            if( flag_add && ctg->get_chapter( date ) != nullptr )
+                return false;
         }
+        break;
+        case ET_TOPIC:
+            ctg = m_topics;
+            date = ctg->get_free_order_ordinal();
+        break;
+        case ET_GROUP:
+            ctg = m_groups;
+            date = ctg->get_free_order_ordinal();
+        break;
+        default:
+            return false;
     }
 
+    if( flag_add && !get_element( chapter_r->get_id() ) )
+        m_force_id = chapter_r->get_id();
+
+    Chapter* chapter_l{ flag_add ?
+            ctg->create_chapter( chapter_r->get_name(), date ) :
+            dynamic_cast< Chapter* >( get_element( chapter_r->get_id() ) ) };
+
+    if( chapter_l == nullptr )
+        return false;
+
+    if( !flag_add )
+    {
+        chapter_l->set_name( chapter_r->get_name() );
+        if( chapter_r->get_type() == ET_CHAPTER )
+            ctg->set_chapter_date( chapter_l, date );
+        else
+            set_topic_order( chapter_l, chapter_r->get_date().m_date );
+    }
+    chapter_l->set_chart_type( chapter_r->get_chart_type() );
+    chapter_l->set_color( chapter_r->get_color() );
+    chapter_l->set_status( chapter_r->get_status() );
+
     return true;
 }
 
+DiaryElement*
+Diary::get_corresponding_elem( const DiaryElement* elem_r ) const
+{
+    DiaryElement* elem_l{ get_element( elem_r->get_id() ) };
+    if( elem_l && elem_r->get_type() == elem_l->get_type() )
+        return elem_l;
+    else
+        return nullptr;
+}
+
+SI
+Diary::compare_foreign_elem( const DiaryElement* elem_r,
+                             const DiaryElement*& elem_l ) const
+{
+    elem_l = get_corresponding_elem( elem_r );
+
+    if( elem_l == nullptr )
+        return SI::NEW;
+    else if( elem_r->get_as_string() == elem_l->get_as_string() )
+        return SI::INTACT;
+
+    //else
+    return SI::CHANGED;
+}
+
