@@ -1,6 +1,6 @@
 /***********************************************************************************
 
-    Copyright (C) 2007-2018 Ahmet Öztürk (aoz_2@yahoo.com)
+    Copyright (C) 2007-2019 Ahmet Öztürk (aoz_2@yahoo.com)
 
     This file is part of Lifeograph.
 
@@ -32,12 +32,8 @@ using namespace LIFEO;
 
 // DIARYELEMENT ====================================================================================
 // STATIC MEMBERS
-#ifndef LIFEO_WINDOZE
 ListData::Colrec*                   ListData::colrec;
 const Icon                          DiaryElement::s_pixbuf_null;
-#else
-const Icon                          DiaryElement::s_pixbuf_null = nullptr;
-#endif
 bool                                DiaryElement::FLAG_ALLOCATE_GUI_FOR_DIARY = true;
 const Ustring                       DiaryElement::s_type_names[] =
 {
@@ -57,19 +53,15 @@ DiaryElement::DiaryElement( Diary* const ptr2diary,
     m_id( ptr2diary ? ptr2diary->create_new_id( this ) :  DEID_UNSET ),
     m_status( status )
 {
-#ifndef LIFEO_WINDOZE
     if( FLAG_ALLOCATE_GUI_FOR_DIARY )
         m_list_data = new ListData;
-#endif
 }
 
 DiaryElement::DiaryElement( Diary* const ptr2diary, DEID id, ElemStatus status )
 :   NamedElement( "" ), m_ptr2diary( ptr2diary ), m_id( id ), m_status( status )
 {
-#ifndef LIFEO_WINDOZE
     if( FLAG_ALLOCATE_GUI_FOR_DIARY )
         m_list_data = new ListData;
-#endif
 }
 
 DiaryElement::~DiaryElement()
@@ -85,6 +77,30 @@ DiaryElement::set_todo_status( ElemStatus s )
     m_status |= s;
 }
 
+SI
+DiaryElement::get_todo_status_si() const
+{
+    switch( get_todo_status() )
+    {
+        case ES::TODO:
+            return SI::TODO;
+        case ES::PROGRESSED:
+            return SI::PROGRESSED;
+        case ES::DONE:
+            return SI::DONE;
+        case ES::CANCELED:
+            return SI::CANCELED;
+        case ( ES::NOT_TODO | ES::TODO ):
+        case ( ES::NOT_TODO | ES::PROGRESSED ):
+        case ( ES::NOT_TODO | ES::DONE ):
+        case ( ES::NOT_TODO | ES::CANCELED ):
+            return SI::AUTO;
+        case ES::NOT_TODO:
+        default:    // should never be the case
+            return SI::_NONE_;
+    }
+}
+
 // CHART POINTS ====================================================================================
 unsigned int
 ChartPoints::calculate_distance( const Date& d1, const Date& d2 ) const
@@ -335,24 +351,69 @@ Tag::~Tag()
 {
 }
 
-#ifndef LIFEO_WINDOZE
+const SKVVec
+Tag::get_as_string() const
+{
+    SKVVec sv;
+    sv.push_back( { SI::TYPE_NAME, get_type_name() } );
+    sv.push_back( { SI::NAME, m_name } );
+    sv.push_back( { SI::ID, STR::compose( m_id ) } );
+    sv.push_back( { SI::CATEGORY, m_ptr2category ? m_ptr2category->get_name() : "" } );
+    sv.push_back( { SI::UNIT, m_unit } );
+    sv.push_back( { SI::CHART_INTERVALS, ( m_chart_type & ChartPoints::MONTHLY ) ?
+                                         STR0/SI::MONTHLY : STR0/SI::YEARLY } );
+    sv.push_back( { SI::CHART_UNDERLAY, ( m_chart_type & ChartPoints::UNDERLAY_PREV_YEAR ) ?
+                                        STR0/SI::YES : STR0/SI::NO } );
+    switch( m_chart_type & ChartPoints::VALUE_TYPE_MASK )
+    {
+        case ChartPoints::BOOLEAN:
+            sv.push_back( { SI::VALUE_TYPE, STR0/SI::BOOLEAN } );
+            break;
+        case ChartPoints::CUMULATIVE:
+            sv.push_back( { SI::VALUE_TYPE, STR0/SI::CUMULATIVE } );
+            break;
+        case ChartPoints::AVERAGE:
+            sv.push_back( { SI::VALUE_TYPE, STR0/SI::AVERAGE } );
+            break;
+    }
+
+    get_theme()->get_as_string( sv );
+
+    return sv;
+}
+
 const Icon&
 Tag::get_icon() const
 {
-    return( get_has_own_theme() ? Lifeograph::icons->tag_theme_16 : Lifeograph::icons->tag_16 );
+    switch( ( get_has_own_theme() ? 0x2 : 0x1 ) | ( is_boolean() ? 0x10 : 0x20 ) )
+    {
+        case 0x11:
+            return Lifeograph::icons->tag_16;
+        case 0x12:
+            return Lifeograph::icons->tag_theme_16;
+        case 0x21:
+            return Lifeograph::icons->tag_p_16;
+        //case 0x22:
+        default:
+            return Lifeograph::icons->tag_p_theme_16;
+    }
 }
 const Icon&
 Tag::get_icon32() const
 {
-    return( get_has_own_theme() ? Lifeograph::icons->tag_theme_32 : Lifeograph::icons->tag_32 );
+    switch( ( get_has_own_theme() ? 0x2 : 0x1 ) | ( is_boolean() ? 0x10 : 0x20 ) )
+    {
+        case 0x11:
+            return Lifeograph::icons->tag_32;
+        case 0x12:
+            return Lifeograph::icons->tag_theme_32;
+        case 0x21:
+            return Lifeograph::icons->tag_p_32;
+        //case 0x22:
+        default:
+            return Lifeograph::icons->tag_p_theme_32;
+    }
 }
-#else
-int
-Tag::get_icon() const
-{
-    return( get_has_own_theme() ? ICON_NO_TAG_THEME : ICON_NO_TAG );
-}
-#endif
 
 Ustring
 Tag::escape_name( const Ustring& name )
@@ -378,7 +439,7 @@ Tag::get_name_and_value( const Entry* entry, bool flag_escape, bool flag_unit )
 
     if( !is_boolean() )
     {
-        result += STR::compose( " = ", get_value( const_cast< Entry* >( entry ) ) );
+        result += STR::compose( " = ", get_value( entry ) );
 
         if( flag_unit && !m_unit.empty() )
             result += " " + m_unit;
@@ -397,7 +458,7 @@ Tag::show()
 }
 
 void
-Tag::set_category( CategoryTags* category_new )
+Tag::set_ctg( CategoryTags* category_new )
 {
     if( m_ptr2category )
         m_ptr2category->erase( this );
@@ -452,10 +513,16 @@ Tag::reset_theme()
     }
 }
 
-Value
-Tag::get_value( Entry* entry ) const
+bool
+Tag::add_entry( Entry* entry, Value value )
 {
-    auto iter = find( entry );
+    return( insert( value_type( entry, value ) ).second );
+}
+
+Value
+Tag::get_value( const Entry* entry ) const
+{
+    auto iter = find( const_cast< Entry* >( entry ) );
     if( iter != end() )
         return iter->second;
     else
@@ -547,14 +614,9 @@ Untagged::Untagged()
 Ustring
 Untagged::get_list_str() const
 {
-#ifndef LIFEO_WINDOZE
     return( "<i>" + Glib::Markup::escape_text( m_name ) + "</i>" );
-#else
-    return( m_name ); // FIXME
-#endif
 }
 
-#ifndef LIFEO_WINDOZE
 const Icon&
 Untagged::get_icon() const
 {
@@ -565,13 +627,6 @@ Untagged::get_icon32() const
 {
     return Lifeograph::icons->untagged_32;
 }
-#else
-int
-Untagged::get_icon() const
-{
-    return ICON_NO_UNTAGGED;
-}
-#endif
 
 // TAGPOOL =========================================================================================
 PoolTags::~PoolTags()
@@ -581,23 +636,15 @@ PoolTags::~PoolTags()
 }
 
 Tag*
-PoolTags::get_tag( const Ustring& name )
+PoolTags::get_tag( const Ustring& name ) const
 {
-    iterator iter = find( name );
+    auto iter = find( name );
     if( iter != end() )
         return( iter->second );
     else
         return nullptr;
 }
 
-bool
-PoolTags::rename( Tag* tag, const Ustring& new_name )
-{
-    erase( tag->m_name );
-    tag->m_name = new_name;
-    return( insert( value_type( new_name, tag ) ).second );
-}
-
 void
 PoolTags::clear()
 {
@@ -616,7 +663,17 @@ CategoryTags::CategoryTags( Diary* const d, const Ustring& name )
 {
 }
 
-#ifndef LIFEO_WINDOZE
+const SKVVec
+CategoryTags::get_as_string() const
+{
+    SKVVec sv;
+    sv.push_back( { SI::TYPE_NAME, get_type_name() } );
+    sv.push_back( { SI::NAME, m_name } );
+    sv.push_back( { SI::ID, STR::compose( m_id ) } );
+
+    return sv;
+}
+
 const Icon&
 CategoryTags::get_icon() const
 {
@@ -627,13 +684,7 @@ CategoryTags::get_icon32() const
 {
     return Lifeograph::icons->tag_category_32;
 }
-#else
-int
-CategoryTags::get_icon() const
-{
-    return ICON_NO_TAG_CTG;
-}
-#endif
+
 
 void
 CategoryTags::show()
@@ -664,12 +715,14 @@ PoolCategoriesTags::clear()
     std::map< Ustring, CategoryTags*, FuncCmpStrings >::clear();
 }
 
-bool
-PoolCategoriesTags::rename_category( CategoryTags* category, const Ustring& new_name )
+CategoryTags*
+PoolCategoriesTags::get_ctg( const Ustring& name ) const
 {
-    erase( category->m_name );
-    category->m_name = new_name;
-    return( insert( value_type( new_name, category ) ).second );
+    auto iter = find( name );
+    if( iter != end() )
+        return( iter->second );
+    else
+        return nullptr;
 }
 
 // TAGSET
@@ -702,7 +755,21 @@ Chapter::Chapter( Diary* const d, const Ustring& name, Date::date_t date )
     update_type();
 }
 
-#ifndef LIFEO_WINDOZE
+const SKVVec
+Chapter::get_as_string() const
+{
+    SKVVec sv;
+    sv.push_back( { SI::TYPE_NAME, get_type_name() } );
+    sv.push_back( { SI::NAME, m_name } );
+    sv.push_back( { SI::DATE, m_date_begin.format_string() } );
+    sv.push_back( { SI::ID, STR::compose( m_id ) } );
+    sv.push_back( { SI::CHART_INTERVALS, ( m_chart_type & ChartPoints::MONTHLY ) ?
+                                    STR0/SI::MONTHLY : STR0/SI::YEARLY } );
+    sv.push_back( { SI::TODO, STR0/get_todo_status_si() } );
+
+    return sv;
+}
+
 const Icon&
 Chapter::get_icon() const
 {
@@ -753,31 +820,9 @@ Chapter::get_date_icon( bool flag_big ) const
     return( icon[ ( m_date_begin.is_ordinal() + m_date_begin.is_hidden() ) * 2 + flag_big ] );
 }
 
-#else
-int
-Chapter::get_icon() const
-{
-    switch( m_status & ES::FILTER_TODO )
-    {
-        case ES::TODO:
-            return ICON_NO_TODO;
-        case ES::PROGRESSED:
-            return ICON_NO_PROGRESSED;
-        case ES::DONE:
-            return ICON_NO_DONE;
-        case ES::CANCELED:
-            return ICON_NO_CANCELED;
-        default:
-            return( m_date_begin.is_ordinal() ?
-                    ICON_NO_CHAPTER_O : ICON_NO_CHAPTER_D );
-    }
-}
-#endif
-
 Ustring
 Chapter::get_list_str() const
 {
-#ifndef LIFEO_WINDOZE
     static const Ustring tpl[] = { "<b>%2</b>", "<b>%1 -  %2</b>",
                                    "<b><s>%2</s></b>", "<b><s>%1 -  %2</s></b>" };
     int i{ m_date_begin.is_hidden() ? 0 : 1 };
@@ -786,11 +831,6 @@ Chapter::get_list_str() const
 
     return Glib::ustring::compose( tpl[ i ], m_date_begin.format_string(),
                                    Glib::Markup::escape_text( m_name ) );
-#else // FIXME
-    if( m_date_begin.is_hidden() )
-        return m_name;
-    return STR::compose( m_date_begin.format_string(), " -  ", m_name );
-#endif
 }
 
 void
@@ -907,13 +947,15 @@ bool
 CategoryChapters::set_chapter_date( Chapter* chapter, Date::date_t date )
 {
     assert( chapter->is_ordinal() == false );
-    assert( find( date ) == end() );
+
+    if( find( date ) != end() )
+        return false;
 
     if( chapter->m_date_begin.m_date != Date::NOT_SET )
     {
         iterator iter( find( chapter->m_date_begin.m_date ) );
         if( iter == end() )
-            return false; // chapter is not a member of the set
+            return false; // chapter is not a member of this category
 
         if( ( ++iter ) != end() ) // fix time span
         {
@@ -925,6 +967,7 @@ CategoryChapters::set_chapter_date( Chapter* chapter, Date::date_t date )
         }
 
         erase( chapter->m_date_begin.m_date );
+        chapter->set_ctg( nullptr );
     }
 
     chapter->set_date( date );
@@ -952,6 +995,8 @@ CategoryChapters::add( Chapter* chapter )
     if( ( ++iter ) != end() ) // if not earliest fix previous
         iter->second->recalculate_span( chapter );
 
+    chapter->set_ctg( this );
+
     return true; // reserved
 }
 
@@ -993,6 +1038,27 @@ PoolCategoriesChapters::clear()
     std::map< Ustring, CategoryChapters*, FuncCmpStrings >::clear();
 }
 
+Chapter*
+PoolCategoriesChapters::get_chapter( const Ustring& ch, const Date::date_t date ) const
+{
+    auto kv = find( ch );
+
+    if( kv == end() )
+        return nullptr;
+
+    return kv->second->get_chapter( date );
+}
+
+CategoryChapters*
+PoolCategoriesChapters::get_ctg( const Ustring& name ) const
+{
+    auto iter = find( name );
+    if( iter != end() )
+        return( iter->second );
+    else
+        return nullptr;
+}
+
 // THEMES ==========================================================================================
 // STATIC MEMBERS
 const Color Theme::s_color_match1( "#33FF33" );
@@ -1005,8 +1071,6 @@ const Color Theme::s_color_broken2( "#990000" );
 const Color Theme::s_color_todo( "#FF0000" );
 const Color Theme::s_color_progressed( "#FF8811" );
 const Color Theme::s_color_done( "#66BB00" );
-const Color Theme::s_color_done1( "#77CC11" );
-const Color Theme::s_color_done2( "#409000" );
 const Color Theme::s_color_canceled( "#AA8855" );
 
 Theme::Theme()
@@ -1053,29 +1117,19 @@ ThemeSystem::get()
 
     if( ! s_flag_initialized )
     {
-#ifndef LIFEO_WINDOZE
         // this may not be the best method to detect the default font:
         Gtk::TextView *tv = new Gtk::TextView;
         tv->show();
         s_theme = new ThemeSystem( tv->get_style_context()->get_font().to_string(),
                                    "white", "black",
-                                   "blue", "#F066FC",
-                                   "#FFF955" );
+                                   "#B72525", "#963F3F",
+                                   "#FFBBBB" );
         delete tv;
-#else
-        s_theme = new ThemeSystem( "sans 11",
-                                   "#FFFFFFFFFFFF", "#000000000000",
-                                   "#00000000FFFF", "#F0F06666FCFC",
-                                   "#FFFFF9F95555" );
 
-#endif
         s_flag_initialized = true;
-        return s_theme;
-    }
-    else
-    {
-        return s_theme;
     }
+
+    return s_theme;
 }
 
 // FILTERS =========================================================================================
