@@ -1,6 +1,6 @@
 /***********************************************************************************
 
-    Copyright (C) 2007-2018 Ahmet Öztürk (aoz_2@yahoo.com)
+    Copyright (C) 2007-2019 Ahmet Öztürk (aoz_2@yahoo.com)
 
     This file is part of Lifeograph.
 
@@ -24,11 +24,10 @@
 #define LIFEOGRAPH_DIARYDATA_HEADER
 
 
-#include "helpers.hpp"	// i18n headers
+#include "helpers.hpp"  // i18n headers
+#include "strings.hpp"
 
-#ifndef LIFEO_WINDOZE
 #include <gtkmm/treemodel.h>
-#endif
 #include <set>
 #include <map>
 #include <vector>
@@ -48,7 +47,6 @@ static const SortingCriteria SD_ASCENDING   = 'a';
 
 typedef unsigned long DEID; // unique diary element id
 static const DEID DEID_DIARY = 10000;       // reserved for Diary
-static const DEID DEID_FIRST = 10001;       // first sub item in the diary
 static const DEID DEID_UNSET = 404;         // :)
 static const DEID HOME_CURRENT_ELEM = 1;    // element shown at startup
 static const DEID HOME_LAST_ELEM = 2;       // element shown at startup
@@ -104,16 +102,15 @@ namespace ES
     // to go parallel with Java version 0x7FFFFFFF is the max
 }
 
-class DiaryElement;     // forward declaration
-class Untagged;         // forward declaration
-class Entry;            // forward declaration
-class Filter;           // forward declaration
-class Diary;            // forward declaration
-#ifndef LIFEO_WINDOZE
-class ListData;	        // forward declaration
-#else
-#include <commctrl.h>
-#endif
+// FORWARD DECLARATIONS
+class DiaryElement;
+class Tag;
+class Untagged;
+class Entry;
+class Filter;
+class Diary;
+class CategoryChapters;
+class ListData;
 
 typedef std::vector< Entry* >               EntryVector;
 typedef std::vector< Entry* >::iterator     EntryVectorIter;
@@ -165,39 +162,30 @@ class DiaryElement : public NamedElement
                                 DiaryElement( Diary* const, DEID, ElemStatus = ES::_VOID_ );
         virtual                 ~DiaryElement();
 
+        virtual const SKVVec    get_as_string() const
+        {
+            SKVVec skv;
+            skv.push_back( { SI::TYPE_NAME, "DiaryElement" } );
+            return skv;
+        }
+
         virtual Type            get_type() const = 0;
-        Ustring                 get_type_name()
+        const Ustring           get_type_name() const
         { return s_type_names[ get_type() ]; }
         virtual int             get_size() const = 0;
-#ifndef LIFEO_WINDOZE
+
         virtual const Icon&     get_icon() const
         { return( s_pixbuf_null ); }
         virtual const Icon&     get_icon32() const
         { return( s_pixbuf_null ); }
 
-#else
-#define ICON_NO_DIARY          0
-#define ICON_NO_ENTRY          1
-#define ICON_NO_TODO           2
-#define ICON_NO_PROGRESSED     3
-#define ICON_NO_DONE           4
-#define ICON_NO_CANCELED       5
-#define ICON_NO_CHAPTER_D      6
-#define ICON_NO_CHAPTER_O      7
-#define ICON_NO_TAG            8
-#define ICON_NO_TAG_THEME      9
-#define ICON_NO_UNTAGGED       10
-#define ICON_NO_TAG_CTG        11
-        virtual int             get_icon() const { return 0; }
-#endif
-
         virtual Date            get_date() const
         { return Date( Date::NOT_APPLICABLE ); }
         Date::date_t            get_date_t() const
         { return get_date().m_date; }
 
         virtual Ustring         get_list_str() const
-        { return ""; }
+        { return m_name; }
 
         DEID                    get_id() const
         { return m_id; }
@@ -205,7 +193,14 @@ class DiaryElement : public NamedElement
         virtual void            show() = 0;
         virtual void            prepare_for_hiding() {}
         virtual bool            get_filtered_out()
-        { return false; }
+        { return( m_status & ES::FILTERED_OUT ); }
+        void                    set_filtered_out( bool filteredout = true )
+        {
+            if( filteredout )
+                m_status |= ES::FILTERED_OUT;
+            else if( m_status & ES::FILTERED_OUT )
+                m_status -= ES::FILTERED_OUT;
+        }
 
         ElemStatus              get_status() const
         { return m_status; }
@@ -230,16 +225,13 @@ class DiaryElement : public NamedElement
             const ElemStatus s{ m_status & ES::FILTER_TODO_PURE };
             return( s ? s : ES::NOT_TODO );
         }
+        SI                      get_todo_status_si() const;
         void                    set_todo_status( ElemStatus );
 
         virtual bool            is_favored() const
         { return false; }
 
-#ifndef LIFEO_WINDOZE
         ListData*               m_list_data = nullptr;
-#else
-        HTREEITEM               m_list_data;
-#endif
 
     protected:
         Diary* const            m_ptr2diary = nullptr;
@@ -248,7 +240,6 @@ class DiaryElement : public NamedElement
         ElemStatus              m_status = ES::_VOID_;
 };
 
-#ifndef LIFEO_WINDOZE
 class ListData
 {
     public:
@@ -273,7 +264,6 @@ class ListData
 
         Gtk::TreePath           treepath;
 };
-#endif
 
 struct FuncCmpStrings
 {
@@ -310,11 +300,7 @@ create_unique_name_for_map( const std::map< Ustring, T*, FuncCmpStrings > &map,
     Ustring name = name0;
     for( int i = 1; map.find( name ) != map.end(); i++ )
     {
-#ifndef LIFEO_WINDOZE
         name = Glib::ustring::compose( "%1 %2", name0, i );
-#else
-        name = STR::compose( name0, " ", i );
-#endif
     }
 
     return name;
@@ -325,21 +311,17 @@ create_unique_name_for_map( const std::map< Ustring, T*, FuncCmpStrings > &map,
 class ElementViewProto
 {
     public:
-#ifndef LIFEO_WINDOZE
                                 ElementViewProto() {}
-#endif
         virtual                 ~ElementViewProto() {}
 
         virtual DiaryElement*   get_element() = 0;
 
-#ifndef LIFEO_WINDOZE
         Gtk::Widget*            get_widget()
         { return m_box; }
         virtual Gtk::Popover*   get_popover()
         { return m_Po_elem; }
         virtual Gtk::Widget*    get_extra_tools()
         { return nullptr; }
-#endif
 
         virtual Ustring         get_title_str() const
         { return ""; }
@@ -356,10 +338,8 @@ class ElementViewProto
         { return false; }
 
     protected:
-#ifndef LIFEO_WINDOZE
         Gtk::Box*               m_box{ nullptr };
         Gtk::Popover*           m_Po_elem{ nullptr };
-#endif
 };
 
 template< class T >
@@ -430,11 +410,23 @@ class Theme
 
         virtual bool                is_system() const
         { return false; }
-#ifndef LIFEO_WINDOZE
+
+        void                        get_as_string( SKVVec& spv ) const
+        {
+            spv.push_back( { SI::THEME_FONT, font.to_string() } );
+            spv.push_back( { SI::THEME_BASE,
+                             convert_gdkrgba_to_string( color_base ) } );
+            spv.push_back( { SI::THEME_TEXT,
+                             convert_gdkrgba_to_string( color_text ) } );
+            spv.push_back( { SI::THEME_HEADING,
+                             convert_gdkrgba_to_string( color_heading ) } );
+            spv.push_back( { SI::THEME_SUBHEADING,
+                             convert_gdkrgba_to_string( color_subheading ) } );
+            spv.push_back( { SI::THEME_HIGHLIGHT,
+                             convert_gdkrgba_to_string( color_highlight ) } );
+        }
+
         Pango::FontDescription      font;
-#else
-        std::string                 font;
-#endif
         Color                       color_base;
         Color                       color_text;
         Color                       color_heading;
@@ -452,8 +444,6 @@ class Theme
         static const Color          s_color_todo;
         static const Color          s_color_progressed;
         static const Color          s_color_done;
-        static const Color          s_color_done1;
-        static const Color          s_color_done2;
         static const Color          s_color_canceled;
 };
 
@@ -477,7 +467,7 @@ class ThemeSystem : public Theme
 class ChartPoints
 {
     public:
-        ChartPoints( int t = MONTHLY|CUMULATIVE ) : type( t ) { }
+        ChartPoints( int t = DEFAULT ) : type( t ) { }
 
         typedef std::pair< float, Color > PairChapter;
         static constexpr int        MONTHLY             = 0x1;
@@ -490,7 +480,9 @@ class ChartPoints
         static constexpr int        AVERAGE             = 0x30;
         static constexpr int        VALUE_TYPE_MASK     = 0xf0;
 
-        static constexpr int        UNDERLAY_PREV_YEAR = 0x100;
+        static constexpr int        UNDERLAY_PREV_YEAR  = 0x100;
+
+        static constexpr int        DEFAULT             = MONTHLY|CUMULATIVE;
 
         unsigned int                calculate_distance( const Date&, const Date& ) const;
         int                         calculate_distance_neg( const Date&, const Date& ) const;
@@ -552,8 +544,6 @@ class DiaryElementChart
 };
 
 // TAG =============================================================================================
-class Tag;	// forward declaration
-
 // TAG CATEGORY
 class CategoryTags : public DiaryElementReferrer< Tag, FuncCmpDiaryElemByName< Tag > >
 {
@@ -562,23 +552,18 @@ class CategoryTags : public DiaryElementReferrer< Tag, FuncCmpDiaryElemByName< T
 
                                 CategoryTags( Diary* const, const Ustring& );
 
+        const SKVVec            get_as_string() const override;
+
         void                    show() override;
 
         Type                    get_type() const
         { return ET_TAG_CTG; }
 
-#ifndef LIFEO_WINDOZE
         const Icon&             get_icon() const override;
         const Icon&             get_icon32() const override;
-#else
-        int                     get_icon() const;
-#endif
+
         Ustring                 get_list_str() const override
-#ifndef LIFEO_WINDOZE
         { return Glib::ustring::compose( "<b>%1</b>", Glib::Markup::escape_text( m_name ) ); }
-#else
-        { return m_name; }
-#endif
 
         bool                    get_expanded() const
         { return( m_status & ES::EXPANDED ); }
@@ -599,7 +584,7 @@ class PoolCategoriesTags : public std::map< Ustring, CategoryTags*, FuncCmpStrin
                                 PoolCategoriesTags();
                                 ~PoolCategoriesTags();
 
-        bool                    rename_category( CategoryTags*, const Ustring& );
+        CategoryTags*           get_ctg( const Ustring& ) const;
         void                    clear();
 };
 
@@ -634,17 +619,18 @@ class Tag : public DiaryElementMapper< Entry, Value >, public DiaryElementChart
                                      int = ChartPoints::MONTHLY|ChartPoints::BOOLEAN );
         virtual                 ~Tag();
 
+        const SKVVec            get_as_string() const override;
+
         void                    show() override;
 
         static Ustring          escape_name( const Ustring& );
         Ustring                 get_name_and_value( const Entry*, bool, bool ) const;
 
-        CategoryTags*           get_category()
+        CategoryTags*           get_ctg() const
         { return m_ptr2category; }
-        void                    set_category( CategoryTags* );
+        void                    set_ctg( CategoryTags* );
 
-        bool                    add_entry( Entry* entry, Value value = 1 )
-        { return( insert( value_type( entry, value ) ).second ); }
+        bool                    add_entry( Entry*, Value = 1 );
 
         Theme*                  get_theme() const;
         bool                    get_has_own_theme() const
@@ -657,21 +643,13 @@ class Tag : public DiaryElementMapper< Entry, Value >, public DiaryElementChart
         virtual Type            get_type() const override
         { return ET_TAG; }
 
-#ifndef LIFEO_WINDOZE
         virtual const Icon&     get_icon() const override;
         virtual const Icon&     get_icon32() const override;
-#else
-        virtual int             get_icon() const;
-#endif
 
         virtual Ustring         get_list_str() const override
-#ifndef LIFEO_WINDOZE
         { return Glib::Markup::escape_text( m_name ); }
-#else
-        { return m_name; }
-#endif
 
-        Value                   get_value( Entry* ) const;
+        Value                   get_value( const Entry* ) const;
         Value                   get_combined_value() const;
         Ustring                 get_combined_value_str() const
         { return( STR::compose( get_combined_value(), " ", m_unit ) ); }
@@ -700,12 +678,9 @@ class Untagged : public Tag
 
         Type                    get_type() const override
         { return ET_UNTAGGED; }
-#ifndef LIFEO_WINDOZE
+
         const Icon&             get_icon() const override;
         const Icon&             get_icon32() const override;
-#else
-        int                     get_icon() const;
-#endif
 
         Ustring                 get_list_str() const override;
 
@@ -724,8 +699,7 @@ class PoolTags : public std::map< Ustring, Tag*, FuncCmpStrings >
                                 ~PoolTags();
 
         bool                    handle_tag_changed( );
-        bool                    rename( Tag*, const Ustring& );
-        Tag*                    get_tag( const Ustring& );
+        Tag*                    get_tag( const Ustring& ) const;
         void                    clear();
 };
 
@@ -755,6 +729,8 @@ class Chapter : public DiaryElementReferrer< Entry >, public DiaryElementChart
         // if it does not, its derived classes cannot be deleted
         virtual                 ~Chapter() {}
 
+        const SKVVec            get_as_string() const override;
+
         void                    show() override;
 
         Ustring                 get_date_str() const;
@@ -765,18 +741,19 @@ class Chapter : public DiaryElementReferrer< Entry >, public DiaryElementChart
         Type                    get_type() const override
         { return m_type; }
 
-#ifndef LIFEO_WINDOZE
         const Icon&             get_icon() const;
         const Icon&             get_icon32() const;
         const Icon&             get_date_icon( bool ) const;
-#else
-        int                     get_icon() const;
-#endif
 
         Date                    get_date() const;
 
         Ustring                 get_list_str() const;
 
+        void                    set_ctg( CategoryChapters* ctg )
+        { m_ptr2ctg = ctg; }
+        CategoryChapters*       get_ctg() const
+        { return m_ptr2ctg; }
+
         Date                    get_free_order() const;
 
         bool                    get_expanded() const
@@ -809,6 +786,7 @@ class Chapter : public DiaryElementReferrer< Entry >, public DiaryElementChart
         int                     m_time_span{ 0 };
         Type                    m_type;
         Color                   m_color{ "White" };
+        CategoryChapters*       m_ptr2ctg{ nullptr };
 
     friend class CategoryChapters;
     friend class ChapterView;
@@ -853,6 +831,8 @@ class PoolCategoriesChapters :
                                 //PoolCategoriesChapters() {}
                                 ~PoolCategoriesChapters();
         void                    clear();
+        Chapter*                get_chapter( const Ustring&, const Date::date_t ) const;
+        CategoryChapters*       get_ctg( const Ustring& ) const;
 };
 
 // FILTERS =========================================================================================
@@ -890,11 +870,7 @@ class Filter : public DiaryElement
         { return 0; }   // redundant
 
         Ustring                     get_list_str() const override
-#ifndef LIFEO_WINDOZE
         { return Glib::Markup::escape_text( m_name ); }
-#else
-        { return m_name; }
-#endif
 
         void                        reset();
         void                        set( const Filter* );
