@@ -1,6 +1,6 @@
 /***********************************************************************************
 
-    Copyright (C) 2007-2018 Ahmet Öztürk (aoz_2@yahoo.com)
+    Copyright (C) 2007-2019 Ahmet Öztürk (aoz_2@yahoo.com)
 
     Parts of this file are loosely based on an example gcrypt program
     on http://punkroy.drque.net/
@@ -29,13 +29,6 @@
 
 #include <iomanip>
 
-#ifdef LIFEO_WINDOZE
-#include <sstream>
-#include <cmath>
-#include <unistd.h>
-#include <sys/stat.h>
-#endif
-
 #include "helpers.hpp"
 
 
@@ -73,7 +66,6 @@ Date::Date( const std::string& str_date )
 //    timeinfo->tm_sec = 0;
 //}
 
-#ifndef LIFEO_WINDOZE
 Glib::Date::Month
 Date::get_month_glib() const
 {
@@ -107,7 +99,6 @@ Date::get_month_glib() const
             return Glib::Date::BAD_MONTH;
     }
 }
-#endif
 
 Result
 Date::parse_string( Date::date_t* date, const Ustring& str_date )
@@ -283,13 +274,11 @@ Date::get_year_str() const
     return result.str();
 }
 
-#ifndef LIFEO_WINDOZE
 Ustring
 Date::get_month_str() const
 {
     return Glib::Date( 20, get_month_glib(), 2000 ).format_string( "%b" );
 }
-#endif
 
 int
 Date::get_weekday() const
@@ -393,14 +382,10 @@ Date::forward_day()
 unsigned int
 Date::calculate_days_between( const Date& date2 ) const
 {
-#ifndef LIFEO_WINDOZE
     // TODO: create own implementation
     Glib::Date gdate_begin( get_glib() );
     Glib::Date gdate_end( date2.get_glib() );
     return( gdate_begin.days_between( gdate_end ) );
-#else
-    return 0; // STUB
-#endif
 }
 
 unsigned int
@@ -428,17 +413,11 @@ Date::calculate_months_between_neg( Date::date_t date1, Date::date_t date2 )
 }
 
 // COLOR OPERATIONS ================================================================================
-#ifndef LIFEO_WINDOZE
-Gdk::RGBA
-contrast2( const Gdk::RGBA& bg, const Gdk::RGBA& c1, const Gdk::RGBA& c2 )
+Color
+contrast2( const Color& bg, const Color& c1, const Color& c2 )
 {
-    double dist1 = ( fabs( bg.get_red() - c1.get_red() ) +
-                     fabs( bg.get_green() - c1.get_green() ) +
-                     fabs( bg.get_blue() - c1.get_blue() ) );
-
-    double dist2 = ( fabs( bg.get_red() - c2.get_red() ) +
-                     fabs( bg.get_green() - c2.get_green() ) +
-                     fabs( bg.get_blue() - c2.get_blue() ) );
+    const float dist1{ get_color_diff( bg, c1 ) };
+    const float dist2{ get_color_diff( bg, c2 ) };
 
     if( dist1 > dist2 )
         return c1;
@@ -446,10 +425,10 @@ contrast2( const Gdk::RGBA& bg, const Gdk::RGBA& c1, const Gdk::RGBA& c2 )
         return c2;
 }
 
-Gdk::RGBA
-midtone( const Gdk::RGBA& c1, const Gdk::RGBA& c2 )
+Color
+midtone( const Color& c1, const Color& c2 )
 {
-    Gdk::RGBA midtone;
+    Color midtone;
     midtone.set_red_u( ( c1.get_red_u() + c2.get_red_u() ) / 2.0 );
     midtone.set_green_u( ( c1.get_green_u() + c2.get_green_u() ) / 2.0 );
     midtone.set_blue_u( ( c1.get_blue_u() + c2.get_blue_u() ) / 2.0 );
@@ -457,10 +436,10 @@ midtone( const Gdk::RGBA& c1, const Gdk::RGBA& c2 )
     return midtone;
 }
 
-Gdk::RGBA
-midtone( const Gdk::RGBA& c1, const Gdk::RGBA& c2, float ratio )
+Color
+midtone( const Color& c1, const Color& c2, float ratio )
 {
-    Gdk::RGBA midtone;
+    Color midtone;
     midtone.set_red_u( ( c1.get_red_u() * ratio ) + ( c2.get_red_u() * ( 1.0 - ratio ) ) );
     midtone.set_green_u( ( c1.get_green_u() * ratio ) + ( c2.get_green_u() * ( 1.0 - ratio ) ) );
     midtone.set_blue_u( ( c1.get_blue_u() * ratio ) + ( c2.get_blue_u() * ( 1.0 - ratio ) ) );
@@ -468,36 +447,6 @@ midtone( const Gdk::RGBA& c1, const Gdk::RGBA& c2, float ratio )
     return midtone;
 }
 
-#else
-
-COLORREF
-contrast2( COLORREF bg, COLORREF c1, COLORREF c2 )
-{
-    double dist1 = ( fabs( GetRValue( bg ) - GetRValue( c1 ) ) +
-                     fabs( GetGValue( bg ) - GetGValue( c1 ) ) +
-                     fabs( GetBValue( bg ) - GetBValue( c1 ) ) );
-
-    double dist2 = ( fabs( GetRValue( bg ) - GetRValue( c2 ) ) +
-                     fabs( GetGValue( bg ) - GetGValue( c2 ) ) +
-                     fabs( GetBValue( bg ) - GetBValue( c2 ) ) );
-
-    if( dist1 > dist2 )
-        return c1;
-    else
-        return c2;
-}
-
-COLORREF
-midtone( COLORREF c1, COLORREF c2, float ratio )
-{
-    return RGB(
-            ( int ) ( ( GetRValue( c1 ) * ratio ) + ( GetRValue( c2 ) * ( 1.0 - ratio ) ) ),
-            ( int ) ( ( GetGValue( c1 ) * ratio ) + ( GetGValue( c2 ) * ( 1.0 - ratio ) ) ),
-            ( int ) ( ( GetBValue( c1 ) * ratio ) + ( GetBValue( c2 ) * ( 1.0 - ratio ) ) ) );
-}
-
-#endif
-
 // FILE OPERATIONS =================================================================================
 #ifdef _WIN32
 std::string
@@ -525,13 +474,11 @@ get_file_size( std::ifstream& file )
 bool
 copy_file_suffix( const std::string& source_path, const std::string& suffix1, int suffix2 )
 {
-#ifndef LIFEO_WINDOZE
     try
     {
         Glib::RefPtr< Gio::File > file_src = Gio::File::create_for_path( source_path );
 
         if( file_src )
-#endif
         {
             std::stringstream ss;
             ss << source_path << suffix1;
@@ -549,20 +496,14 @@ copy_file_suffix( const std::string& source_path, const std::string& suffix1, in
                 ss << " (" << ++index << ")";
             }
 
-#ifndef LIFEO_WINDOZE
             Glib::RefPtr< Gio::File > file_dest = Gio::File::create_for_path( ss.str() );
             file_src->copy( file_dest );
-#else
-            CopyFileA( source_path.c_str(), ss.str().c_str(), true );
-#endif
         }
-#ifndef LIFEO_WINDOZE
     }
     catch( ... )
     {
         return false;
     }
-#endif
     return true;
 }
 
@@ -584,6 +525,36 @@ is_dir( const std::string& path )
         throw Error( "Stat failed" );
 }
 
+void
+get_all_files_in_path( const std::string& path, ListPaths& list )
+{
+    Gio::init();
+    try
+    {
+        Glib::RefPtr< Gio::File > directory = Gio::File::create_for_path( path );
+        if( directory )
+        {
+            Glib::RefPtr< Gio::FileEnumerator > enumerator = directory->enumerate_children();
+            if( enumerator )
+            {
+                Glib::RefPtr< Gio::FileInfo > file_info = enumerator->next_file();
+                while( file_info )
+                {
+                    if( file_info->get_file_type() != Gio::FILE_TYPE_DIRECTORY )
+                    {
+                        list.insert( Glib::build_filename( path, file_info->get_name() ) );
+                    }
+                    file_info = enumerator->next_file();
+                }
+            }
+        }
+    }
+    catch( const Glib::Exception& ex )
+    {
+        print_error( ex.what() );
+    }
+}
+
 #ifdef _WIN32
 std::string
 convert_filename_to_win32_locale( const std::string& fn )
@@ -668,7 +639,6 @@ str_ends_with ( const std::string& str, const std::string& end )
     }
 }
 
-#ifndef LIFEO_WINDOZE
 std::string
 get_env_lang()
 {
@@ -696,102 +666,6 @@ is_char_alpha( gunichar c )
     return Glib::Unicode::isalpha( c );
 }
 
-#else
-wchar_t
-char_lower( wchar_t c )
-{
-    return LOWORD( CharLower( ( LPWSTR ) MAKELONG( c, 0 ) ) );
-}
-
-bool
-is_char_alpha( wchar_t c )
-{
-    return IsCharAlpha( c );
-}
-
-// code taken from: http://alter.org.ua/docs/win/args/
-PCHAR*
-command_line_to_argvA( PCHAR CmdLine, int* _argc )
-{
-    PCHAR* argv;
-    PCHAR  _argv;
-    ULONG  len;
-    ULONG  argc;
-    CHAR   a;
-    ULONG  i, j;
-
-    BOOLEAN  in_QM;
-    BOOLEAN  in_TEXT;
-    BOOLEAN  in_SPACE;
-
-    len = strlen(CmdLine);
-    i = ((len+2)/2)*sizeof(PVOID) + sizeof(PVOID);
-
-    argv = (PCHAR*)GlobalAlloc(GMEM_FIXED,
-        i + (len+2)*sizeof(CHAR));
-
-    _argv = (PCHAR)(((PUCHAR)argv)+i);
-
-    argc = 0;
-    argv[argc] = _argv;
-    in_QM = FALSE;
-    in_TEXT = FALSE;
-    in_SPACE = TRUE;
-    i = 0;
-    j = 0;
-
-    while( ( a = CmdLine[i] ) ) {
-        if(in_QM) {
-            if(a == '\"') {
-                in_QM = FALSE;
-            } else {
-                _argv[j] = a;
-                j++;
-            }
-        } else {
-            switch(a) {
-            case '\"':
-                in_QM = TRUE;
-                in_TEXT = TRUE;
-                if(in_SPACE) {
-                    argv[argc] = _argv+j;
-                    argc++;
-                }
-                in_SPACE = FALSE;
-                break;
-            case ' ':
-            case '\t':
-            case '\n':
-            case '\r':
-                if(in_TEXT) {
-                    _argv[j] = '\0';
-                    j++;
-                }
-                in_TEXT = FALSE;
-                in_SPACE = TRUE;
-                break;
-            default:
-                in_TEXT = TRUE;
-                if(in_SPACE) {
-                    argv[argc] = _argv+j;
-                    argc++;
-                }
-                _argv[j] = a;
-                j++;
-                in_SPACE = FALSE;
-                break;
-            }
-        }
-        i++;
-    }
-    _argv[j] = '\0';
-    argv[argc] = NULL;
-
-    (*_argc) = argc;
-    return argv;
-}
-#endif
-
 #ifdef _WIN32
 
 wchar_t*
@@ -1030,9 +904,7 @@ Cipher::decrypt_buffer ( unsigned char* buffer,
     gcry_cipher_close( cipher );
 }
 
-#ifndef LIFEO_WINDOZE
-
-// MARKED UP MENU ITEM
+// MARKED UP MENU ITEM =============================================================================
 Gtk::MenuItem*
 create_menuitem_markup( const Glib::ustring& str_markup,
                         const Glib::SignalProxy0< void >::SlotType& handler )
@@ -1046,6 +918,18 @@ create_menuitem_markup( const Glib::ustring& str_markup,
     return menuitem;
 }
 
+// QUICK CSS SETTER ================================================================================
+bool set_widget_css( Gtk::Widget* w, const Ustring& css )
+{
+    Glib::RefPtr< Gtk::CssProvider > css_prov = Gtk::CssProvider::create();
+    if( css_prov->load_from_data( css ) )
+        w->get_style_context()->add_provider( css_prov, GTK_STYLE_PROVIDER_PRIORITY_APPLICATION );
+    else
+        return false;
+
+    return true;
+}
+
 // ENTRYCLEAR ======================================================================================
 EntryClear::EntryClear( BaseObjectType* cobject, const Glib::RefPtr<Gtk::Builder>& )
     :   Gtk::Entry( cobject ), m_pressed( false )
@@ -1168,6 +1052,4 @@ bool is_treepath_more( const Gtk::TreePath& p, const Gtk::TreePath& pe )
     return false; // less or equal
 }
 
-#endif
-
 } // end of name space
