@@ -1,6 +1,6 @@
 /***********************************************************************************
 
-    Copyright (C) 2007-2018 Ahmet Öztürk (aoz_2@yahoo.com)
+    Copyright (C) 2007-2019 Ahmet Öztürk (aoz_2@yahoo.com)
 
     This file is part of Lifeograph.
 
@@ -33,31 +33,16 @@
 #include <sstream>
 #include <set>
 
-#ifndef LIFEO_WINDOZE
 #include <gtkmm.h>
-#else
-#include<winsock2.h>    // to silence the compiler
-#include <shlwapi.h>
-#endif
-
 #include <gcrypt.h>
 
-#ifndef LIFEO_WINDOZE
+
 typedef Glib::ustring Ustring;
-typedef Glib::ustring Wstring;
+typedef Glib::ustring::size_type UstringSize;
 typedef gunichar Wchar;
-#define W(s) s
 typedef Glib::RefPtr< Gdk::Pixbuf > Icon;
 typedef Gdk::RGBA Color;
-#else
-// values are temporary
-typedef std::string Ustring;
-typedef std::wstring Wstring;
-typedef wchar_t Wchar;
-#define W(s) L##s
-typedef void* Icon;
-typedef std::string Color;
-#endif
+
 
 namespace HELPERS
 {
@@ -123,6 +108,12 @@ enum Result
 static const int        RESPONSE_CANCEL     = 0;
 static const int        RESPONSE_GO         = 1;
 
+// GENERIC FUNCTIONS
+inline int max( const int l, const int r )
+{ return( l > r ? l : r ); }
+inline unsigned max_u( const unsigned l, const unsigned r )
+{ return( l > r ? l : r ); }
+
 // DATE ============================================================================================
 // order: 10 bits
 // day:    5 bits
@@ -252,10 +243,7 @@ class Date
         { return( ( m_date & MONTH_FILTER ) >> 15 ); }
         static unsigned int         get_month( const date_t d )
         { return( ( d & MONTH_FILTER ) >> 15 ); }
-
-#ifndef LIFEO_WINDOZE
         Glib::Date::Month           get_month_glib() const;
-#endif
 
         unsigned int                get_day() const
         { return( ( m_date & DAY_FILTER ) >> 10 ); }
@@ -276,10 +264,9 @@ class Date
             return ORDINAL_FLAG;
         }
 
-#ifndef LIFEO_WINDOZE
         Glib::Date                  get_glib() const
         { return Glib::Date( get_day(), get_month_glib(), get_year() ); }
-#endif
+
         time_t                      get_ctime() const
         { return get_ctime( m_date ); }
         static time_t               get_ctime( const date_t );
@@ -462,13 +449,16 @@ void print_error( Args... args )
 #endif
 
 // COLOR OPERATIONS ================================================================================
-#ifndef LIFEO_WINDOZE
-Gdk::RGBA           contrast2( const Gdk::RGBA&, const Gdk::RGBA&, const Gdk::RGBA& );
-Gdk::RGBA           midtone( const Gdk::RGBA&, const Gdk::RGBA& );
-Gdk::RGBA           midtone( const Gdk::RGBA&, const Gdk::RGBA&, float );
+inline float
+get_color_diff( const Color& c1, const Color c2 )
+{
+    return( fabs( c2.get_red() - c1.get_red() ) +
+            fabs( c2.get_green() - c1.get_green() ) +
+            fabs( c2.get_blue() - c1.get_blue() ) );
+}
 
-inline Glib::ustring
-convert_gdkcolor_to_html( const Gdk::RGBA& gdkcolor )
+inline Ustring
+convert_gdkcolor_to_html( const Color& gdkcolor )
 {
     // this function's source of inspiration is Geany
     char buffer[ 8 ];
@@ -480,8 +470,8 @@ convert_gdkcolor_to_html( const Gdk::RGBA& gdkcolor )
     return buffer;
 }
 
-inline Glib::ustring
-convert_gdkrgba_to_string( const Gdk::RGBA& gdkcolor )
+inline Ustring
+convert_gdkrgba_to_string( const Color& gdkcolor )
 {
     char buffer[ 14 ];
     g_snprintf( buffer, 14, "#%04X%04X%04X",
@@ -491,56 +481,9 @@ convert_gdkrgba_to_string( const Gdk::RGBA& gdkcolor )
     return buffer;
 }
 
-#else
-
-inline int
-parse_color_sub( const std::string& color, int begin, int end )
-{
-    int ret_val = 0;
-
-    for( int i = begin; i <= end; i++ )
-    {
-        char c = color[ i ];
-        if( c >= '0' && c <= '9' )
-        {
-            ret_val *= 16;
-            ret_val += ( c - '0' );
-        }
-        else if( c >= 'a' && c <= 'f' )
-        {
-            ret_val *= 16;
-            ret_val += ( c - 'a' + 10 );
-        }
-        else if( c >= 'A' && c <= 'F' )
-        {
-            ret_val *= 16;
-            ret_val += ( c - 'A' + 10 );
-        }
-    }
-
-    return ret_val;
-}
-
-inline COLORREF
-parse_color( const std::string& color )
-{
-    if( color.size() == 13 ) // #RRRRGGGGBBBB
-        return RGB( parse_color_sub( color, 1, 2 ),
-                    parse_color_sub( color, 5, 6 ),
-                    parse_color_sub( color, 9, 10 ) );
-    else // #RRGGBB
-        return RGB( parse_color_sub( color, 1, 2 ),
-                    parse_color_sub( color, 3, 4 ),
-                    parse_color_sub( color, 5, 6 ) );
-}
-
-COLORREF
-contrast2( COLORREF, COLORREF, COLORREF );
-
-COLORREF
-midtone( COLORREF, COLORREF, float );
-
-#endif
+Color               contrast2( const Color&, const Color&, const Color& );
+Color               midtone( const Color&, const Color& );
+Color               midtone( const Color&, const Color&, float );
 
 // FILE OPERATIONS =================================================================================
 #ifdef _WIN32
@@ -556,14 +499,7 @@ bool                is_dir( const std::string& );
 inline std::string
 get_filename_base( const std::string& path )
 {
-#ifndef LIFEO_WINDOZE
     return Glib::filename_display_basename( path );
-#else
-    char* stripped_path = new char[ path.size() + 1 ];
-    strcpy( stripped_path, path.c_str() );
-    PathStripPathA( stripped_path );
-    return std::string( stripped_path );
-#endif
 }
 
 #ifdef _WIN32
@@ -578,24 +514,17 @@ convert_filename_to_win32_locale( const std::string& );
 
 typedef std::set< std::string > ListPaths;
 
+// the following is not used right now but may be helpful in the future
+void                get_all_files_in_path( const std::string&, ListPaths& );
+
 // TEXT OPERATIONS =================================================================================
 long                convert_string( const std::string& );
 double              convert_string_d( const std::string& );
 bool                str_ends_with( const std::string&, const std::string& );
-
-#ifndef LIFEO_WINDOZE
-gunichar            char_lower( gunichar );
-bool                is_char_alpha( gunichar );
-
+Wchar               char_lower( Wchar );
+bool                is_char_alpha( Wchar );
 std::string         get_env_lang();
 
-#else
-wchar_t             char_lower( wchar_t );
-bool                is_char_alpha( wchar_t );
-
-PCHAR*              command_line_to_argvA( PCHAR, int* );
-#endif
-
 #ifdef _WIN32
 wchar_t*            convert_utf8_to_16( const Ustring& );
 char*               convert_utf16_to_8( const wchar_t* );
@@ -655,7 +584,6 @@ struct CipherBuffers
     }
 };
 
-#ifndef LIFEO_WINDOZE
 // ENTRY WIDGET WITH SELF-HANDLED CLEAR ICON =======================================================
 class EntryClear : public Gtk::Entry
 {
@@ -719,7 +647,7 @@ Gtk::MenuItem*      create_menuitem_markup( const Glib::ustring&,
 
 typedef sigc::signal< void >        SignalVoid;
 
-#endif // ifndef LIFEO_WINDOZE
+bool set_widget_css( Gtk::Widget*, const Ustring& );
 
 } // end of namespace HELPERS
 
