@@ -1,6 +1,6 @@
 /***********************************************************************************
 
-    Copyright (C) 2007-2018 Ahmet Öztürk (aoz_2@yahoo.com)
+    Copyright (C) 2007-2019 Ahmet Öztürk (aoz_2@yahoo.com)
 
     This file is part of Lifeograph.
 
@@ -33,7 +33,7 @@
 using namespace LIFEO;
 
 // STATIC MEMBERS
-const char                          Lifeograph::PROGRAM_VERSION_STRING[] = "1.5.1"
+const char                          Lifeograph::PROGRAM_VERSION_STRING[] = "1.6.1"
 #if LIFEOGRAPH_DEBUG_BUILD || defined( LIFEOGRAPH_BETA_RELEASE )
         "\n" BUILD_TIMESTAMP
 #endif
@@ -50,17 +50,20 @@ Glib::RefPtr< Gtk::ActionGroup >    Lifeograph::m_actions_edit;
 int                                 Lifeograph::s_internaloperation = 0;
 SignalVoid                          Lifeograph::s_signal_logout;
 Lifeograph::LoginStatus             Lifeograph::loginstatus = Lifeograph::LOGGED_OUT;
-std::set< std::string >             Lifeograph::stock_diaries;
 DiaryElement*                       Lifeograph::s_elem_dragged = nullptr;
-bool                                Lifeograph::s_flag_dragging = false;
+DiaryElement*                       Lifeograph::s_elem_drop_target = nullptr;
 LanguageList                        Lifeograph::s_lang_list;
 EnchantBroker*                      Lifeograph::s_enchant_broker = nullptr;
 std::string                         Lifeograph::s_color_insensitive;
 
+std::string                         Lifeograph::BINPATH;
 std::string                         Lifeograph::UIDIR;
 std::string                         Lifeograph::ICONDIR;
-std::string                         Lifeograph::DIARYDIR;
+std::string                         Lifeograph::MANUALPATH;
+std::string                         Lifeograph::EXAMPLEDIARYPATH;
+#ifdef _WIN32
 std::string                         Lifeograph::PIXMAPDIR;
+#endif
 std::string                         Lifeograph::LOCALEDIR;
 
 
@@ -72,8 +75,10 @@ Lifeograph::Lifeograph()
     p = this;
 
     Glib::set_application_name( LIFEO::PROGRAM_NAME );
-    if( Glib::file_test( PIXMAPDIR + "/lifeograph.png", Glib::FILE_TEST_EXISTS ) )
-        Gtk::Window::set_default_icon_from_file( PIXMAPDIR + "/lifeograph.png" );
+#ifdef _WIN32
+    if( Glib::file_test( PIXMAPDIR + "/net.sourceforge.Lifeograph.png", Glib::FILE_TEST_EXISTS ) )
+        Gtk::Window::set_default_icon_from_file( PIXMAPDIR + "/net.sourceforge.Lifeograph.png" );
+#endif
 
     m_actions_read = Gtk::ActionGroup::create( "read actions" );
     m_actions_edit = Gtk::ActionGroup::create( "edit actions" );
@@ -116,51 +121,21 @@ Lifeograph::quit()
             return false;
 
     // SAVE SETTINGS
-    AppWindow::p->get_size( settings.width, settings.height );
-    AppWindow::p->get_position( settings.position_x, settings.position_y );
-    settings.state_maximized = (
-            AppWindow::p->get_window()->get_state() & Gdk::WINDOW_STATE_MAXIMIZED );
+    if( m_flag_read_only == false )
+    {
+        AppWindow::p->get_size( settings.width, settings.height );
+        AppWindow::p->get_position( settings.position_x, settings.position_y );
+        settings.state_maximized = (
+                AppWindow::p->get_window()->get_state() & Gdk::WINDOW_STATE_MAXIMIZED );
 
-    settings.write();
+        settings.write();
+    }
+
+    AppWindow::p->hide();
 
     return true;
 }
 
-// APP MENU COMMAND REDIRECTION FUNCTIONS
-void appmenu_preferences()
-{
-    DialogPreferences::create();
-}
-void appmenu_about()
-{
-    AppWindow::p->show_about();
-}
-void appmenu_quit()
-{
-    if( Lifeograph::quit() )
-        AppWindow::p->hide();
-}
-
-void
-Lifeograph::init_paths()
-{
-#ifdef _WIN32
-    std::string exe_path{ HELPERS::get_exec_path() + "/" };
-    UIDIR = exe_path + "../share/lifeograph/ui";
-    ICONDIR = exe_path + "../share/lifeograph/icons";
-    DIARYDIR = exe_path + "../share/lifeograph/diaries";
-    PIXMAPDIR = exe_path + "../share/pixmaps";
-    LOCALEDIR = exe_path + "../share/locale";
-PRINT_DEBUG( "UIDIR: ", UIDIR );
-#else
-    UIDIR = DEF_UIDIR;
-    ICONDIR = DEF_ICONDIR;
-    DIARYDIR = DEF_DIARYDIR;
-    PIXMAPDIR = DEF_PIXMAPDIR;
-    LOCALEDIR = DEF_LOCALEDIR;
-#endif
-}
-
 void
 Lifeograph::on_startup()
 {
@@ -169,14 +144,16 @@ Lifeograph::on_startup()
     // SETTINGS
     Lifeograph::settings.read();
 
+    update_gtk_theme();
+
     // ENCHANT
     s_enchant_broker = enchant_broker_init();
     enchant_broker_list_dicts( s_enchant_broker, fill_lang_list_cb, nullptr );
 
     // APP MENU
-    add_action( "preferences", sigc::ptr_fun( &appmenu_preferences ) );
-    add_action( "about", sigc::ptr_fun( &appmenu_about ) );
-    add_action( "quit", sigc::ptr_fun( &appmenu_quit ) );
+    add_action( "help", sigc::mem_fun( this, &Lifeograph::show_manual ) );
+    add_action( "about", sigc::ptr_fun( &AppWindow::show_about ) );
+    add_action( "quit", sigc::hide_return( sigc::mem_fun( this, &Lifeograph::quit ) ) );
 
     m_menu = Gio::Menu::create();
     m_menu->append( _( "Preferences" ), "app.preferences" );
@@ -239,6 +216,12 @@ Lifeograph::on_command_line( const Glib::RefPtr< Gio::ApplicationCommandLine >&
             m_flag_read_only = true;
         }
         else
+        if( ! strcmp( argv[ i ], "--read-only-single" ) )
+        {
+            m_flag_single_file = true;
+            m_flag_read_only = true;
+        }
+        else
         if( access( argv[ i ], F_OK ) == 0 && ! is_dir( argv[ i ] ) )
         {
             ViewLogin::m_path_cur = argv[ i ];
@@ -256,11 +239,11 @@ Lifeograph::on_command_line( const Glib::RefPtr< Gio::ApplicationCommandLine >&
 }
 
 bool
-Lifeograph::load_gui( const std::string& path )
+Lifeograph::load_gui( Glib::RefPtr< Gtk::Builder >&b, const std::string& path )
 {
     try
     {
-        builder->add_from_file( path );
+        b->add_from_file( path );
     }
     catch( const Glib::FileError& ex )
     {
@@ -286,6 +269,13 @@ Lifeograph::get_builder2()
     return builder;
 }
 
+std::string
+Lifeograph::get_icon_dir()
+{
+    return settings.icon_theme.empty() ?
+            ICONDIR : ICONDIR + "/" + Lifeograph::settings.icon_theme;
+}
+
 // ACTIONS
 void
 Lifeograph::create_action( bool                             flag_edit,
@@ -335,6 +325,20 @@ Lifeograph::create_toggle_action( bool                                  flag_edi
     action->signal_activate().connect( slot );
 }
 
+Icon
+Lifeograph::load_icon( const std::string& name )
+{
+    std::string path{ get_icon_dir() + "/" + name };
+
+    if( !settings.icon_theme.empty() )
+    {
+        if( !g_file_test( PATH( path ).c_str(), G_FILE_TEST_EXISTS ) )
+            path = ICONDIR + "/" + name;    // fallback icon
+    }
+
+    return Gdk::Pixbuf::create_from_file( path );
+}
+
 Glib::RefPtr< Gdk::Pixbuf >
 Lifeograph::get_thumbnail( const std::string& path, int width )
 {
@@ -357,6 +361,13 @@ Lifeograph::get_thumbnail( const std::string& path, int width )
         return buf;
 }
 
+void
+Lifeograph::show_manual()
+{
+    std::string cmd{ PATH( BINPATH ) + " --read-only-single \"" + PATH( MANUALPATH ) + "\"" };
+    print_info( "CMD ", cmd, " returned: ", std::system( cmd.c_str() ) );
+}
+
 // DIALOGEVENT =====================================================================================
 HELPERS::DialogEvent::DialogEvent( const Glib::ustring& title )
 :   Gtk::Dialog( title )
