@@ -1,6 +1,6 @@
 /***********************************************************************************
 
-    Copyright (C) 2007-2018 Ahmet Öztürk (aoz_2@yahoo.com)
+    Copyright (C) 2007-2019 Ahmet Öztürk (aoz_2@yahoo.com)
 
     This file is part of Lifeograph.
 
@@ -36,7 +36,7 @@
 
 //#define LIFEOGRAPH_BETA_RELEASE
 
-#define LIFEOGRAPH_RELEASE_CODENAME "artemisia"
+#define LIFEOGRAPH_RELEASE_CODENAME "Do not be too hard, lest you be broken; do not be too soft, lest you be squeezed."
 
 
 namespace LIFEO
@@ -61,6 +61,10 @@ struct Icons
     Icon                            tag_32;
     Icon                            tag_theme_16;
     Icon                            tag_theme_32;
+    Icon                            tag_p_16;
+    Icon                            tag_p_32;
+    Icon                            tag_p_theme_16;
+    Icon                            tag_p_theme_32;
     Icon                            tag_category_16;
     Icon                            tag_category_32;
     Icon                            untagged_16;
@@ -72,6 +76,7 @@ struct Icons
     Icon                            chapter_temp_16;
     Icon                            chapter_temp_32;
     Icon                            filter_16;
+    Icon                            next_16;
     Icon                            bullet_16;
     Icon                            todo_auto_32;
     Icon                            todo_open_16;
@@ -103,7 +108,7 @@ class Lifeograph : public Gtk::Application
         static Glib::RefPtr< Lifeograph >
                                     create();
 
-        static bool                 quit();
+        bool                        quit();
 
                                     Lifeograph();
         virtual                     ~Lifeograph();
@@ -114,13 +119,23 @@ class Lifeograph : public Gtk::Application
         void                        on_activate();
         int             on_command_line( const Glib::RefPtr< Gio::ApplicationCommandLine >& );
 
+        static void                 update_gtk_theme()
+        {
+            Gtk::Settings::get_default()->property_gtk_application_prefer_dark_theme() =
+                settings.use_dark_theme;
+        }
+
         static Settings             settings;
         static Icons*               icons;
 
+        static std::string          BINPATH;
         static std::string          UIDIR;
         static std::string          ICONDIR;
-        static std::string          DIARYDIR;
+        static std::string          MANUALPATH;
+        static std::string          EXAMPLEDIARYPATH;
+#ifdef _WIN32
         static std::string          PIXMAPDIR;
+#endif
         static std::string          LOCALEDIR;
 
         static void                 create_action(
@@ -149,6 +164,10 @@ class Lifeograph : public Gtk::Application
         static Glib::RefPtr< Gtk::Builder >&
                                     get_builder2();
 
+        static std::string          get_icon_dir();
+
+        void                        show_manual();
+
         static SignalVoid           s_signal_logout;
 
         static int                  s_internaloperation;
@@ -158,8 +177,21 @@ class Lifeograph : public Gtk::Application
                                     drag_targets_tag;
         std::vector< Gtk::TargetEntry >
                                     drag_targets_entry;
-        static DiaryElement*        s_elem_dragged;
-        static bool                 s_flag_dragging;
+        static bool                 is_drag_in_progress()
+        { return( s_elem_dragged != nullptr ); }
+        static void                 begin_drag( DiaryElement* elem )
+        { s_elem_dragged = elem; }
+        static void                 update_drop_target( DiaryElement* elem )
+        { s_elem_drop_target = elem; }
+        static DiaryElement*        get_drop_target()
+        { return s_elem_drop_target; }
+        static DiaryElement*        get_dragged_elem()
+        { return s_elem_dragged; }
+        static void                 end_drag()
+        {
+            s_elem_dragged = nullptr;
+            s_elem_drop_target = nullptr;
+        }
 
         // COLORS
         static std::string          s_color_insensitive;
@@ -168,7 +200,9 @@ class Lifeograph : public Gtk::Application
         static EnchantBroker*       s_enchant_broker;
         static LanguageList         s_lang_list;
 
-        static std::set< std::string > stock_diaries;
+        static bool                 load_gui( Glib::RefPtr< Gtk::Builder >&, const std::string& );
+
+        static Icon                 load_icon( const std::string& );
 
         static Icon                 get_thumbnail( const std::string&, int );
 
@@ -182,11 +216,10 @@ class Lifeograph : public Gtk::Application
 
         static LoginStatus          loginstatus;
 
-        //std::string                 m_program_path;  // XXX NOT USED
-
         bool                        m_flag_force_welcome{ false };
         bool                        m_flag_open_directly{ false };
-        bool                        m_flag_read_only{ false }; // for command-line supplied file
+        bool                        m_flag_read_only{ false };  // complete read-only mode
+        bool                        m_flag_single_file{ false };    // single file mode
 
     protected:
         static void                 fill_lang_list_cb( const char* const l, const char* const,
@@ -194,7 +227,11 @@ class Lifeograph : public Gtk::Application
                                                        void* )
         { s_lang_list.insert( l ); }
 
-        static bool                 load_gui( const std::string& );
+        static bool                 load_gui( const std::string& path )
+        { return load_gui( builder, path ); }
+
+        static DiaryElement*        s_elem_dragged;
+        static DiaryElement*        s_elem_drop_target;
 
         // BUILDER
         static Glib::RefPtr< Gtk::Builder >
