@@ -32,15 +32,36 @@
 int
 main( int argc, char* argv[] )
 {
-    LIFEO::Lifeograph::init_paths();
+    using namespace LIFEO;
+
+    // INIT PATHS
+#ifdef _WIN32
+    const std::string bindir{ HELPERS::get_exec_path() };
+    Lifeograph::BINPATH     = bindir + "/lifeograph.exe";
+    Lifeograph::UIDIR       = bindir + "/../share/lifeograph/ui";
+    Lifeograph::ICONDIR     = bindir + "/../share/lifeograph/icons";
+    Lifeograph::MANUALPATH  = bindir + "/../share/lifeograph/diaries/Lifeograph Manual";
+    Lifeograph::EXAMPLEDIARYPATH = bindir + "/../share/lifeograph/diaries/example.diary";
+    Lifeograph::PIXMAPDIR   = bindir + "/../share/pixmaps";
+    Lifeograph::LOCALEDIR   = bindir + "/../share/locale";
+
+    PRINT_DEBUG( "UIDIR: ", Lifeograph::UIDIR );
+#else
+    Lifeograph::BINPATH = argv[ 0 ];
+    Lifeograph::UIDIR = DEF_UIDIR;
+    Lifeograph::ICONDIR = DEF_ICONDIR;
+    Lifeograph::MANUALPATH = DEF_MANUALPATH;
+    Lifeograph::EXAMPLEDIARYPATH = DEF_EXAMPLEDIARYPATH;
+    Lifeograph::LOCALEDIR = DEF_LOCALEDIR;
+#endif
 
     // LIBGETTEXT REQUIREMENTS
     setlocale( LC_ALL, "" );
-    bindtextdomain( PACKAGE, LIFEO::Lifeograph::LOCALEDIR.c_str() );
+    bindtextdomain( PACKAGE, Lifeograph::LOCALEDIR.c_str() );
     textdomain( PACKAGE );
 
     // APPLICATION
-    Glib::RefPtr< LIFEO::Lifeograph > app = LIFEO::Lifeograph::create();
+    Glib::RefPtr< LIFEO::Lifeograph > app = Lifeograph::create();
 
     return app->run( argc, argv );
 }
