@@ -1,6 +1,6 @@
 /***********************************************************************************
 
-    Copyright (C) 2007-2018 Ahmet Öztürk (aoz_2@yahoo.com)
+    Copyright (C) 2007-2019 Ahmet Öztürk (aoz_2@yahoo.com)
 
     This file is part of Lifeograph.
 
@@ -58,6 +58,8 @@ PanelDiary::PanelDiary()
         builder->get_widget( "L_add_topic", m_L_add_topic );
         builder->get_widget( "L_add_group", m_L_add_group );
 
+        builder->get_widget( "B_enable_edit", m_B_enable_edit );
+
         builder->get_widget( "Bx_logout", m_Bx_logout );
         builder->get_widget( "B_logout", m_B_logout );
         builder->get_widget( "MB_logout", m_MB_logout );
@@ -95,10 +97,10 @@ PanelDiary::PanelDiary()
                 "add_entry", sigc::mem_fun( this, &PanelDiary::handle_add_entry ) );
         m_A_add_chapter = Lifeograph::p->add_action(
                 "add_chapter", sigc::mem_fun( this, &PanelDiary::handle_add_chapter ) );
-        m_A_add_topic = Lifeograph::p->add_action(
-                "add_topic", sigc::mem_fun( this, &PanelDiary::handle_add_topic ) );
-        m_A_add_group = Lifeograph::p->add_action(
-                "add_group", sigc::mem_fun( this, &PanelDiary::handle_add_group ) );
+        m_A_add_topic = Lifeograph::p->add_action( "add_topic",
+                [ this ](){ add_topic_or_group_common( Diary::d->get_topics() ); } );
+        m_A_add_group = Lifeograph::p->add_action( "add_group",
+                [ this ](){ add_topic_or_group_common( Diary::d->get_groups() ); } );
 
         // FALLBACK ICON FOR SYSTEMS WITHOUT SYMBOLIC ICONS
         Glib::RefPtr< Gtk::IconTheme > theme = Gtk::IconTheme::get_default();
@@ -120,9 +122,10 @@ PanelDiary::PanelDiary()
         throw LIFEO::Error( "Failed to create the diary panel" );
     }
 
-
-
     // SIGNALS
+    m_B_enable_edit->signal_clicked().connect(
+            sigc::mem_fun( this, &PanelDiary::enable_editing ) );
+
     m_B_logout->signal_clicked().connect(
             sigc::bind( sigc::mem_fun( AppWindow::p, &AppWindow::logout ), true ) );
 
@@ -328,7 +331,8 @@ PanelDiary::update_entry_list()
     Lifeograph::s_internaloperation++;
     m_flag_calendar_updated = false;
 
-    AppWindow::p->m_entry_view->sync();
+    if( AppWindow::p->panel_main->get_cur_elem_type() == DiaryElement::ET_ENTRY )
+        AppWindow::p->m_entry_view->sync();
 
     // REMOVE MODEL AND DISBLE SORTING FOR SPEED-UP
     m_TV_entries->unset_model();
@@ -374,7 +378,6 @@ PanelDiary::update_entry_list()
                 Gtk::TreeRow row = * m_TV_entries->m_treestore->append( m_row_diary.children() );
                 fill_elem_row( row, entry );
             }
-
         }
     }
 
@@ -495,7 +498,7 @@ PanelDiary::update_calendar()
 void
 PanelDiary::handle_login()
 {
-    m_MB_logout->set_visible( Diary::d->is_encrypted() || ! Diary::d->is_read_only() );
+    m_MB_logout->set_visible( Diary::d->is_encrypted() );
 
     switch( Diary::d->get_sorting_criteria() )
     {
@@ -518,28 +521,87 @@ PanelDiary::handle_login()
     // not called automatically because of m_internaloperation:
     m_TV_entries->reset_sorting_criteria();
 
-    if( Diary::d->is_read_only() )
-    {
-        m_TV_entries->unset_rows_drag_source();
-        m_TV_entries->unset_rows_drag_dest();
-    }
-    else
-    {
-        m_TV_entries->enable_model_drag_source( Lifeograph::p->drag_targets_entry );
-        m_TV_entries->enable_model_drag_dest( Lifeograph::p->drag_targets_entry,
-                                              Gdk::ACTION_MOVE );
-    }
+    m_TV_entries->unset_rows_drag_source();
+    m_TV_entries->unset_rows_drag_dest();
 
     update_entry_list();
     if( !m_flag_calendar_updated )
         update_calendar();  // if calendar is not updated in update_entry_list()
 
-    m_MB_add->set_visible( ! Diary::d->is_read_only() );
+    m_MB_add->set_visible( false );
     m_Bx_new_elem->set_visible( true );
     m_G_elem_header->set_visible( true );
-    m_Bx_logout->set_visible( true );
-    m_B_tools->set_visible( true );
+    m_Bx_logout->set_visible( !Lifeograph::p->m_flag_single_file );
     m_Bx_search->set_visible( true );
+    m_B_enable_edit->set_visible( !Lifeograph::p->m_flag_read_only );
+    m_B_enable_edit->set_sensitive( !Diary::d->is_read_only() );
+}
+
+void
+PanelDiary::enable_editing()
+{
+    if( Lifeograph::loginstatus != Lifeograph::LOGGED_IN || Diary::d->is_editing_enabled() )
+        return;
+
+    switch( Diary::d->enable_editing() )
+    {
+        case LIFEO::SUCCESS:
+            break;
+        case FILE_LOCKED:
+            AppWindow::p->show_info(
+                    Gtk::MESSAGE_INFO,
+                    Glib::ustring::compose( _( STRING::DIARY_LOCKED ),
+                        Glib::ustring::compose( "<a href=\"file://%1\" title=\"%2\">%3</a>",
+                                                Glib::path_get_dirname( Diary::d->get_path() ),
+                                                _( "Click to open the containing folder" ),
+                                                Diary::d->get_path() + LOCK_SUFFIX ) ) );
+            return;
+        case FILE_NOT_WRITABLE:
+            AppWindow::p->show_info( Gtk::MESSAGE_ERROR, _( STRING::DIARY_NOT_WRITABLE ) );
+            return;
+        default:
+            AppWindow::p->show_info( Gtk::MESSAGE_ERROR, _( STRING::FAILED_TO_OPEN_DIARY ) );
+            return;
+    }
+    if( Diary::d->is_old() )
+    {
+        Gtk::MessageDialog* messagedialog
+                = new Gtk::MessageDialog( *AppWindow::p,
+                                          "",
+                                          false,
+                                          Gtk::MESSAGE_WARNING,
+                                          Gtk::BUTTONS_CANCEL,
+                                          true );
+        messagedialog->set_message( _( "Are You Sure You Want to Upgrade The Diary?" ) );
+        messagedialog->set_secondary_text( _( STRING::UPGRADE_DIARY_CONFIRM ) );
+        messagedialog->add_button( _( "Upgrade The Diary" ), Gtk::RESPONSE_ACCEPT );
+
+        int response( messagedialog->run() );
+
+        delete messagedialog;
+
+        if( response != Gtk::RESPONSE_ACCEPT )
+        {
+            return;
+        }
+    }
+
+    m_MB_logout->set_visible( true );
+    m_B_enable_edit->set_visible( false );
+
+    m_TV_entries->enable_model_drag_source( Lifeograph::p->drag_targets_entry );
+    m_TV_entries->enable_model_drag_dest( Lifeograph::p->drag_targets_entry,
+                                          Gdk::ACTION_MOVE );
+    m_MB_add->set_visible( true );
+
+    AppWindow::p->handle_edit_enabled();
+    AppWindow::p->m_tag_view->handle_edit_enabled();
+    AppWindow::p->m_filter_view->handle_edit_enabled();
+    AppWindow::p->m_entry_view->handle_edit_enabled();
+    AppWindow::p->m_diary_view->handle_edit_enabled();
+    AppWindow::p->m_chapter_view->handle_edit_enabled();
+    AppWindow::p->panel_main->handle_edit_enabled();
+    AppWindow::p->panel_extra->handle_edit_enabled();
 }
 
 void
@@ -552,9 +614,9 @@ PanelDiary::handle_logout()
     m_Bx_new_elem->set_visible( false );
     m_G_elem_header->set_visible( false );
     m_Bx_logout->set_visible( false );
-    m_B_tools->set_visible( false );
     m_Bx_search->set_visible( false );
     m_B_filter_revert->set_visible( false );
+    m_B_enable_edit->set_visible( false );
 
     m_TV_entries->clear();
 
@@ -614,10 +676,10 @@ PanelDiary::show_today()
 
     if( entry_today == nullptr )   // add new entry if no entry exists on selected date
     {
-        if( Diary::d->is_read_only() )
-            return;
-        else
+        if( Diary::d->is_editing_enabled() )
             flag_add = true;
+        else
+            return;
     }
     else                        // or current entry is already at today
     if( entry_today->get_date().get_pure() ==
@@ -636,7 +698,8 @@ PanelDiary::show_today()
 void
 PanelDiary::add_todo_entry()
 {
-    if( Lifeograph::p->loginstatus != Lifeograph::LOGGED_IN || Diary::d->is_read_only() )
+    if( Lifeograph::p->loginstatus != Lifeograph::LOGGED_IN ||
+        Diary::d->is_editing_enabled() == false )
         return;
 
     Entry* entry{ Diary::d->add_today() };
@@ -720,7 +783,7 @@ PanelDiary::handle_calendar_monthchanged()
 void
 PanelDiary::handle_add_entry()
 {
-    if( Diary::d->is_read_only() )
+    if( Diary::d->is_editing_enabled() == false )
         return;
 
     Entry* entry{ Diary::d->create_entry( m_date_cur.m_date ) };
@@ -737,25 +800,12 @@ PanelDiary::handle_add_entry()
 }
 
 void
-PanelDiary::handle_add_group()
+PanelDiary::add_topic_or_group_common( CategoryChapters* ctg )
 {
-    if( Diary::d->is_read_only() )
+    if( Diary::d->is_editing_enabled() == false )
         return;
 
-    Chapter* c( Diary::d->get_groups()->create_chapter_ordinal( _( STRING::NEW_CHAPTER_NAME ) ) );
-    Diary::d->update_entries_in_chapters();
-    update_entry_list();
-    c->show();
-    AppWindow::p->panel_main->start_title_edit();
-}
-
-void
-PanelDiary::handle_add_topic()
-{
-    if( Diary::d->is_read_only() )
-        return;
-
-    Chapter* c( Diary::d->get_topics()->create_chapter_ordinal( _( STRING::NEW_CHAPTER_NAME ) ) );
+    Chapter* c( ctg->create_chapter_ordinal( _( STRING::NEW_CHAPTER_NAME ) ) );
     Diary::d->update_entries_in_chapters();
     update_entry_list();
     c->show();
@@ -765,13 +815,13 @@ PanelDiary::handle_add_topic()
 void
 PanelDiary::handle_add_chapter()
 {
-    if( Diary::d->is_read_only() )
+    if( Diary::d->is_editing_enabled() == false )
         return;
 
     if( Diary::d->get_current_chapter_ctg()->get_chapter( m_date_cur.m_date ) == nullptr )
     {
         Chapter* c( Diary::d->get_current_chapter_ctg()->create_chapter(
-                _( "Untitled chapter" ), m_date_cur.m_date ) );
+                _( STRING::NEW_CHAPTER_NAME ), m_date_cur.m_date ) );
 
         Diary::d->update_entries_in_chapters();
         update_entry_list();
@@ -962,7 +1012,7 @@ PanelDiary::handle_search_string_changed()
     else
     {
         m_Po_replace->show();
-        bool show_replace( ! Diary::d->is_read_only() );
+        bool show_replace{ Diary::d->is_editing_enabled() };
         m_E_replace->set_visible( show_replace );
         m_B_replace->set_visible( show_replace );
         m_B_replace_all->set_visible( show_replace );
