@@ -1,6 +1,6 @@
 /***********************************************************************************
 
-    Copyright (C) 2007-2018 Ahmet Öztürk (aoz_2@yahoo.com)
+    Copyright (C) 2007-2019 Ahmet Öztürk (aoz_2@yahoo.com)
 
     This file is part of Lifeograph.
 
@@ -100,14 +100,9 @@ PanelExtra::PanelExtra( BaseObjectType* cobject, const Glib::RefPtr<Gtk::Builder
 
     // STYLE
     Glib::ustring data;
-    if( gtk_get_minor_version() >= 20 )
-        data = "treeview { background-color: transparent } "
-               "treeview:selected { color: #bb3333; "
-                                   "background-color: #dddddd }";
-    else
-        data = "GtkTreeView { background-color: transparent } "
-               "GtkTreeView:Selected { color: #bb3333; "
-                                      "background-color: #dddddd }";
+    data = "treeview { background-color: transparent } "
+           "treeview:selected { color: #bb3333; "
+           "background-color: #dddddd }";
 
     Glib::RefPtr< Gtk::CssProvider > css = Gtk::CssProvider::create();
     if( css->load_from_data( data ) )
@@ -126,21 +121,20 @@ PanelExtra::PanelExtra( BaseObjectType* cobject, const Glib::RefPtr<Gtk::Builder
 void
 PanelExtra::handle_login()
 {
-    if( Diary::d->is_read_only() )
-    {
-        unset_rows_drag_source();
-        unset_rows_drag_dest();
-    }
-    else
-    {
-        enable_model_drag_source( Lifeograph::p->drag_targets_tag );
-        enable_model_drag_dest( Lifeograph::p->drag_targets_tag, Gdk::ACTION_MOVE );
-        enable_model_drag_dest( Lifeograph::p->drag_targets_entry, Gdk::ACTION_COPY );
-    }
+    unset_rows_drag_source();
+    unset_rows_drag_dest();
 
     populate();
 }
 
+void
+PanelExtra::handle_edit_enabled()
+{
+    enable_model_drag_source( Lifeograph::p->drag_targets_tag );
+    enable_model_drag_dest( Lifeograph::p->drag_targets_tag, Gdk::ACTION_MOVE );
+    enable_model_drag_dest( Lifeograph::p->drag_targets_entry, Gdk::ACTION_COPY );
+}
+
 void
 PanelExtra::handle_logout()
 {
@@ -229,7 +223,7 @@ PanelExtra::populate()
     // ROOT TAGS
     for( auto& kv_tag : *Diary::d->get_tags() )
     {
-        if( kv_tag.second->get_category() == nullptr )
+        if( kv_tag.second->get_ctg() == nullptr )
             add_elem_to_list( kv_tag.second );
     }
 
@@ -320,12 +314,10 @@ PanelExtra::refresh_elem( DiaryElement* elem )
 void
 PanelExtra::on_drag_begin( const Glib::RefPtr< Gdk::DragContext >& context )
 {
-    if( Lifeograph::s_elem_dragged )
+    if( Lifeograph::is_drag_in_progress() )
     {
-        Lifeograph::s_flag_dragging = true;
-
         //Gtk::TreeView::on_drag_begin( context );
-        context->set_icon( Lifeograph::s_elem_dragged->get_icon32(), 0, 0 );
+        context->set_icon( Lifeograph::get_dragged_elem()->get_icon32(), 0, 0 );
     }
 }
 
@@ -333,17 +325,15 @@ void
 PanelExtra::on_drag_end( const Glib::RefPtr< Gdk::DragContext >& context )
 {
     Gtk::TreeView::on_drag_end( context );
-    Lifeograph::s_elem_dragged = nullptr;
+    Lifeograph::end_drag();
+
 }
 
 bool
 PanelExtra::on_drag_motion( const Glib::RefPtr< Gdk::DragContext >& context,
                                  int x, int y, guint time )
 {
-    if( ! Lifeograph::s_flag_dragging )
-        return false;
-
-    if( ! Lifeograph::s_elem_dragged )
+    if( ! Lifeograph::is_drag_in_progress() )
         return false;
 
     bool retval( false );
@@ -380,24 +370,24 @@ PanelExtra::on_drag_motion( const Glib::RefPtr< Gdk::DragContext >& context,
     {
         Gtk::TreeRow row( * m_treestore->get_iter( path ) );
         element = row[ m_treestore->colrec.ptr ];
-        if( element != nullptr && element != Lifeograph::s_elem_dragged )
+        if( element != nullptr && element != Lifeograph::get_dragged_elem() )
         {
-            switch( Lifeograph::s_elem_dragged->get_type() )
+            switch( Lifeograph::get_dragged_elem()->get_type() )
             {
                 case DiaryElement::ET_TAG:
                 {
-                    Tag *tag_dragged( dynamic_cast< Tag* >( Lifeograph::s_elem_dragged ) );
+                    Tag *tag_dragged( dynamic_cast< Tag* >( Lifeograph::get_dragged_elem() ) );
 
                     if( element->get_type() == DiaryElement::ET_TAG_CTG )
                     {
                         CategoryTags* ctg_dropped( dynamic_cast< CategoryTags* >( element ) );
-                        if( tag_dragged->get_category() != ctg_dropped )
+                        if( tag_dragged->get_ctg() != ctg_dropped )
                             retval = true;
                     }
                     else if( element->get_type() == DiaryElement::ET_TAG )
                     {
                         Tag* tag_dropped( dynamic_cast< Tag* >( element ) );
-                        if( tag_dragged->get_category() != tag_dropped->get_category() )
+                        if( tag_dragged->get_ctg() != tag_dropped->get_ctg() )
                             retval = true;
                     }
                     break;
@@ -419,7 +409,7 @@ PanelExtra::on_drag_motion( const Glib::RefPtr< Gdk::DragContext >& context,
 
     if( retval )
     {
-        m_elem_drop_target = element;
+        Lifeograph::update_drop_target( element );
         context->drag_status( action, time );
         get_selection()->select( path );
     }
@@ -434,47 +424,45 @@ PanelExtra::on_drag_drop( const Glib::RefPtr< Gdk::DragContext >& context,
                           int x, int y, guint time )
 {
     bool flag_successful( false );
+    auto drop_target{ Lifeograph::get_drop_target() };
 
-    if( Lifeograph::s_flag_dragging )
+    if( Lifeograph::is_drag_in_progress() && drop_target != nullptr )
     {
-        if( Lifeograph::s_elem_dragged != nullptr || m_elem_drop_target != nullptr )
+        switch( Lifeograph::get_dragged_elem()->get_type() )
         {
-            switch( Lifeograph::s_elem_dragged->get_type() )
+            case DiaryElement::ET_TAG:
             {
-                case DiaryElement::ET_TAG:
+                Tag* tag( dynamic_cast< Tag* >( Lifeograph::get_dragged_elem() ) );
+                if( drop_target->get_type() == DiaryElement::ET_TAG_CTG )
                 {
-                    Tag* tag( dynamic_cast< Tag* >( Lifeograph::s_elem_dragged ) );
-                    if( m_elem_drop_target->get_type() == DiaryElement::ET_TAG_CTG )
-                    {
-                        CategoryTags* ctg( dynamic_cast< CategoryTags* >( m_elem_drop_target ) );
-                        tag->set_category( ctg );
-                        flag_successful = true;
-                    }
-                    else if( m_elem_drop_target->get_type() == DiaryElement::ET_TAG )
-                    {
-                        Tag* target_tag( dynamic_cast< Tag* >( m_elem_drop_target ) );
-                        tag->set_category( target_tag->get_category() );
-                        flag_successful = true;
-                    }
-                    break;
+                    CategoryTags* ctg( dynamic_cast< CategoryTags* >( drop_target ) );
+                    tag->set_ctg( ctg );
+                    flag_successful = true;
                 }
-                case DiaryElement::ET_ENTRY:
-                case DiaryElement::ET_MULTIPLE_ENTRIES:
-                    if( m_elem_drop_target->get_type() == DiaryElement::ET_TAG ||
-                        m_elem_drop_target->get_type() == DiaryElement::ET_UNTAGGED )
-                    {
-                        Tag* target_tag( dynamic_cast< Tag* >( m_elem_drop_target ) );
-
-                        sigc::slot< void, Entry* > s = sigc::bind(
-                                 sigc::mem_fun( this, &PanelExtra::tag_entry ), target_tag );
-                        AppWindow::p->panel_diary->do_for_each_selected_entry( s );
-
-                        flag_successful = true;
-                    }
-                    break;
-                default:
-                    break;
+                else if( drop_target->get_type() == DiaryElement::ET_TAG )
+                {
+                    Tag* target_tag( dynamic_cast< Tag* >( drop_target ) );
+                    tag->set_ctg( target_tag->get_ctg() );
+                    flag_successful = true;
+                }
+                break;
             }
+            case DiaryElement::ET_ENTRY:
+            case DiaryElement::ET_MULTIPLE_ENTRIES:
+                if( drop_target->get_type() == DiaryElement::ET_TAG ||
+                    drop_target->get_type() == DiaryElement::ET_UNTAGGED )
+                {
+                    Tag* target_tag( dynamic_cast< Tag* >( drop_target ) );
+
+                    sigc::slot< void, Entry* > s = sigc::bind(
+                             sigc::mem_fun( this, &PanelExtra::tag_entry ), target_tag );
+                    AppWindow::p->panel_diary->do_for_each_selected_entry( s );
+
+                    flag_successful = true;
+                }
+                break;
+            default:
+                break;
         }
     }
 
@@ -490,30 +478,30 @@ PanelExtra::on_drag_drop( const Glib::RefPtr< Gdk::DragContext >& context,
 bool
 PanelExtra::on_button_press_event( GdkEventButton* event )
 {
-    Lifeograph::s_flag_dragging = false;
-    Lifeograph::s_elem_dragged = nullptr;
+    Lifeograph::end_drag();
 
     Gtk::TreePath path;
     if( this->get_path_at_pos( event->x, event->y, path ) )
     {
-        Gtk::TreeRow row = * m_treestore->get_iter( path );
-        Lifeograph::s_elem_dragged = row[ m_treestore->colrec.ptr ];
-        if( Lifeograph::s_elem_dragged == nullptr )
-            return true;    // ignore
-        else
+        Gtk::TreeRow row{ * m_treestore->get_iter( path ) };
+        DiaryElement* elem{ row[ m_treestore->colrec.ptr ] };
+        if( elem )
         {
-            switch( Lifeograph::s_elem_dragged->get_type() )
+            switch( elem->get_type() )
             {
                 case DiaryElement::ET_TAG:
                 case DiaryElement::ET_UNTAGGED:
                 case DiaryElement::ET_FILTER:   // actually not included now
                 case DiaryElement::ET_TAG_CTG:
+                    Lifeograph::begin_drag( elem );
                     break;
                 default:
-                    Lifeograph::s_elem_dragged = nullptr;
+                    Lifeograph::end_drag();
                     return true;
             }
         }
+        else
+            return true;    // ignore
     }
     else
     {
@@ -526,12 +514,10 @@ PanelExtra::on_button_press_event( GdkEventButton* event )
 bool
 PanelExtra::on_button_release_event( GdkEventButton* event )
 {
-    if( ! Lifeograph::s_flag_dragging /*&& event->button != 1*/ )
+    if( Lifeograph::is_drag_in_progress() /*&& event->button != 1*/ )
     {
-        if( Lifeograph::s_elem_dragged )
-            Lifeograph::s_elem_dragged->show();
-
-        Lifeograph::s_elem_dragged = nullptr;
+        Lifeograph::get_dragged_elem()->show();
+        Lifeograph::end_drag();
     }
 
     get_selection()->unselect_all();
