@@ -24,6 +24,7 @@
 #include "app_window.hpp"
 #include "printing.hpp"
 #include "diary.hpp"
+#include "chart_surface.hpp"
 
 
 using namespace LIFEO;
@@ -62,11 +63,12 @@ PrintableText::render( const Glib::RefPtr< Gtk::PrintContext > &print_context )
 
             if( flag_1st )
             {
-                start_pos = logical_rect.get_y() / 1024.0 - m_v_offset;
+                start_pos = logical_rect.get_y() / Pango::SCALE - m_v_offset;
                 flag_1st = false;
             }
 
-            cr->move_to( logical_rect.get_x() / 1024.0, baseline / 1024.0 - start_pos );
+            cr->move_to( logical_rect.get_x() / Pango::SCALE + PrintOpr::s_margin_h / 2,
+                         baseline / Pango::SCALE - start_pos );
 
             layout_line->show_in_cairo_context( cr );
         }
@@ -93,6 +95,9 @@ PrintableImage::render( const Glib::RefPtr< Gtk::PrintContext > &print_context )
 }
 
 // PRINT OPERATION =================================================================================
+double PrintOpr::s_margin_h{ 0 };
+double PrintOpr::s_margin_v{ 0 };
+
 PrintOpr::PrintOpr()
 {
     m_refSettings = Gtk::PrintSettings::create();
@@ -119,8 +124,11 @@ PrintOpr::clear_content()
 void
 PrintOpr::init_variables( const Glib::RefPtr< Gtk::PrintContext >& print_context  )
 {
+    s_margin_h = print_context->get_dpi_x() * m_opt_margin; // 1 inch when full
+    s_margin_v = print_context->get_dpi_y() * m_opt_margin; // 1 inch when full
+    m_v_offset = s_margin_v / 2.0;
+
     m_n_pages = 1;
-    m_v_offset = 0.0;
     m_last_chapter_date = 0;
     m_flag_chapter_round = DiaryElement::ET_CHAPTER;    // start with chapters
 
@@ -150,66 +158,46 @@ PrintOpr::init_variables( const Glib::RefPtr< Gtk::PrintContext >& print_context
 void
 PrintOpr::set_hide_comments( bool flag_hide )
 {
-    m_parser.option_hide_comments = flag_hide;
+    m_parser.m_pango_opt_hide_comments = flag_hide;
 }
 
 Gtk::Widget*
 PrintOpr::on_create_custom_widget()
 {
-    Gtk::Grid *grid_main( new Gtk::Grid );
-    Gtk::Box *vbox_extent( Gtk::manage( new Gtk::Box( Gtk::ORIENTATION_VERTICAL, 5 ) ) );
-    Gtk::RadioButtonGroup rgroup_extent;
-    m_rbutton_entry = ( Gtk::manage(
-            new Gtk::RadioButton( rgroup_extent, _( "Only Current Entry" ) ) ) );
-    m_rbutton_filtered = ( Gtk::manage(
-            new Gtk::RadioButton( rgroup_extent, _( "All Filtered Entries" ) ) ) );
-    m_rbutton_diary = ( Gtk::manage(
-            new Gtk::RadioButton( rgroup_extent, _( "Entire Diary" ) ) ) );
-    Gtk::Frame *frame_extent( create_frame( _( "What to Print?" ), *vbox_extent ) );
+    if( m_builder )
+        m_builder.reset();
 
-    Gtk::Box *vbox_font( Gtk::manage( new Gtk::Box( Gtk::ORIENTATION_VERTICAL, 5 ) ) );
-    m_fontbutton = Gtk::manage( new Gtk::FontButton( ThemeSystem::get()->font.to_string() ) );
-    m_check_use_entry_font = Gtk::manage(
-            new Gtk::CheckButton( _( "Use Theme Fonts in Entries" ) ) );
-    m_check_justify = Gtk::manage( new Gtk::CheckButton( _( "Justify Text on the Page" ) ) );
-    Gtk::Frame *frame_font( create_frame( _( "Text Options" ), *vbox_font ) );
+    m_builder = Gtk::Builder::create();
+    Lifeograph::load_gui( m_builder, Lifeograph::UIDIR + "/print.ui" );
 
-    grid_main->set_border_width( 8 );
-    grid_main->set_column_homogeneous( true );
+    m_builder->get_widget( "Bx_print", m_Bx_print );
 
-    vbox_extent->set_border_width( 5 );
-    vbox_extent->pack_start( *m_rbutton_entry, Gtk::PACK_SHRINK );
-    vbox_extent->pack_start( *m_rbutton_filtered, Gtk::PACK_SHRINK );
-    vbox_extent->pack_start( *m_rbutton_diary, Gtk::PACK_SHRINK );
-    grid_main->attach( *frame_extent, 0, 0, 1, 2 );
+    m_builder->get_widget( "RB_print_current", m_RB_current );
+    m_builder->get_widget( "RB_print_all", m_RB_all );
 
-    vbox_font->set_border_width( 5 );
-    vbox_font->pack_start( *m_fontbutton, Gtk::PACK_SHRINK );
-    vbox_font->pack_start( *m_check_use_entry_font, Gtk::PACK_SHRINK );
-    vbox_font->pack_start( *m_check_justify, Gtk::PACK_SHRINK );
-    grid_main->attach( *frame_font, 1, 0, 1, 1 );
+    m_builder->get_widget( "FB_print", m_FB );
 
-    // could not figure out how to use Gtk::Builder in conjunction with PrintDialog
-    // using a dedicated ui file may be one way of doing it
-//    Lifeograph::get_builder()->get_widget( "vbox_printing", s_vbox_extras );
-//    Lifeograph::get_builder()->get_widget( "rbutton_print_entry", m_rbutton_entry );
-//    Lifeograph::get_builder()->get_widget( "rbutton_print_filtered", m_rbutton_filtered );
-//    Lifeograph::get_builder()->get_widget( "rbutton_print_diary", m_rbutton_diary );
+    m_builder->get_widget( "CB_print_theme_font", m_CB_theme_font );
+    m_builder->get_widget( "CB_print_justify", m_CB_justify );
 
-    grid_main->show_all_children();
+    m_builder->get_widget( "RB_print_margin_off", m_RB_margin_off );
+    m_builder->get_widget( "RB_print_margin_half", m_RB_margin_half );
 
-    return grid_main;
+    return m_Bx_print;
 }
 
 void
 PrintOpr::on_custom_widget_apply( Gtk::Widget* widget )
 {
-    m_opt_one_entry = m_rbutton_entry->get_active();
-    m_opt_entire_diary = m_rbutton_diary->get_active();
+    m_opt_one_entry = m_RB_current->get_active();
+    m_opt_entire_diary = m_RB_all->get_active();
 
-    m_font = Pango::FontDescription( m_fontbutton->get_font_name() );
-    m_parser.option_use_theme_font = m_check_use_entry_font->get_active();
-    m_opt_justify = m_check_justify->get_active();
+    m_font = Pango::FontDescription( m_FB->get_font_name() );
+    m_parser.m_pango_opt_use_theme_font = m_CB_theme_font->get_active();
+    m_opt_justify = m_CB_justify->get_active();
+
+    m_opt_margin = m_RB_margin_off->get_active() ? 0.0f :
+                                ( m_RB_margin_half->get_active() ? 0.5f : 1.0f );
 }
 
 Glib::RefPtr< PrintOpr >
@@ -277,7 +265,7 @@ PrintOpr::add_page()
 {
     m_content.push_back( new PrintablePageBreak );
     m_n_pages++;
-    m_v_offset = 0.0;
+    m_v_offset = ( s_margin_v / 2.0 );
 }
 
 void
@@ -285,8 +273,8 @@ PrintOpr::handle_text( const Glib::RefPtr< Gtk::PrintContext >& print_context )
 {
     m_layout = print_context->create_pango_layout();
 
-    const double width = print_context->get_width();
-    const double height = print_context->get_height();
+    const double width = print_context->get_width() - s_margin_h;
+    const double height = print_context->get_height() - s_margin_v;
     double total_line_h = 0.0;
 
     m_layout->set_width( static_cast< int >( width * Pango::SCALE ) );
@@ -305,7 +293,7 @@ PrintOpr::handle_text( const Glib::RefPtr< Gtk::PrintContext >& print_context )
     {
         Pango::Rectangle ink_rect, logical_rect;
         m_layout->get_line( line )->get_extents( ink_rect, logical_rect );
-        const double line_height = logical_rect.get_height() / 1024.0;
+        const double line_height = logical_rect.get_height() / Pango::SCALE;
 
         if( ( m_v_offset + total_line_h + line_height ) > height )
         {
@@ -313,7 +301,7 @@ PrintOpr::handle_text( const Glib::RefPtr< Gtk::PrintContext >& print_context )
             if( line < 0 )  // if first line is higher than the remaining space on the page
             {
                 // at least one line should fit into the page
-                if( m_v_offset == 0.0 )
+                if( m_v_offset == ( s_margin_v / 2.0 ) )
                 {
                     print_error( "Cannot print, line is too big for the page" );
                     line++;
@@ -374,8 +362,8 @@ PrintOpr::handle_image( const Glib::RefPtr< Gtk::PrintContext >& print_context )
 Printable::Type
 PrintOpr::get_more()
 {
-    Chapter* chapter( NULL );
-    Entry* entry( NULL );
+    Chapter* chapter( nullptr );
+    Entry* entry( nullptr );
     Printable::Type return_type( Printable::PT_PAGE_BREAK );
 
     m_marked_up_text.clear();
@@ -454,7 +442,7 @@ PrintOpr::get_more()
         // ENTRY
         if( m_opt_entire_diary || entry->get_filtered_out() == false || m_opt_one_entry )
         {
-            return_type = m_parser.parse();
+            return_type = m_parser.process();
             m_marked_up_text += m_parser.m_pango_text;
             if( return_type == Printable::PT_ENTRY_BREAK )
                 ++m_iter_entry;
@@ -536,281 +524,362 @@ EntryParserPango::set( const Entry* entry )
     m_pango_text.clear();
     if( m_ptr2entry != entry )
     {
+        auto size{ entry->get_text()->size() };
         m_pango_begin = 0;  // do not reset when continuing
         m_ptr2entry = entry;
+        m_format_map.clear();
+        m_format_map.assign( size + 1, 0 ); // last zero (i.e. +1) is for the file end
+
+        EntryParser::parse( 0, size );
     }
 }
 
 Printable::Type
-EntryParserPango::parse()
+EntryParserPango::process()
 {
-    m_pango_return = Printable::PT_ENTRY_BREAK;
-    m_pango_text.clear();
+    Recipe::Id format_prev{ 0 };
+    Recipe::Id format_curr{ 0 };
 
-    if( m_pango_begin == 0 )
+    m_pango_return = Printable::PT_ENTRY_BREAK;
+
+    if( m_pango_begin == 0 )    // only at the beginning of the entry
     {
+        // ENTRY DATE
         if( ! m_ptr2entry->get_date().is_hidden() )
-        {
             m_pango_text = Glib::ustring::compose(
                     "<b>%1</b>\n", m_ptr2entry->get_date().format_string() );
-        }
 
-        if( option_use_theme_font && m_ptr2entry->get_theme_is_set() )
+        // THEME
+        if( m_pango_opt_use_theme_font && m_ptr2entry->get_theme_is_set() )
         {
             // TODO: health of the font should also be checked as it may crash the program
             m_pango_text += "<span font='";
             m_pango_text += m_ptr2entry->get_theme()->font.to_string();
             m_pango_text += "'>";
         }
+
+        // ENTRY TO-DO STATUS
+        if( m_ptr2entry->get_status() & ES::FILTER_TODO_PURE )
+        {
+            m_pango_text += "<big><big>";
+            switch( m_ptr2entry->get_status() & ES::FILTER_TODO )
+            {
+                case ES::TODO:
+                    m_pango_text += "[ ] ";
+                    break;
+                case ES::PROGRESSED:
+                    m_pango_text += "[~] ";
+                    break;
+                case ES::DONE:
+                    m_pango_text += "[+] ";
+                    break;
+                case ES::CANCELED:
+                    m_pango_text += "[X] ";
+                    break;
+            }
+            m_pango_text += "</big></big>";
+        }
     }
 
-    EntryParser::parse( m_pango_begin, m_ptr2entry->get_text()->size() );
+    for( UstringSize i = m_pango_begin; i <= m_pos_end; i++ ) // <= is for handling the file end
+    {
+        format_curr = m_format_map[ i ];
 
-    if( option_use_theme_font && m_ptr2entry->get_theme_is_set() )
+        //Wchar ch = i < m_pos_end ? get_char_at( i ) : '\n' ;
+        //PRINT_DEBUG( ">>>>>>>> ", ch, " - ", format_curr );
+
+        if( format_curr != format_prev )
+        {
+            if( not( format_curr & RID_MARKUP ) )
+            {
+                // close last
+                if( format_prev & RID_MONOSPACE )
+                    m_pango_text += "</span>";
+                if( format_prev & RID_LINK )
+                    m_pango_text += "</span>";
+                if( format_prev & RID_STRIKETHROUGH )
+                    m_pango_text += "</s>";
+                if( format_prev & RID_DONE )
+                    m_pango_text += "</span>";
+                if( format_prev & RID_HIGHLIGHT )
+                    m_pango_text += "</span>";
+                if( format_prev & RID_ITALIC )
+                    m_pango_text += "</i>";
+                if( format_prev & RID_BOLD )
+                    m_pango_text += "</b>";
+                if( format_prev & RID_COMMENT )
+                    m_pango_text += "</sup></small>";
+                if( format_prev & RID_SUBSUBHEADING )
+                    m_pango_text += "</b></big>";
+                if( format_prev & RID_SUBHEADING )
+                    m_pango_text += "</b></big></big>";
+                if( format_prev & RID_HEADING )
+                    m_pango_text += "</b></big></big></big>";
+
+                // open new (in reverse order)
+                if( format_curr & RID_HEADING )
+                    m_pango_text += "<big><big><big><b>";
+                if( format_curr & RID_SUBHEADING )
+                    m_pango_text += "<big><big><b>";
+                if( format_curr & RID_SUBSUBHEADING )
+                    m_pango_text += "<big><b>";
+                if( format_curr & RID_COMMENT )
+                    m_pango_text += "<small><sup>";
+                if( format_curr & RID_BOLD )
+                    m_pango_text += "<b>";
+                if( format_curr & RID_ITALIC )
+                    m_pango_text += "<i>";
+                if( format_curr & RID_HIGHLIGHT )
+                    m_pango_text += STR::compose( "<span bgcolor='",
+                            convert_gdkcolor_to_html( ThemeSystem::get()->color_highlight ),
+                            "'>" );
+                if( format_curr & RID_DONE )
+                    m_pango_text += STR::compose( "<span bgcolor='",
+                            convert_gdkcolor_to_html( midtone( Theme::s_color_done,
+                                     ThemeSystem::get()->color_base,
+                                     0.35 ) ),
+                            "'>" );
+                if( format_curr & RID_STRIKETHROUGH )
+                    m_pango_text += "<s>";
+                if( format_curr & RID_LINK )
+                    m_pango_text += "<span color='blue' underline='single'>";
+                if( format_curr & RID_MONOSPACE )
+                    m_pango_text += "<span font='Monospace'>";
+            }
+
+            // IMAGES
+            if( format_curr & RID_IMAGE )
+            {
+                auto c{ get_format_string_size( i, RID_IMAGE ) };
+                std::string link{ m_ptr2entry->get_text()->substr( i, c ) };
+                PRINT_DEBUG( "process_RID_IMAGE: ", link );
+
+                // HANDLE RELATIVE LINKS
+                if( link.find( "rel://" ) == 0 )
+                    link.replace( 0, 5, "file://" + Glib::path_get_dirname( Diary::d->get_path() ) );
+
+                // we have already used try{} in apply_link() we don't "try" again:
+                m_pango_pixbuf = Lifeograph::get_thumbnail( Glib::filename_from_uri( link ),
+                                                            m_max_thumbnail_width );
+                m_pango_return = Printable::PT_IMAGE;
+                m_pango_begin = i+c;
+                return m_pango_return;
+            }
+            // CHARTS
+            else if( format_curr & RID_CHART )
+            {
+                auto c{ get_format_string_size( i, RID_CHART ) };
+                Tag* tag{ Diary::d->get_tags()->get_tag(
+                        m_ptr2entry->get_text()->substr( i + 6, c - 6 )  ) };
+                        // "chart:" is 6 chars long
+                if( tag )
+                {
+                    try
+                    {
+                        auto surface = new ChartSurface( tag, m_max_thumbnail_width*1.5 );
+
+                        m_pango_pixbuf = surface->get_pixbuf();
+                        m_pango_return = Printable::PT_IMAGE;
+                        m_pango_begin = i+c;
+
+                        delete surface;
+
+                        return m_pango_return;
+                    }
+                    catch( ... )
+                    {
+                        PRINT_DEBUG( "Could not add chart" );
+                    }
+                }
+            }
+        }
+
+        if( not( format_curr & RID_MARKUP ) && i < m_pos_end )
+        {
+            m_pango_text += Glib::Markup::escape_text( m_ptr2entry->get_text()->substr( i, 1 ) );
+
+            format_prev = format_curr;
+        }
+    } // close loop
+
+    // CLOSE THEME
+    if( m_pango_opt_use_theme_font && m_ptr2entry->get_theme_is_set() )
         m_pango_text += "</span>";
 
     return m_pango_return;
 }
 
 void
-EntryParserPango::apply_regular()
+EntryParserPango::apply_markup( Recipe::Id format )
 {
-    if( pos_regular < m_pos_start )
-    {
-        const Glib::ustring esc_text(
-                m_ptr2entry->get_text()->substr( pos_regular, m_pos_start - pos_regular ) );
-        m_pango_text.append( Glib::Markup::escape_text( esc_text ) );
-    }
+    m_format_map[ m_recipe_curr->m_pos_start ] |= RID_MARKUP;
+    m_format_map[ m_pos_curr ] |= RID_MARKUP;
+
+    apply_format( format, m_recipe_curr->m_pos_start, m_pos_curr );
 }
 
 void
-EntryParserPango::apply_markup( const Glib::ustring& tag_o, const Glib::ustring& tag_c,
-                                int off_o, int off_c )
+EntryParserPango::apply_heading( bool flag_non_empty )
 {
-    // before adding any marked up text, last regular text chunk must be handled:
-    begin_apply();  // in turn calls apply_regular()
-
-    const Glib::ustring esc_text(
-            m_ptr2entry->get_text()->substr( m_pos_start + off_o,
-                                             m_pos_current - m_pos_start + off_c ) );
-    m_pango_text += tag_o;
-    m_pango_text += Glib::Markup::escape_text( esc_text );
-    m_pango_text += tag_c;
-}
-
-void
-EntryParserPango::apply_heading_end()
-{
-    const Glib::ustring esc_text( m_ptr2entry->get_text()->substr( 0, m_pos_current ) );
-    m_pango_text += "<big><b>";
-    switch( m_ptr2entry->get_status() & ES::FILTER_TODO )
-    {
-        case ES::TODO:
-            m_pango_text += "[ ] ";
-            break;
-        case ES::PROGRESSED:
-            m_pango_text += "[~] ";
-            break;
-        case ES::DONE:
-            m_pango_text += "[+] ";
-            break;
-        case ES::CANCELED:
-            m_pango_text += "[X] <s>";
-            break;
-    }
-    m_pango_text += Glib::Markup::escape_text( esc_text );
-    if( m_ptr2entry->get_status() & ES::CANCELED )
-        m_pango_text += "</s>";
-    m_pango_text += "</b></big>";
-    pos_regular = m_pos_current;
+    apply_format( ( m_ptr2entry->get_status() & ES::CANCELED ) ?
+                        RID_HEADING|RID_STRIKETHROUGH : RID_HEADING, 0, get_para_end( 0 ) );
 }
 
 void
 EntryParserPango::apply_subheading()
 {
-    // before adding any marked up text, last regular text chunk must be handled:
-    begin_apply();  // in turn calls apply_regular()
-    pos_regular = m_pos_start;
-
-    m_pango_text += "<big>";
+    apply_format( RID_SUBHEADING,
+                  m_recipe_curr->m_pos_start, get_para_end( m_recipe_curr->m_pos_start ) );
 }
 
 void
-EntryParserPango::apply_subheading_end()
+EntryParserPango::apply_subsubheading()
 {
-    if( pos_regular < m_pos_current )
-    {
-        const Glib::ustring esc_text(
-                m_ptr2entry->get_text()->substr( pos_regular, m_pos_current - pos_regular + 1 ) );
-        m_pango_text += Glib::Markup::escape_text( esc_text );
-        pos_regular = m_pos_current + 1;
-    }
-    m_pango_text += "</big>";
+    apply_format( RID_SUBSUBHEADING,
+                  m_recipe_curr->m_pos_start, get_para_end( m_recipe_curr->m_pos_start ) );
 }
 
 void
 EntryParserPango::apply_bold()
 {
-    apply_markup( "<b>", "</b>", 1, -1 );
+    apply_markup( RID_BOLD );
 }
 
 void
 EntryParserPango::apply_italic()
 {
-    apply_markup( "<i>", "</i>", 1, -1 );
+    apply_markup( RID_ITALIC );
 }
 
 void
 EntryParserPango::apply_strikethrough()
 {
-    apply_markup( "<s>", "</s>", 1, -1 );
+    apply_markup( RID_STRIKETHROUGH );
 }
 
 void
 EntryParserPango::apply_highlight()
 {
-    apply_markup( "<span bgcolor='yellow'>", "</span>", 1, -1 );
+    apply_markup( RID_HIGHLIGHT );
 }
 
 void
 EntryParserPango::apply_comment()
 {
-    if( option_hide_comments )
-        begin_apply();  // ignores comment
-    else
-        apply_markup( "<small><sup>", "</sup></small>", 1, -1 );
+    apply_markup( m_pango_opt_hide_comments ? RID_MARKUP : RID_COMMENT );
 }
 
 void
 EntryParserPango::apply_check_ccl()
 {
-    const Glib::ustring esc_text(
-            m_ptr2entry->get_text()->substr( pos_regular, m_pos_current - pos_regular - 3 ) );
-
-    m_pango_text += Glib::Markup::escape_text( esc_text );
-    m_pango_text += "<span font=\"Monospace\">";
-    m_pango_text += m_ptr2entry->get_text()->substr( m_pos_current - 3, 3 );
-    m_pango_text += "</span>";
-    pos_regular = m_pos_current + 1;
-
-    m_pango_text += " <s>";
-}
-
-void
-EntryParserPango::apply_check_ccl_end()
-{
-    if( pos_regular < m_pos_current )
-    {
-        const Glib::ustring esc_text(
-                m_ptr2entry->get_text()->substr( pos_regular, m_pos_current - pos_regular ) );
-        m_pango_text += Glib::Markup::escape_text( esc_text );
-        pos_regular = m_pos_current;
-    }
-    m_pango_text += "</s>";
+    apply_format( RID_MONOSPACE, m_recipe_curr->m_pos_start, m_pos_curr );
+    apply_format( RID_STRIKETHROUGH, m_pos_curr, get_para_end( m_pos_curr ) );
 }
 
 void
 EntryParserPango::apply_check_unf()
 {
-    const Glib::ustring esc_text(
-            m_ptr2entry->get_text()->substr( pos_regular, m_pos_current - pos_regular - 3 ) );
-    m_pango_text += Glib::Markup::escape_text( esc_text );
-    m_pango_text += "<span font=\"Monospace\">";
-    m_pango_text += m_ptr2entry->get_text()->substr( m_pos_current - 3, 3 );
-    m_pango_text += "</span>";
-    pos_regular = m_pos_current + 1;
+    apply_format( RID_MONOSPACE, m_recipe_curr->m_pos_start, m_pos_curr );
 }
 
 void
 EntryParserPango::apply_check_prg()
 {
-    const Glib::ustring esc_text(
-            m_ptr2entry->get_text()->substr( pos_regular, m_pos_current - pos_regular - 3 ) );
-    m_pango_text += Glib::Markup::escape_text( esc_text );
-    m_pango_text += "<span font=\"Monospace\">";
-    m_pango_text += m_ptr2entry->get_text()->substr( m_pos_current - 3, 3 );
-    m_pango_text += "</span>";
-    pos_regular = m_pos_current + 1;
+    apply_check_unf();
 }
 
 void
 EntryParserPango::apply_check_fin()
 {
-    const Glib::ustring esc_text(
-            m_ptr2entry->get_text()->substr( pos_regular, m_pos_current - pos_regular - 3 ) );
-    m_pango_text += Glib::Markup::escape_text( esc_text );
-    m_pango_text += "<span font=\"Monospace\">";
-    m_pango_text += m_ptr2entry->get_text()->substr( m_pos_current - 3, 3 );
-    m_pango_text += "</span>";
-    pos_regular = m_pos_current + 1;
-
-    m_pango_text += " <span bgcolor='yellow'>";
-}
-
-void
-EntryParserPango::apply_check_fin_end()
-{
-    if( pos_regular < m_pos_current )
-    {
-        const Glib::ustring esc_text(
-                m_ptr2entry->get_text()->substr( pos_regular, m_pos_current - pos_regular ) );
-        m_pango_text += Glib::Markup::escape_text( esc_text );
-        pos_regular = m_pos_current;
-    }
-    m_pango_text += "</span>";
+    apply_format( RID_MONOSPACE|RID_DONE, m_recipe_curr->m_pos_start, m_pos_curr );
+    apply_format( RID_DONE, m_pos_curr, get_para_end( m_pos_curr ) ); // TODO change color
 }
 
 void
 EntryParserPango::apply_link()
 {
-    int offset1( 0 ), offset2( 1 );
-    bool flag_img( false );
+    if( m_link_type_last & RID_DATE )
+        return;
 
-    if( m_flag_hidden_link )
-    {
-        offset1 = pos_tab - m_pos_start + 1;
-        offset2 = -offset1;
-    }
-    else if( m_pos_start > 0 )   // first line is reserved for title
+    auto begin{ m_recipe_curr->m_pos_start };
+    Glib::RefPtr< Gdk::Pixbuf > pixbuf;
+    bool flag_ends_para{ m_pos_curr == m_ptr2entry->get_text()->length() };
+    if( !flag_ends_para )
+        flag_ends_para = ( get_char_at( m_pos_curr ) == '\n' );
+
+    // there can be no text on the same line as the image:
+    if( begin > 0 && get_char_at( begin - 1 ) == '\n' && flag_ends_para )
     {
+        std::string link{ m_ptr2entry->get_text()->substr( begin, m_pos_curr - begin ) };
+
         // HANDLE RELATIVE LINKS
-        if( m_word_last.find( "rel://" ) == 0 )
-            m_word_last.replace( 0, 5, "file://" + Glib::path_get_dirname( Diary::d->get_path() ) );
+        if( link.find( "rel://" ) == 0 )
+            link.replace( 0, 5, "file://" + Glib::path_get_dirname( Diary::d->get_path() ) );
 
-        if( m_word_last.find( "file:///" ) == 0 &&
-            // there can be no text on the same line as the image:
-            get_char_at( m_pos_start - 1 ) == '\n' && m_char_current == '\n' )
+        if( link.find( "file:///" ) == 0 )
         {
             try
             {
-                m_pango_pixbuf = Lifeograph::get_thumbnail( Glib::filename_from_uri( m_word_last ),
-                                                            m_max_thumbnail_width );
-                m_pango_return = Printable::PT_IMAGE;
-                m_pango_begin = m_pos_current;
-
-                begin_apply();  // calls apply_regular()
-
-                m_pos_end = 0;  // force parsing loop to finish
-
-                flag_img = true;
+                pixbuf = Lifeograph::get_thumbnail( Glib::filename_from_uri( link ),
+                                                    m_max_thumbnail_width );
+                PRINT_DEBUG( "pixbuf loaded" );
             }
-            catch( Glib::FileError &er )
+            catch( ... )
             {
-                print_error( "Link target not found" );
-            }
-            catch( Gdk::PixbufError &er )
-            {
-                PRINT_DEBUG( "Link is not an image" );
+                PRINT_DEBUG( "Could not load image" );
             }
         }
     }
 
-    if( !flag_img )
-        apply_markup( "<span color='blue' underline='single'>", "</span>", offset1, offset2 );
+    if( pixbuf )
+        apply_format( RID_MARKUP|RID_IMAGE, begin, m_pos_curr );
+    else
+        apply_format( RID_LINK, begin, m_pos_curr );
 }
 
-Wchar
+void
+EntryParserPango::apply_link_hidden( )
+{
+    auto pos_label( m_recipe_curr->m_pos_middle + 1 );
+
+    apply_format( RID_MARKUP, m_recipe_curr->m_pos_start, pos_label );
+    apply_format( RID_MARKUP, m_pos_curr, m_pos_curr + 1 );
+    apply_format( RID_LINK, pos_label, m_pos_curr );
+}
+
+void
+EntryParserPango::apply_chart()
+{
+    apply_format( RID_CHART, m_recipe_curr->m_pos_start, m_pos_curr );
+}
+
+inline Wchar
 EntryParserPango::get_char_at( int i )
 {
     return m_ptr2entry->get_text()->at( i );
 }
+
+inline UstringSize
+EntryParserPango::get_para_end( UstringSize begin )
+{
+    auto end{ m_ptr2entry->get_text()->find_first_of( '\n', begin ) };
+    return( end == Ustring::npos ? end = m_ptr2entry->get_text()->length() : end );
+}
+
+inline void
+EntryParserPango::apply_format( Recipe::Id format, UstringSize begin, UstringSize end )
+{
+    for( auto i = begin; i< end; i++ )
+        m_format_map[ i ] |= format;
+}
+
+inline UstringSize
+EntryParserPango::get_format_string_size( UstringSize begin, Recipe::Id format )
+{
+    UstringSize size{ 0 };
+    for( ; ( begin + size ) < m_format_map.size(); size++ )
+        if( not( m_format_map[ begin + size ] & format ) )
+            break;
+
+    return size;
+}
