@@ -21,14 +21,10 @@
 
 
 #include "settings.hpp"
+#include "lifeograph.hpp"
 
 #include <fstream>
-#ifndef LIFEO_WINDOZE
 #include <glib/gstdio.h>    // g_mkdir()
-#else
-#include <shlwapi.h>
-#include <unistd.h>
-#endif
 
 
 using namespace LIFEO;
@@ -44,19 +40,21 @@ Settings::read()
 
     if( m_path.empty() )
     {
-        flag_maiden_voyage = true; // maiden voyage is not used when custom config path is given
         m_path = Glib::build_filename( Glib::get_user_config_dir(), "lifeograph", "lifeo.conf" );
     }
 
     std::ifstream file( PATH( m_path ).c_str() );
 
-    if( ! file.is_open() )
+    if( ! file.is_open() ) // maiden voyage
     {
-        PRINT_DEBUG( "Configuration file does not exist!" );
+        PRINT_DEBUG( "Configuration file does not exist, maiden voyage!" );
+        Lifeograph::p->m_flag_force_welcome = true;
+        recentfiles.insert( Lifeograph::EXAMPLEDIARYPATH );
+
         return false;
     }
     else
-        flag_maiden_voyage = false;
+        Lifeograph::p->m_flag_force_welcome = false;
 
     reset();
 
@@ -72,6 +70,8 @@ Settings::read()
             case 'D':
                 if( line[1] == 'o' )
                     date_format_order = convert_string( line.substr( 2 ) );
+                else if( line[1] == 'a' )
+                    use_dark_theme = true;
                 else
                     date_format_separator = convert_string( line.substr( 2 ) );
                 break;
@@ -147,13 +147,11 @@ Settings::read()
 bool
 Settings::write()
 {
-#ifndef LIFEO_WINDOZE
     if( access( PATH( m_path ).c_str(), F_OK ) != 0 )
     {
         if( ! Glib::file_test( Glib::path_get_dirname( m_path ), Glib::FILE_TEST_IS_DIR ) )
             g_mkdir_with_parents( Glib::path_get_dirname( m_path ).c_str(), 0700 );
     }
-#endif
 
     std::ofstream file( PATH( m_path ).c_str(), std::ios::out | std::ios::trunc );
 
@@ -185,6 +183,9 @@ Settings::write()
     if( diary_extension != EXTENSION_DEFAULT )
         file << "\nE " << diary_extension;
 
+    if( use_dark_theme )
+        file << "\nDark Theme";
+
     if( ! icon_theme.empty() )
         file << "\nT " << icon_theme;
 
