@@ -1,6 +1,6 @@
 /***********************************************************************************
 
-    Copyright (C) 2007-2018 Ahmet Öztürk (aoz_2@yahoo.com)
+    Copyright (C) 2007-2019 Ahmet Öztürk (aoz_2@yahoo.com)
 
     This file is part of Lifeograph.
 
@@ -260,10 +260,11 @@ EntryView::EntryView()
 void
 EntryView::sync()
 {
-    if( m_flag_entrychanged )
+    if( m_flag_entrychanged && m_entry_sync )
     {
         m_entry_sync->set_date_edited( time( nullptr ) );
-        //shouln't be needed any longer: m_entry_sync->set_text( m_textview->m_buffer->get_text() );
+        m_entry_sync->set_text( m_textview->m_buffer->get_text_to_save() );
+
         m_flag_entrychanged = false;
         PRINT_DEBUG( "entry synced" );
     }
@@ -292,24 +293,47 @@ EntryView::handle_login()
 {
     m_textview->m_buffer->handle_login();
 
-    // READ ONLY
-    bool editable( ! Diary::d->is_read_only() );
-    m_textview->set_editable( editable );
-    m_W_tag_list->set_editable( editable );
-    m_RB_not_todo->set_visible( editable );
-    m_RB_todo->set_visible( editable );
-    m_RB_progressed->set_visible( editable );
-    m_RB_done->set_visible( editable );
-    m_RB_canceled->set_visible( editable );
-    //m_menuseparator_4->set_visible( editable );
-    m_TB_toggle_favorite->set_visible( editable );
-    //m_menuseparator_1->set_visible( editable );
-    //m_menuseparator_2->set_visible( editable );
-    m_B_trash->set_visible( editable );
-    m_B_restore->set_visible( editable );
-    m_B_dismiss->set_visible( editable );
-    m_Bx_editing_toolbar->set_visible( Lifeograph::settings.show_formatting_toolbar &&
-                                       editable );
+    m_textview->set_editable( false );
+    m_W_tag_list->set_editable( false );
+    m_RB_not_todo->set_visible( false );
+    m_RB_todo->set_visible( false );
+    m_RB_progressed->set_visible( false );
+    m_RB_done->set_visible( false );
+    m_RB_canceled->set_visible( false );
+    m_TB_toggle_favorite->set_visible( false );
+    m_B_trash->set_visible( false );
+    m_B_restore->set_visible( false );
+    m_B_dismiss->set_visible( false );
+    m_Bx_editing_toolbar->set_visible( false );
+
+    WidgetEntryTags::handle_gtk_theme_changed();
+}
+
+void
+EntryView::handle_edit_enabled()
+{
+    m_textview->m_buffer->handle_edit_enabled();
+
+    m_textview->set_editable( true );
+    m_W_tag_list->set_editable( true );
+    m_RB_not_todo->set_visible( true );
+    m_RB_todo->set_visible( true );
+    m_RB_progressed->set_visible( true );
+    m_RB_done->set_visible( true );
+    m_RB_canceled->set_visible( true );
+    m_TB_toggle_favorite->set_visible( true );
+    m_Bx_editing_toolbar->set_visible( Lifeograph::settings.show_formatting_toolbar );
+
+    if( AppWindow::p->panel_main->is_cur_elem( m_ptr2elem ) )
+    {
+        if( m_ptr2elem->is_trashed() )
+        {
+            m_B_restore->set_visible( true );
+            m_B_dismiss->set_visible( true );
+        }
+        else
+            m_B_trash->set_visible( true );
+    }
 }
 
 void
@@ -327,10 +351,6 @@ EntryView::handle_textview_changed()
     m_flag_entrychanged = true;
 
     PRINT_DEBUG( "entry changed" );
-
-    // TODO:
-    //if( m_hbox_tagtools->is_visible() )
-        //m_hbox_tagtools->hide();
 }
 
 void
@@ -341,22 +361,24 @@ EntryView::handle_drag_data_received(
                 guint info,
                 guint time )
 {
-    if( Diary::d->is_read_only() )
+    if( Diary::d->is_editing_enabled() == false )
         return;
 
     context->drag_finish( true, false, time );
 
-    if( Lifeograph::s_elem_dragged == nullptr )
+    if( Lifeograph::is_drag_in_progress() == false )
         return;
 
-    switch( Lifeograph::s_elem_dragged->get_type() )
+    DiaryElement* elem{ Lifeograph::get_dragged_elem() };
+
+    switch( elem->get_type() )
     {
         case DiaryElement::ET_ENTRY:
-            m_textview->m_buffer->insert_link( Lifeograph::s_elem_dragged );
+            m_textview->m_buffer->insert_link( elem );
             break;
         case DiaryElement::ET_TAG:
         {
-            Tag* tag = dynamic_cast< Tag* >( Lifeograph::s_elem_dragged );
+            Tag* tag = dynamic_cast< Tag* >( elem );
             if( m_ptr2elem->add_tag( tag ) ) // if does not already have the tag
             {
                 m_W_tag_list->redraw();
@@ -409,7 +431,7 @@ void
 set_entry_todo_status( Entry* e, ElemStatus status )
 {
     if( status == ES::NOT_TODO )
-        e->calculate_todo_status();
+        e->update_todo_status( "" );
     else
     {
         e->set_todo_status( status );
@@ -466,24 +488,21 @@ EntryView::hide()
     AppWindow::p->panel_main->hide_popover();
 
     Diary::d->get_filter()->add_entry( m_ptr2elem );
-    AppWindow::p->panel_diary->update_calendar();
-    AppWindow::p->panel_diary->update_entry_list();
+    AppWindow::p->panel_diary->handle_filter_changed();
 }
 
 void
 EntryView::hide_before()
 {
     Diary::d->set_filter_date_begin( m_ptr2elem->get_date_t() );
-    AppWindow::p->panel_diary->update_calendar();
-    AppWindow::p->panel_diary->update_entry_list();
+    AppWindow::p->panel_diary->handle_filter_changed();
 }
 
 void
 EntryView::hide_after()
 {
     Diary::d->set_filter_date_end( m_ptr2elem->get_date_t() );
-    AppWindow::p->panel_diary->update_calendar();
-    AppWindow::p->panel_diary->update_entry_list();
+    AppWindow::p->panel_diary->handle_filter_changed();
 }
 
 void
@@ -559,11 +578,15 @@ EntryView::dismiss_entry()
     // or a segfault will ensue:
     m_W_tag_list->set_entry( nullptr );
 
+    Diary::d->show();
+
     Lifeograph::s_internaloperation++;
 
     AppWindow::p->panel_main->remove_element_from_history( m_ptr2elem );
     Diary::d->dismiss_entry( m_ptr2elem );
 
+    m_ptr2elem = nullptr;
+
     AppWindow::p->panel_diary->update_calendar();
     AppWindow::p->panel_diary->update_entry_list();
 
@@ -626,7 +649,7 @@ EntryView::show( Entry& entry )
 
     Lifeograph::s_internaloperation++;
 
-    if( ! Diary::d->is_read_only() )
+    if( Diary::d->is_editing_enabled() )
     {
         m_TB_toggle_favorite->set_active( entry.is_favored() );
         m_B_trash->set_visible( !entry.is_trashed() );
@@ -677,13 +700,14 @@ EntryView::prepare_for_hiding( Entry& entry )
     sync();
     m_W_tag_list->clear();
     PRINT_DEBUG( "EntryView::prepare_for_hiding()" );
+    m_entry_sync = nullptr;
 }
 
 void
 EntryView::update_formatting_toolbar()
 {
     m_Bx_editing_toolbar->set_visible( Lifeograph::settings.show_formatting_toolbar &&
-                                       Diary::d->is_read_only() == false );
+                                       Diary::d->is_editing_enabled() );
 }
 
 void
