@@ -40,8 +40,7 @@ std::string         ViewLogin::m_path_cur;
 
 ViewLogin::ViewLogin()
 {
-    Gtk::ModelButton* B_new_diary;
-    Gtk::ModelButton* B_browse_diary;
+    Gtk::TreeView::Column* col_icon;
     Gtk::TreeView::Column* col_diary;
     Gtk::TreeView::Column* col_date;
 
@@ -50,24 +49,19 @@ ViewLogin::ViewLogin()
         colrec = new Colrec;
 
         auto builder{ Lifeograph::get_builder() };
-        builder->get_widget( "G_login", m_G_login );
+        builder->get_widget( "Bx_login", m_Bx_login );
         builder->get_widget( "TV_diaries", m_TV_diaries );
-        builder->get_widget( "MB_login_add", m_B_add );
-        builder->get_widget( "B_login_new_diary", B_new_diary );
-        builder->get_widget( "B_login_open_diary", B_browse_diary );
-        builder->get_widget( "B_login_edit", m_B_edit_diary );
-        builder->get_widget( "B_login_read", m_B_read_diary );
-        builder->get_widget( "B_login_remove_diary", m_B_remove_diary );
-        builder->get_widget( "IB_login", m_IB_login );
-        builder->get_widget( "L_login_infobar", m_L_info );
-        builder->get_widget( "B_login_infobar", m_B_info );
-        builder->get_widget( "Po_login_add", m_Po_add );
+        builder->get_widget( "B_login_select", m_B_select );
+        builder->get_widget( "B_login_new_diary", m_B_new );
+        builder->get_widget( "B_login_open_diary", m_B_open );
+        builder->get_widget( "B_login_remove", m_B_remove_diary );
 
+        col_icon = Gtk::manage( new Gtk::TreeView::Column( "", colrec->icon ) );
         col_diary = Gtk::manage( new Gtk::TreeView::Column( _( "Name" ), colrec->name ) );
         col_date = Gtk::manage( new Gtk::TreeView::Column( _( STRING::COLHEAD_BY_LAST_SAVE ),
                                                            colrec->date ) );
 
-        m_treestore_diaries = Gtk::ListStore::create( *colrec );
+        m_LS_diaries = Gtk::ListStore::create( *colrec );
         m_treesel_diaries = m_TV_diaries->get_selection();
     }
     catch( ... )
@@ -75,27 +69,55 @@ ViewLogin::ViewLogin()
         throw LIFEO::Error( "Failed to create the login view" );
     }
 
-    m_G_login->remove( *m_IB_login ); // necessary due to GNOME Bugzilla bug 710888
-
     // LIST OF DIARIES
-    m_TV_diaries->set_model( m_treestore_diaries );
+    m_TV_diaries->set_model( m_LS_diaries );
     col_diary->set_expand( true );
     col_diary->set_clickable( true );
     col_date->set_clickable( true );
+    m_TV_diaries->append_column( *col_icon );
     m_TV_diaries->append_column( *col_diary );
     m_TV_diaries->append_column( *col_date );
-    m_treestore_diaries->set_default_sort_func(
+    m_LS_diaries->set_default_sort_func(
             sigc::mem_fun( this, &ViewLogin::sort_by_date_func ) );
-    m_treestore_diaries->set_sort_column( Gtk::ListStore::DEFAULT_SORT_COLUMN_ID,
+    m_LS_diaries->set_sort_column( Gtk::ListStore::DEFAULT_SORT_COLUMN_ID,
                                           Gtk::SORT_DESCENDING );
     m_TV_diaries->set_has_tooltip( true );
     m_TV_diaries->drag_dest_set( Gtk::DEST_DEFAULT_ALL, Gdk::ACTION_COPY );
     m_TV_diaries->drag_dest_add_uri_targets();
 
-    m_B_edit_diary->signal_clicked().connect(
-            sigc::mem_fun( this, &ViewLogin::handle_button_edit ) );
-    m_B_read_diary->signal_clicked().connect(
-            sigc::mem_fun( this, &ViewLogin::handle_button_read ) );
+    // STYLE
+    Glib::RefPtr< Gtk::CssProvider > css = Gtk::CssProvider::create();
+    if( css->load_from_data(
+            "treeview#diary_list { background-color: transparent } "
+            "treeview#diary_list:selected { color: #bb3333; background-color: #dddddd } "
+            "treeview#diary_list button { background: transparent } " ) )
+    Gtk::StyleContext::add_provider_for_screen(
+            Gdk::Screen::get_default(), css, GTK_STYLE_PROVIDER_PRIORITY_USER );
+
+    m_TV_diaries->get_column( 1 )->set_cell_data_func(
+            * m_TV_diaries->get_column_cell_renderer( 1 ),
+            [ this ]( Gtk::CellRenderer* cell, const Gtk::TreeModel::iterator& iter )
+            {
+                std::string path{ ( * iter )[ colrec->path ] };
+                Gtk::CellRendererText* crt{ dynamic_cast< Gtk::CellRendererText* >( cell ) };
+
+                if( path == m_path_cur )
+                {
+                    cell->property_cell_background_rgba() = Gdk::RGBA( "#dddddd" );
+                    crt->property_weight() = Pango::WEIGHT_BOLD;
+                    crt->property_foreground_rgba() = Gdk::RGBA( "#bb3333" );
+                }
+                else
+                {
+                    cell->property_cell_background_rgba() = Gdk::RGBA( "transparent" );
+                    crt->property_weight() = Pango::WEIGHT_NORMAL;
+                    crt->property_foreground_set() = false;
+                }
+            } );
+
+    // SIGNALS
+    m_B_select->signal_toggled().connect(
+            sigc::mem_fun( this, &ViewLogin::handle_select_mode_toggled ) );
 
     m_B_remove_diary->signal_clicked().connect(
             sigc::mem_fun( this, &ViewLogin::remove_selected_diary ) );
@@ -114,94 +136,77 @@ ViewLogin::ViewLogin()
     m_TV_diaries->signal_drag_data_received().connect(
             sigc::mem_fun( this, &ViewLogin::handle_dragged_file ) );
 
-    B_new_diary->signal_clicked().connect( sigc::mem_fun( this, &ViewLogin::create_new_diary ) );
-    B_browse_diary->signal_clicked().connect(
+    if( Lifeograph::p->m_flag_read_only )
+        m_B_new->set_visible( false );
+    else
+        m_B_new->signal_clicked().connect(
+                sigc::mem_fun( this, &ViewLogin::create_new_diary ) );
+    m_B_open->signal_clicked().connect(
             sigc::mem_fun( this, &ViewLogin::add_existing_diary ) );
 
     col_diary->signal_clicked().connect( sigc::mem_fun( this, &ViewLogin::sort_by_name ) );
     col_date->signal_clicked().connect( sigc::mem_fun( this, &ViewLogin::sort_by_date ) );
+}
 
-    m_IB_login->signal_response().connect(
-            sigc::mem_fun( this, &ViewLogin::handle_infobar_response ) );
+void
+ViewLogin::handle_start()
+{
+    // ICONS
+    std::string icon_dir{ Lifeograph::get_icon_dir() };
+    Lifeograph::icons->diary_16 =
+            Gdk::Pixbuf::create_from_file( icon_dir + "/diary-16.png" );
+    Lifeograph::icons->diary_32 =
+            Gdk::Pixbuf::create_from_file( icon_dir + "/diary-32.png" );
 
-    // DETECT STOCK DIARIES
-    Gio::init();
-    try
+    if( Lifeograph::p->m_flag_force_welcome )
     {
-        Glib::RefPtr< Gio::File > directory = Gio::File::create_for_path( Lifeograph::DIARYDIR );
-        if( directory )
-        {
-            Glib::RefPtr< Gio::FileEnumerator > enumerator = directory->enumerate_children();
-            if( enumerator )
-            {
-                Glib::RefPtr< Gio::FileInfo > file_info = enumerator->next_file();
-                while( file_info )
-                {
-                    if( file_info->get_file_type() != Gio::FILE_TYPE_DIRECTORY )
-                    {
-                        std::string path( Lifeograph::DIARYDIR + "/" );
-                        path += file_info->get_name();
-                        Lifeograph::stock_diaries.insert( path );
-                    }
-                    file_info = enumerator->next_file();
-                }
-            }
-        }
-    }
-    catch( const Glib::Exception& ex )
-    {
-        LIFEO::Error( ex.what() );
+        Gtk::Button* B_new_diary;
+        Lifeograph::get_builder2()->get_widget( "G_welcome", m_G_welcome );
+        Lifeograph::get_builder2()->get_widget( "B_welcome_new_diary", B_new_diary );
+
+        m_Bx_login->pack_start( *m_G_welcome, Gtk::PACK_SHRINK );
+
+        B_new_diary->signal_clicked().connect(
+                sigc::mem_fun( this, &ViewLogin::create_new_diary ) );
     }
+
+    if( Lifeograph::p->m_flag_open_directly )
+        open_selected_diary();
+
+    if( Lifeograph::loginstatus != Lifeograph::LOGGED_IN ) // i.e. open directly was not successful
+        initialize();
 }
 
 void
 ViewLogin::initialize()
 {
     AppWindow::p->set_title( PROGRAM_NAME );
-    m_B_add->set_visible( true );
-    AppWindow::p->set_default( *m_B_edit_diary );
-
-    if( Lifeograph::p->m_flag_open_directly )
-        if( open_selected_diary( Lifeograph::p->m_flag_read_only ) == LIFEO::SUCCESS )
-            return;
+    m_B_new->set_visible( Lifeograph::p->m_flag_read_only == false );
+    m_B_open->set_visible( Lifeograph::p->m_flag_read_only == false );
+    m_B_select->set_visible( Lifeograph::p->m_flag_read_only == false );
 
     populate_diaries();
 }
 
-void
-ViewLogin::handle_start()
-{
-    initialize();
-
-    if( Lifeograph::settings.flag_maiden_voyage || Lifeograph::p->m_flag_force_welcome )
-    {
-        Gtk::Grid* grid_welcome;
-        Gtk::Button* button_new_diary;
-        Lifeograph::get_builder2()->get_widget( "G_welcome", grid_welcome );
-        Lifeograph::get_builder2()->get_widget( "B_welcome_new_diary", button_new_diary );
-
-        m_G_login->insert_row( 1 );
-        m_G_login->attach( *grid_welcome, 0, 1, 1, 1 );
-
-        button_new_diary->signal_clicked().connect(
-                sigc::mem_fun( this, &ViewLogin::create_new_diary ) );
-    }
-}
-
 void
 ViewLogin::handle_login()
 {
-    m_B_add->set_visible( false );
+    m_B_select->set_active( false );
+    m_B_select->set_visible( false );
+    m_B_new->set_visible( false );
+    m_B_open->set_visible( false );
 }
 
 void
 ViewLogin::handle_logout()
 {
     initialize();
-    if( Lifeograph::loginstatus == Lifeograph::LOGGED_TIME_OUT )
-        show_info( Gtk::MESSAGE_INFO, _( STRING::ENTER_PASSWORD_TIMEOUT ) );
-    else if( m_flag_info_is_visible )
-        hide_infobar();
+
+    if( Lifeograph::p->m_flag_force_welcome )
+    {
+        m_Bx_login->remove( *m_G_welcome );
+        Lifeograph::p->m_flag_force_welcome = false;
+    }
 }
 
 void
@@ -211,12 +216,13 @@ ViewLogin::populate_diaries()
 
     struct stat fst;    // file date stat
 
-    m_treestore_diaries->clear();
+    m_LS_diaries->clear();
     for( ListPaths::reverse_iterator iter = Lifeograph::settings.recentfiles.rbegin();
          iter != Lifeograph::settings.recentfiles.rend();
          ++iter )
     {
-        row = *( m_treestore_diaries->append() );
+        row = *( m_LS_diaries->append() );
+        row[ colrec->icon ] = Lifeograph::icons->diary_32;
         row[ colrec->name ] = Glib::filename_display_basename( *iter );
         row[ colrec->path ] = *iter;
 
@@ -233,24 +239,8 @@ ViewLogin::populate_diaries()
         }
     }
 
-    // STOCK DIARIES
-    for( ListPaths::iterator itr_diary = Lifeograph::stock_diaries.begin();
-         itr_diary != Lifeograph::stock_diaries.end();
-         ++itr_diary )
-    {
-        std::string path( *itr_diary );
-        row = *( m_treestore_diaries->append() );
-        row[ colrec->name ] = "[*] " + Glib::filename_display_basename( path );
-        row[ colrec->path ] = path;
-
-        stat( path.c_str(), &fst );
-        time_t file_time( m_sort_date_type == SDT_SAVE ? fst.st_mtime : fst.st_atime );
-        row[ colrec->date ] = Date::format_string_dt( file_time );
-        row[ colrec->date_sort ] = file_time;
-    }
-
-    if( m_treestore_diaries->children().size() > 0 )
-        m_treesel_diaries->select( m_treestore_diaries->get_iter( "0" ) );
+    if( m_LS_diaries->children().size() > 0 )
+        m_treesel_diaries->select( m_LS_diaries->get_iter( "0" ) );
 }
 
 void
@@ -259,8 +249,8 @@ ViewLogin::sort_by_date()
     if( m_sort_date_type == SDT_NAME )
     {
         m_sort_date_type = SDT_SAVE;
-        m_TV_diaries->get_column( 1 )->set_title( _( STRING::COLHEAD_BY_LAST_SAVE ) );
-        m_treestore_diaries->set_default_sort_func(
+        m_TV_diaries->get_column( 2 )->set_title( _( STRING::COLHEAD_BY_LAST_SAVE ) );
+        m_LS_diaries->set_default_sort_func(
                 sigc::mem_fun( this, &ViewLogin::sort_by_date_func ) );
         PRINT_DEBUG( "sort by date" );
     }
@@ -269,12 +259,12 @@ ViewLogin::sort_by_date()
         if( m_sort_date_type == SDT_SAVE )
         {
             m_sort_date_type = SDT_ACCESS;
-            m_TV_diaries->get_column( 1 )->set_title( _( STRING::COLHEAD_BY_LAST_ACCESS ) );
+            m_TV_diaries->get_column( 2 )->set_title( _( STRING::COLHEAD_BY_LAST_ACCESS ) );
         }
         else
         {
             m_sort_date_type = SDT_SAVE;
-            m_TV_diaries->get_column( 1 )->set_title( _( STRING::COLHEAD_BY_LAST_SAVE ) );
+            m_TV_diaries->get_column( 2 )->set_title( _( STRING::COLHEAD_BY_LAST_SAVE ) );
         }
     }
     populate_diaries();
@@ -285,7 +275,7 @@ ViewLogin::sort_by_name()
 {
     m_sort_date_type = SDT_NAME;
     //m_treestore_diaries->set_sort_column( 0, Gtk::SORT_ASCENDING );
-    m_treestore_diaries->set_default_sort_func(
+    m_LS_diaries->set_default_sort_func(
             sigc::mem_fun( this, &ViewLogin::sort_by_name_func ) );
     PRINT_DEBUG( "sort by name" );
     populate_diaries();
@@ -329,57 +319,47 @@ ViewLogin::sort_by_name_func( const Gtk::TreeModel::iterator& itr1,
         return 0;
 }
 
-LIFEO::Result
-ViewLogin::open_selected_diary( bool read_only )
+void
+ViewLogin::open_selected_diary()
 {
-    // BEWARE: clear the diary before returning any result but SUCCESS
+    // BEWARE: clear the diary before returning unless successful
 
     // SET PATH
     Result result( Diary::d->set_path( m_path_cur,
-                                       read_only ? Diary::SPT_READ_ONLY : Diary::SPT_NORMAL ) );
+                                       Lifeograph::p->m_flag_read_only ?
+                                            Diary::SPT_READ_ONLY : Diary::SPT_NORMAL) );
     switch( result )
     {
         case LIFEO::SUCCESS:
             break;
         case FILE_NOT_FOUND:
-            show_info( Gtk::MESSAGE_INFO, _( STRING::DIARY_NOT_FOUND ) );
+            AppWindow::p->show_info( Gtk::MESSAGE_INFO, _( STRING::DIARY_NOT_FOUND ) );
             break;
         case FILE_NOT_READABLE:
-            show_info( Gtk::MESSAGE_INFO, _( STRING::DIARY_NOT_READABLE ) );
-            break;
-        case FILE_LOCKED:
-            show_info( Gtk::MESSAGE_INFO,
-                       Glib::ustring::compose( _( STRING::DIARY_LOCKED ),
-                               Glib::ustring::compose( "<a href=\"file://%1\" title=\"%2\">%3</a>",
-                                                       Glib::path_get_dirname( m_path_cur ),
-                                                       _( "Click to open the containing folder" ),
-                                                       m_path_cur + LOCK_SUFFIX ) ) );
-            m_B_edit_diary->set_visible( false );
+            AppWindow::p->show_info( Gtk::MESSAGE_INFO, _( STRING::DIARY_NOT_READABLE ) );
             break;
         default:
-            show_info( Gtk::MESSAGE_ERROR, _( STRING::FAILED_TO_OPEN_DIARY ) );
+            AppWindow::p->show_info( Gtk::MESSAGE_ERROR, _( STRING::FAILED_TO_OPEN_DIARY ) );
             break;
     }
     if( result != LIFEO::SUCCESS )
     {
         Diary::d->clear();
-        return result;
+        populate_diaries();
+        return;
     }
 
     // FORCE ACCESS TIME UPDATE
     // for performance reasons atime update policy on linux is usually once per day. see: relatime
 #ifndef _WIN32
-    if( read_only )
-    {
-        struct stat fst;    // file date stat
-        struct timeval tvs[2];
-        stat( m_path_cur.c_str(), &fst );
-        tvs[ 0 ].tv_sec = time( NULL );
-        tvs[ 0 ].tv_usec = 0;
-        tvs[ 1 ].tv_sec = fst.st_mtime;
-        tvs[ 1 ].tv_usec = 0;
-        utimes( m_path_cur.c_str(), tvs );
-    }
+    struct stat fst;    // file date stat
+    struct timeval tvs[2];
+    stat( m_path_cur.c_str(), &fst );
+    tvs[ 0 ].tv_sec = time( NULL );
+    tvs[ 0 ].tv_usec = 0;
+    tvs[ 1 ].tv_sec = fst.st_mtime;
+    tvs[ 1 ].tv_usec = 0;
+    utimes( m_path_cur.c_str(), tvs );
 #endif
 
     // READ HEADER
@@ -388,95 +368,103 @@ ViewLogin::open_selected_diary( bool read_only )
         case LIFEO::SUCCESS:
             break;
         case INCOMPATIBLE_FILE_OLD:
-            show_info( Gtk::MESSAGE_ERROR, _( STRING::INCOMPATIBLE_DIARY_OLD ) );
+            AppWindow::p->show_info( Gtk::MESSAGE_ERROR, _( STRING::INCOMPATIBLE_DIARY_OLD ) );
             break;
         case INCOMPATIBLE_FILE_NEW:
-            show_info( Gtk::MESSAGE_ERROR, _( STRING::INCOMPATIBLE_DIARY_NEW ) );
+            AppWindow::p->show_info( Gtk::MESSAGE_ERROR, _( STRING::INCOMPATIBLE_DIARY_NEW ) );
             break;
         case CORRUPT_FILE:
-            show_info( Gtk::MESSAGE_ERROR, _( STRING::CORRUPT_DIARY ) );
+            AppWindow::p->show_info( Gtk::MESSAGE_ERROR, _( STRING::CORRUPT_DIARY ) );
             break;
         default:
-            show_info( Gtk::MESSAGE_ERROR, _( STRING::FAILED_TO_OPEN_DIARY ) );
+            AppWindow::p->show_info( Gtk::MESSAGE_ERROR, _( STRING::FAILED_TO_OPEN_DIARY ) );
             break;
     }
     if( result != LIFEO::SUCCESS )
     {
-        return result;
-    }
-
-    // HANDLE OLD DIARY
-    if( Diary::d->is_old() && read_only == false && m_password_attempt_no == 0 )
-    {
-        Gtk::MessageDialog* messagedialog
-                = new Gtk::MessageDialog( *AppWindow::p,
-                                          "",
-                                          false,
-                                          Gtk::MESSAGE_WARNING,
-                                          Gtk::BUTTONS_CANCEL,
-                                          true );
-        messagedialog->set_message( _( "Are You Sure You Want to Upgrade The Diary?" ) );
-        messagedialog->set_secondary_text( _( STRING::UPGRADE_DIARY_CONFIRM ) );
-        messagedialog->add_button( _( "Upgrade The Diary" ), Gtk::RESPONSE_ACCEPT );
-
-        int response( messagedialog->run() );
-
-        delete messagedialog;
-
-        if( response !=  Gtk::RESPONSE_ACCEPT )
-        {
-            Diary::d->clear();
-            return LIFEO::ABORTED;
-        }
+        Diary::d->clear();
+        populate_diaries();
+        return;
     }
 
     // HANDLE ENCRYPTION
     if( Diary::d->is_encrypted() )
     {
-        if( DialogPassword::launch( DialogPassword::OT_OPEN, Diary::d, m_password_attempt_no )
-                != RESPONSE_GO )
-        {
-            Diary::d->clear();
-            return LIFEO::ABORTED;
-        }
-    }
+        Gtk::TreePath path;
+        m_LS_diaries->foreach_iter(
+                [ & ]( const Gtk::TreeIter& i )
+                {
+                    std::string sp{ ( *i )[ colrec->path ] };
+                    if( sp == m_path_cur )
+                    {
+                        path = m_LS_diaries->get_path( i );
+                        return true;
+                    }
+                    return false;
+                } );
+        Gdk::Rectangle rect;
+        m_TV_diaries->get_cell_area( path, * m_TV_diaries->get_column( 1 ), rect );
+        int x, y;
+        m_TV_diaries->convert_tree_to_widget_coords( rect.get_x(), rect.get_y(), x, y );
+        rect.set_x( x );
+        rect.set_y( y );
 
+        DialogPassword::launch( DialogPassword::OT_OPEN, Diary::d,
+                &rect, m_TV_diaries,
+                sigc::mem_fun( this, &ViewLogin::open_selected_diary2 ),
+                [ this ]{ } );
+    }
+    else
+        open_selected_diary2();
+}
+
+void
+ViewLogin::open_selected_diary2()
+{
     // FINALLY READ BODY
-    switch( result = Diary::d->read_body() )
+    switch( Diary::d->read_body() )
     {
         case LIFEO::SUCCESS:
+            DialogPassword::finish( Diary::d->get_path() );
             AppWindow::p->login();
             break;
         case WRONG_PASSWORD:
-            m_password_attempt_no++;
-            return open_selected_diary( read_only );
+            open_selected_diary();
+            break;
         case CORRUPT_FILE:
-            show_info( Gtk::MESSAGE_ERROR, _( STRING::CORRUPT_DIARY ) );
+            AppWindow::p->show_info( Gtk::MESSAGE_ERROR, _( STRING::CORRUPT_DIARY ) );
             // no break
         default:
-            return result;
+            DialogPassword::finish( "" );   // "" denotes unsuccessful finish
+            break;
     }
-
-    m_password_attempt_no = 0;
-    return LIFEO::SUCCESS;
 }
 
 void
 ViewLogin::remove_selected_diary()
 {
-    m_path_removed = m_path_cur;
-    Lifeograph::settings.recentfiles.erase( m_path_removed );
+    if( m_treesel_diaries->count_selected_rows() == 1 )
+    {
+        m_path_removed = ( *m_treesel_diaries->get_selected() )[ colrec->path ];
+        Lifeograph::settings.recentfiles.erase( m_path_removed );
+        populate_diaries();
+        AppWindow::p->show_info(
+                Gtk::MESSAGE_INFO,
+                Glib::ustring::compose( _( "Removed diary: %1" ), m_path_removed ),
+                _( "Undo" ), AppWindow::RESP_UNDO_REMOVE_DIARY );
+    }
+}
+
+void
+ViewLogin::undo_remove_selected_diary()
+{
+    Lifeograph::settings.recentfiles.insert( m_path_removed );
     populate_diaries();
-    show_info( Gtk::MESSAGE_INFO,
-               Glib::ustring::compose( _( "Removed diary: %1" ), m_path_removed ),
-               _( "Undo" ), RESP_UNDO );
 }
 
 void
 ViewLogin::create_new_diary()
 {
-    m_Po_add->hide();
-
     std::string path( Glib::get_home_dir() + "/new diary.diary" );
     std::string pass( "" );
 
@@ -490,7 +478,7 @@ ViewLogin::create_new_diary()
                 AppWindow::p->login();
                 break;
             case LIFEO::FILE_LOCKED:
-                show_info( Gtk::MESSAGE_INFO,
+                AppWindow::p->show_info( Gtk::MESSAGE_INFO,
                            Glib::ustring::compose( _( STRING::DIARY_LOCKED ),
                                    Glib::ustring::compose( "<a href=\"file://%1\" title=\"%2\">%3</a>",
                                                            Glib::path_get_dirname( m_path_cur ),
@@ -498,7 +486,7 @@ ViewLogin::create_new_diary()
                                                            m_path_cur + LOCK_SUFFIX ) ) );
                 break;
             default:
-                show_info( Gtk::MESSAGE_ERROR, _( STRING::FAILED_TO_OPEN_DIARY ) );
+                AppWindow::p->show_info( Gtk::MESSAGE_ERROR, _( STRING::FAILED_TO_OPEN_DIARY ) );
                 break;
         }
     }
@@ -507,8 +495,6 @@ ViewLogin::create_new_diary()
 void
 ViewLogin::add_existing_diary()
 {
-    m_Po_add->hide();
-
     DialogOpenDiary* filechooserdialog = new DialogOpenDiary;
 
     if( m_treesel_diaries->count_selected_rows() > 0 )
@@ -518,107 +504,67 @@ ViewLogin::add_existing_diary()
 
     int result( filechooserdialog->run() );
 
-    if( result == DialogOpenDiary::RESPONSE_READ || result == DialogOpenDiary::RESPONSE_EDIT )
+    if( result == Gtk::RESPONSE_OK )
     {
-        m_path_cur = filechooserdialog->get_filename();
+        Lifeograph::settings.recentfiles.insert( filechooserdialog->get_filename() );
         delete filechooserdialog;
-        if( open_selected_diary( result == DialogOpenDiary::RESPONSE_READ ) != LIFEO::SUCCESS )
-            m_treesel_diaries->unselect_all(); // mostly to clear m_path_cur
+        populate_diaries();
     }
     else
         delete filechooserdialog;
 }
 
 void
-ViewLogin::show_info( Gtk::MessageType type, const Glib::ustring& text,
-                      const Glib::ustring &button_text, InfoResponse response )
+ViewLogin::handle_select_mode_toggled()
 {
-    m_L_info->set_markup( text );
-    m_B_info->set_visible( !button_text.empty() );
-    m_B_info->set_label( button_text );
-    m_resp_cur = response;
-    m_IB_login->set_message_type( type );
-    if( m_IB_login->get_parent() == nullptr )
-        m_G_login->attach( *m_IB_login, 0, 0, 1, 1 );
-    m_IB_login->show();
-    m_flag_info_is_visible = true;
-}
-
-void
-ViewLogin::handle_infobar_response( int response )
-{
-    if( response != Gtk::RESPONSE_CLOSE && m_resp_cur == RESP_UNDO )
+    if( m_B_select->get_active() )
     {
-        Lifeograph::settings.recentfiles.insert( m_path_removed );
-        populate_diaries();
+        AppWindow::p->m_Hb_main->get_style_context()->add_class( "selection-mode" );
+        //m_TV_diaries->get_selection()->set_mode( Gtk::SELECTION_MULTIPLE );
+        m_TV_diaries->set_hover_selection( false );
+        m_B_remove_diary->set_visible( true );
     }
-
-    hide_infobar();
-}
-
-inline void
-ViewLogin::hide_infobar()
-{
-    if( m_flag_info_is_visible )
+    else
     {
-        m_IB_login->hide();
-        m_G_login->remove( *m_IB_login ); // necessary due to GNOME Bugzilla bug 710888
-        m_flag_info_is_visible = false;
+        AppWindow::p->m_Hb_main->get_style_context()->remove_class( "selection-mode" );
+        //m_TV_diaries->get_selection()->set_mode( Gtk::SELECTION_SINGLE );
+        m_TV_diaries->set_hover_selection( true );
+        m_B_remove_diary->set_visible( false );
     }
+    populate_diaries();
 }
 
 void
 ViewLogin::handle_diary_selection_changed()
 {
-    m_password_attempt_no = 0;
-
-    if( Lifeograph::s_internaloperation )
-        return;
-
-    bool flag_selection( m_treesel_diaries->count_selected_rows() > 0 );
-
-    if( flag_selection )
-        m_path_cur = ( *m_treesel_diaries->get_selected() )[ colrec->path ];
-    else
-        m_path_cur = "";
-
-    bool flag_not_stock( Lifeograph::stock_diaries.find( m_path_cur ) ==
-            Lifeograph::stock_diaries.end() );
-    m_B_remove_diary->set_visible( flag_selection && flag_not_stock );
-    m_B_edit_diary->set_visible( flag_selection && flag_not_stock );
-    m_B_read_diary->set_visible( flag_selection );
-
-    m_flag_diary_activated = false;
+    //m_flag_diary_activated = false;
 }
 
 void
 ViewLogin::handle_diary_activated( const Gtk::TreePath &path, Gtk::TreeView::Column* col )
 {
     // not handled directly to prevent BUTTON RELEASE event to be sent to edit screen widgets
-    m_flag_diary_activated = true;
+
+    if( ! m_B_select->get_active() )
+    {
+        //m_flag_diary_activated = true;
+        m_path_cur = ( *m_treesel_diaries->get_selected() )[ colrec->path ];
+        open_selected_diary();
+    }
 }
 
 void
 ViewLogin::handle_button_release( GdkEventButton* )
 {
-    if( m_flag_diary_activated )
-        open_activated_diary();
+//    if( m_flag_diary_activated )
+//        open_selected_diary();
 }
 
 void
 ViewLogin::handle_key_release( GdkEventKey* event )
 {
-    if( m_flag_diary_activated && event->keyval == GDK_KEY_Return )
-        open_activated_diary();
-}
-
-void
-ViewLogin::open_activated_diary()
-{
-    open_selected_diary( Lifeograph::stock_diaries.find( m_path_cur ) !=
-            Lifeograph::stock_diaries.end() ); // read only if stock diary
-
-    m_flag_diary_activated = false;
+//    if( m_flag_diary_activated && event->keyval == GDK_KEY_Return )
+//        open_selected_diary();
 }
 
 bool
@@ -660,14 +606,11 @@ ViewLogin::handle_dragged_file( const Glib::RefPtr<Gdk::DragContext>& context,
     if( Lifeograph::settings.recentfiles.find( filename ) != Lifeograph::settings.recentfiles.end() )
         return;
 
-    if( Lifeograph::stock_diaries.find( filename ) != Lifeograph::stock_diaries.end() )
-        return;
-
-    Gtk::TreeRow row = *( m_treestore_diaries->append() );
+    Gtk::TreeRow row = *( m_LS_diaries->append() );
     row[ colrec->name ] = "   +++   " + Glib::filename_display_basename( filename );
     row[ colrec->path ] = filename;
     m_TV_diaries->get_selection()->select( row );
-    m_TV_diaries->scroll_to_row(  m_treestore_diaries->get_path( row ) );
+    m_TV_diaries->scroll_to_row(  m_LS_diaries->get_path( row ) );
 
     Lifeograph::settings.recentfiles.insert( filename );
 
