@@ -1,6 +1,6 @@
 /***********************************************************************************
 
-    Copyright (C) 2007-2018 Ahmet Öztürk (aoz_2@yahoo.com)
+    Copyright (C) 2007-2019 Ahmet Öztürk (aoz_2@yahoo.com)
 
     This file is part of Lifeograph.
 
@@ -55,6 +55,10 @@ ElementViewWithChart< T >::ElementViewWithChart( const Ustring type )
         throw LIFEO::Error( "Failed to create the chart" );
     }
 
+    m_widgetchart->signal_size_allocate().connect(
+            [ this ]( Gtk::Allocation& )
+            { m_Sc_chart->set_visible( m_widgetchart->is_zoom_possible() ); } );
+
     m_A_underlay = Lifeograph::p->add_action_bool( "underlay_prev_year_" + type,
             sigc::mem_fun( *this, &ElementViewWithChart::handle_chart_underlay ), false);
     m_Sc_chart->signal_change_value().connect(
@@ -131,8 +135,8 @@ ElementViewWithChart< T >::handle_chart_shown()
             break;
     }
 
-    bool flag_underlay{ ElementView< T >::m_ptr2elem->get_chart_type() &
-                        ChartPoints::UNDERLAY_PREV_YEAR };
+    bool flag_underlay{ bool( ElementView< T >::m_ptr2elem->get_chart_type() &
+                        ChartPoints::UNDERLAY_PREV_YEAR ) };
 
     m_A_underlay->change_state( flag_underlay );
 
@@ -312,7 +316,7 @@ TagView::apply_title( const Glib::ustring& name )
     if( ! check_title_applicable( name ) )
         return false;
 
-    Diary::d->get_tags()->rename( m_ptr2elem, name );
+    Diary::d->rename_tag( m_ptr2elem, name );
     AppWindow::p->panel_extra->refresh_elem( m_ptr2elem );
 
     return true;
@@ -322,8 +326,17 @@ void
 TagView::handle_login()
 {
     update_combobox_categories();
-    m_G_general->set_visible( ! Diary::d->is_read_only() );
-    m_G_theme->set_visible( ! Diary::d->is_read_only() );
+    m_G_general->set_visible( false );
+    m_G_theme->set_visible( false );
+}
+
+void
+TagView::handle_edit_enabled()
+{
+    m_G_general->set_visible( true );
+    m_G_theme->set_visible( true );
+    if( AppWindow::p->panel_main->get_cur_elem_type() == DiaryElement::ET_TAG )
+        AppWindow::p->panel_main->show( this ); // mostly, to enable the icon
 }
 
 void
@@ -352,15 +365,15 @@ TagView::handle_category_changed()
         if( row[ colrec_category.type ]  == TCT_NEW ) // create new
         {
             CategoryTags *category = Diary::d->create_tag_ctg();
-            m_ptr2elem->set_category( category );
+            m_ptr2elem->set_ctg( category );
             update_combobox_categories();
             AppWindow::p->panel_extra->populate();
             category->show();
             AppWindow::p->panel_main->start_title_edit();
         }
-        else	// assign a category or reset
+        else    // assign a category or reset
         {
-            m_ptr2elem->set_category( row[ colrec_category.ptr ] );
+            m_ptr2elem->set_ctg( row[ colrec_category.ptr ] );
             AppWindow::p->panel_extra->populate();
         }
     }
@@ -388,6 +401,8 @@ TagView::handle_type_changed()
     }
 
     update_points( false );
+    AppWindow::p->panel_main->set_icon( m_ptr2elem->get_icon32() );
+    AppWindow::p->panel_extra->refresh_elem( m_ptr2elem );
 }
 
 void
@@ -595,7 +610,7 @@ TagView::show( Tag& tag )
          iter != m_liststore_categories->children().end();
          ++iter )
     {
-        if( ( *iter )[ colrec_category.ptr ] == tag.get_category() )
+        if( ( *iter )[ colrec_category.ptr ] == tag.get_ctg() )
             m_CB_category->set_active( iter );
     }
 
@@ -621,7 +636,7 @@ TagView::show( Tag& tag )
 
     Lifeograph::s_internaloperation--;
 
-    m_G_general->set_visible( ! Diary::d->is_read_only() &&
+    m_G_general->set_visible( Diary::d->is_editing_enabled() &&
                               tag.get_type() == DiaryElement::ET_TAG );
 
     update_theme_view();
@@ -631,7 +646,8 @@ TagView::show( Tag& tag )
 Gtk::Popover*
 TagView::get_popover()
 {
-    return( ( Diary::d->is_read_only() || m_ptr2elem->get_type() == DiaryElement::ET_UNTAGGED ) ?
+    return( ( Diary::d->is_editing_enabled() == false ||
+              m_ptr2elem->get_type() == DiaryElement::ET_UNTAGGED ) ?
             nullptr : m_Po_elem );
 }
 
@@ -801,7 +817,7 @@ CategoryTagsView::show( CategoryTags& category )
 {
     // do nothing if it is already the current element or when in read-only mode:
     if( AppWindow::p->panel_main->is_cur_elem( &category ) ||
-        Diary::d->is_read_only() )
+        Diary::d->is_editing_enabled() == false )
         return;
 
     m_ptr2elem = &category;
@@ -812,7 +828,7 @@ CategoryTagsView::show( CategoryTags& category )
 Gtk::Popover*
 CategoryTagsView::get_popover()
 {
-    return( Diary::d->is_read_only() ? nullptr : m_Po_elem );
+    return( Diary::d->is_editing_enabled() ? m_Po_elem : nullptr );
 }
 
 bool
@@ -828,7 +844,7 @@ CategoryTagsView::apply_title( const Glib::ustring& name )
     if( ! check_title_applicable( name ) )
         return false;
 
-    Diary::d->m_tag_categories.rename_category( m_ptr2elem, name );
+    Diary::d->rename_tag_ctg( m_ptr2elem, name );
     AppWindow::p->panel_extra->populate();
     AppWindow::p->m_tag_view->update_combobox_categories();
 
@@ -1033,18 +1049,27 @@ ChapterView::add_new_entry()
 void
 ChapterView::handle_login()
 {
-    bool editable( ! Diary::d->is_read_only() );
-    m_G_edit->set_visible( editable );
-    m_B_date_apply->set_visible( editable );
-    m_G_actions->set_visible( editable );
+    m_G_edit->set_visible( false );
+    m_B_date_apply->set_visible( false );
+    m_G_actions->set_visible( false );
+}
+
+void
+ChapterView::handle_edit_enabled()
+{
+    m_G_edit->set_visible( true );
+    m_B_date_apply->set_visible( true );
+    m_G_actions->set_visible( true );
+
+    if( AppWindow::p->panel_main->get_cur_elem_type() == DiaryElement::ET_CHAPTER )
+        AppWindow::p->panel_main->show( this ); // mostly, to enable the icon
 }
 
 void
 ChapterView::show( Chapter& chapter )
 {
-
     const bool flag_ordinal{ chapter.is_ordinal() };
-    const bool flag_read_only{ Diary::d->is_read_only() };
+    const bool flag_read_only{ Diary::d->is_editing_enabled() == false };
     // do nothing if entry is already the current element or
     // there is nothing to do due to read-only mode:
     if( AppWindow::p->panel_main->is_cur_elem( &chapter ) ||
@@ -1095,7 +1120,7 @@ ChapterView::show( Chapter& chapter )
 Gtk::Popover*
 ChapterView::get_popover()
 {
-    return( Diary::d->is_read_only() ? nullptr : m_Po_elem );
+    return( Diary::d->is_editing_enabled() ? m_Po_elem : nullptr );
 }
 
 void
@@ -1218,7 +1243,15 @@ FilterView::FilterView()
 void
 FilterView::handle_login()
 {
-    m_B_make_default->set_visible( ! Diary::d->is_read_only() );
+    m_B_make_default->set_visible( false );
+    m_ptr2elem = Diary::d->get_filter();
+    // the above line prevents segfaults when filter is reverted before being shown
+}
+
+void
+FilterView::handle_edit_enabled()
+{
+    m_B_make_default->set_visible( true );
     m_ptr2elem = Diary::d->get_filter();
     // the above line prevents segfaults when filter is reverted before being shown
 }
@@ -1379,15 +1412,15 @@ FilterView::handle_tag_dropped( const Glib::RefPtr< Gdk::DragContext >& context,
 {
     bool retval( true );
 
-    if( Lifeograph::s_elem_dragged != NULL )
+    if( Lifeograph::is_drag_in_progress() )
     {
-        switch( Lifeograph::s_elem_dragged->get_type() )
+        switch( Lifeograph::get_dragged_elem()->get_type() )
         {
             case DiaryElement::ET_TAG:
             case DiaryElement::ET_UNTAGGED:
             {
-                const Tag *tag( dynamic_cast< Tag* >( Lifeograph::s_elem_dragged ) );
-                m_L_tag->set_text( Lifeograph::s_elem_dragged->get_name() );
+                const Tag *tag( dynamic_cast< Tag* >( Lifeograph::get_dragged_elem() ) );
+                m_L_tag->set_text( tag->get_name() );
                 m_B_clear_tag->show();
                 m_ptr2elem->set_tag( tag );
                 AppWindow::p->panel_extra->set_filtered_tag( tag );
@@ -1500,13 +1533,13 @@ FilterView::handle_entry_dropped( const Glib::RefPtr< Gdk::DragContext >& contex
 {
     bool retval( true );
 
-    if( Lifeograph::s_elem_dragged != NULL )
+    if( Lifeograph::is_drag_in_progress() )
     {
-        switch( Lifeograph::s_elem_dragged->get_type() )
+        switch( Lifeograph::get_dragged_elem()->get_type() )
         {
             case DiaryElement::ET_ENTRY:
             {
-                Entry* entry( dynamic_cast< Entry* >( Lifeograph::s_elem_dragged ) );
+                Entry* entry( dynamic_cast< Entry* >( Lifeograph::get_dragged_elem() ) );
                 m_ptr2elem->add_entry( entry );
                 update_entries();
                 AppWindow::p->panel_diary->handle_filter_changed();
@@ -1666,21 +1699,34 @@ DiaryView::handle_login()
 
     Lifeograph::s_internaloperation--;
 
-    bool editable( ! Diary::d->is_read_only() );
-    m_CB_startup_type->set_sensitive( editable );
-    m_CB_spellcheck->set_sensitive( editable );
-    m_CB_chapter_ctg->set_sensitive( editable );
+    // READ-ONLY RELATED STUFF
+    m_CB_startup_type->set_sensitive( false );
+    m_CB_spellcheck->set_sensitive( false );
+    m_CB_chapter_ctg->set_sensitive( false );
 
-    m_B_import->set_visible( editable );
-    m_S_encryption->set_visible( editable );
-    m_B_encryption->set_visible( editable );
-
-    m_B_encryption->set_label( Diary::d->m_passphrase.empty() ? _( "Encrypt..." ) :
-                                                _( STRING::CHANGE_PASSWORD ) );
+    m_B_import->set_visible( false );
+    m_S_encryption->set_visible( false );
+    m_B_encryption->set_visible( false );
 
     // BODY
-    m_L_path->set_markup( Glib::ustring::compose( "<a href=\"file://%1\">%1</a>",
-            Glib::path_get_dirname( Diary::d->m_path ) ) );
+    const std::string path{ Glib::path_get_dirname( Diary::d->m_path ) };
+    m_L_path->set_markup( Glib::ustring::compose( "<a href=\"%1\">%2</a>",
+            Glib::filename_to_uri( path ), path ) );
+}
+
+void
+DiaryView::handle_edit_enabled()
+{
+    m_CB_startup_type->set_sensitive( true );
+    m_CB_spellcheck->set_sensitive( true );
+    m_CB_chapter_ctg->set_sensitive( true );
+
+    m_B_import->set_visible( true );
+    m_S_encryption->set_visible( true );
+    m_B_encryption->set_visible( true );
+
+    m_B_encryption->property_text() = ( Diary::d->m_passphrase.empty() ?
+            _( "Encrypt..." ) : _( STRING::CHANGE_PASSWORD ) );
 }
 
 void
@@ -1704,13 +1750,13 @@ DiaryView::start_dialog_password()
 {
     AppWindow::p->panel_main->hide_popover();
 
-    if( DialogPassword::launch(
+    DialogPassword::launch(
             m_ptr2elem->is_encrypted() ? DialogPassword::OT_CHANGE : DialogPassword::OT_ADD,
-            m_ptr2elem ) == RESPONSE_GO )
-    {
-        AppWindow::p->panel_main->refresh_extra_info();
-        m_B_encryption->set_label( _( STRING::CHANGE_PASSWORD ) );
-    }
+            m_ptr2elem, nullptr,
+            AppWindow::p->panel_main->get_B_elem(),
+            [ this ]{ AppWindow::p->panel_main->refresh_extra_info();
+                      m_B_encryption->set_label( _( STRING::CHANGE_PASSWORD ) ); },
+            [ this ]{} );
 }
 
 void
@@ -1719,7 +1765,7 @@ DiaryView::start_dialog_import()
     AppWindow::p->panel_main->hide_popover();
 
     if( m_dialog_import == nullptr )
-        Lifeograph::get_builder2()->get_widget_derived( "D_import",
+        Lifeograph::get_builder2()->get_widget_derived( "D_synchronize",
                                                         m_dialog_import );
 
     if(  m_dialog_import->run() == RESPONSE_GO )
@@ -1737,17 +1783,24 @@ DiaryView::start_dialog_export()
 {
     AppWindow::p->panel_main->hide_popover();
 
-    if( m_ptr2elem->is_encrypted() ?
-            DialogPassword::launch( DialogPassword::OT_AUTHENTICATE,
-                                    m_ptr2elem ) == RESPONSE_GO : true )
-    {
-        if( m_dialog_export == nullptr )
-            Lifeograph::get_builder2()->get_widget_derived( "D_export",
-                                                            m_dialog_export );
+    Gdk::Rectangle rect;
 
-        m_dialog_export->run();
-        m_dialog_export->hide();
-    }
+    auto do_start = [ this ]()
+            {
+                if( m_dialog_export == nullptr )
+                    Lifeograph::get_builder2()->get_widget_derived( "D_export",
+                                                                    m_dialog_export );
+
+                m_dialog_export->run();
+                m_dialog_export->hide();
+            };
+
+    if( m_ptr2elem->is_encrypted() )
+        DialogPassword::launch( DialogPassword::OT_AUTHENTICATE, m_ptr2elem,
+                                nullptr, AppWindow::p->panel_main->get_B_elem(),
+                                do_start, []{} );
+    else
+        do_start();
 }
 
 void
@@ -1787,14 +1840,13 @@ bool
 DiaryView::handle_startup_elem_drag_motion( const Glib::RefPtr<Gdk::DragContext>& context,
                                             int x, int y, guint time )
 {
-    if( Lifeograph::s_flag_dragging )
-        if( Lifeograph::s_elem_dragged )
-            if( Lifeograph::s_elem_dragged->get_type() == DiaryElement::ET_DIARY ||
-                Lifeograph::s_elem_dragged->get_type() == DiaryElement::ET_ENTRY )
-            {
-                context->drag_status( Gdk::ACTION_COPY, time );
-                return true;
-            }
+    if( Lifeograph::is_drag_in_progress() )
+        if( Lifeograph::get_dragged_elem()->get_type() == DiaryElement::ET_DIARY ||
+            Lifeograph::get_dragged_elem()->get_type() == DiaryElement::ET_ENTRY )
+        {
+            context->drag_status( Gdk::ACTION_COPY, time );
+            return true;
+        }
 
     m_L_startup_elem_drop->drag_unhighlight();  // is called many times unnecessarily :(
     context->drag_refuse( time );
@@ -1811,21 +1863,18 @@ DiaryView::handle_startup_elem_dropped(
 
     context->drag_finish( true, false, time );
 
-    if( Lifeograph::s_flag_dragging )
+    if( Lifeograph::is_drag_in_progress() )
     {
-        if( Lifeograph::s_elem_dragged != NULL )
+        switch( Lifeograph::get_dragged_elem()->get_type() )
         {
-            switch( Lifeograph::s_elem_dragged->get_type() )
-            {
-                case DiaryElement::ET_ENTRY:
-                case DiaryElement::ET_DIARY:
-                    Diary::d->m_startup_elem_id = Lifeograph::s_elem_dragged->get_id();
-                    update_startup_elem();
-                    retval = true;
-                    break;
-                default:
-                    break;
-            }
+            case DiaryElement::ET_ENTRY:
+            case DiaryElement::ET_DIARY:
+                Diary::d->m_startup_elem_id = Lifeograph::get_dragged_elem()->get_id();
+                update_startup_elem();
+                retval = true;
+                break;
+            default:
+                break;
         }
     }
 
@@ -1865,7 +1914,7 @@ DiaryView::update_combobox_chapter_ctg()
     }
 
     m_B_chapter_ctg_rename->set_visible( false );
-    if( ! Diary::d->is_read_only() )
+    if( Diary::d->is_editing_enabled() )
     {
         m_B_chapter_ctg_dismiss->set_visible( true );
         m_B_chapter_ctg_dismiss->set_sensitive(
@@ -1926,7 +1975,7 @@ DiaryView::handle_cur_chapter_ctg_changed()
         m_B_chapter_ctg_dismiss->set_sensitive( flag_existing_ctg );
     }
 
-    if( ! Diary::d->is_read_only() )
+    if( Diary::d->is_editing_enabled() )
     {
         m_B_chapter_ctg_rename->set_visible( flag_renamed );
         m_B_chapter_ctg_dismiss->set_visible( ! flag_renamed );
@@ -1964,3 +2013,4 @@ DiaryView::is_row_separator( const Glib::RefPtr< Gtk::TreeModel >& model,
     Gtk::TreeRow row = * iter;
     return ( row[ colrec_chapter_ctg.type ] == CCCT_SEPARATOR );
 }
+
