@@ -30,6 +30,7 @@
 #include "widget_chart.hpp"
 #include "dialog_export.hpp"
 #include "dialog_password.hpp"
+#include "dialog_sync.hpp"
 #include "widget_textview.hpp"
 
 
@@ -79,6 +80,7 @@ class TagView : public ElementViewWithChart< Tag >
         Gtk::Popover*           get_popover() override;
 
         void                    handle_login();
+        void                    handle_edit_enabled();
         void                    show( Tag& tag );
 
         void                    update_combobox_categories();
@@ -228,6 +230,7 @@ class ChapterView : public ElementViewWithChart< Chapter >
         Gtk::Popover*           get_popover() override;
 
         void                    handle_login();
+        void                    handle_edit_enabled();
         void                    show( Chapter& );
 
         void                    set_todo_status( ElemStatus status );
@@ -280,6 +283,7 @@ class FilterView
         Glib::ustring           get_info_str() const;*/
 
         void                    handle_login();
+        void                    handle_edit_enabled();
         //void                    show( Filter& );
 
         void                    update();
@@ -383,6 +387,7 @@ class DiaryView : public ElementViewWithChart< Diary >
         }
 
         void                    handle_login();
+        void                    handle_edit_enabled();
         void                    show( Diary& );
 
         void                    update_startup_elem();
@@ -458,7 +463,7 @@ class DiaryView : public ElementViewWithChart< Diary >
         ColrecChapterCtg        colrec_chapter_ctg;
 
         // DIALOGS
-        DialogImport*           m_dialog_import{ nullptr };
+        DialogSync*             m_dialog_import{ nullptr };
         DialogExport*           m_dialog_export{ nullptr };
 };
 
