@@ -1,6 +1,6 @@
 /***********************************************************************************
 
-    Copyright (C) 2007-2018 Ahmet Öztürk (aoz_2@yahoo.com)
+    Copyright (C) 2007-2019 Ahmet Öztürk (aoz_2@yahoo.com)
 
     This file is part of Lifeograph.
 
@@ -70,6 +70,14 @@ WidgetEntryList::WidgetEntryList( BaseObjectType* cobject,
 
     append_column( *column );
 
+    Glib::RefPtr< Gtk::CssProvider > css = Gtk::CssProvider::create();
+    if( css->load_from_data(
+            "treeview#elem_list { background-color: transparent } "
+            "treeview#elem_list:selected { color: #bb3333; background-color: #dddddd } "
+            "treeview#elem_list button { background: transparent } " ) )
+        Gtk::StyleContext::add_provider_for_screen(
+                Gdk::Screen::get_default(), css, GTK_STYLE_PROVIDER_PRIORITY_USER );
+
     get_selection()->signal_changed().connect(
             sigc::mem_fun( this, &WidgetEntryList::handle_selection_changed ) );
 }
@@ -78,7 +86,7 @@ void
 WidgetEntryList::cell_data_func_icon( Gtk::CellRenderer* cell, const Gtk::TreeModel::iterator& iter )
 {
     DiaryElement* elem( ( * iter )[ ListData::colrec->ptr ] );
-    if( elem != NULL )
+    if( elem != nullptr )
     {
         bool favored( elem->is_favored() );
         cell->set_property( "visible", favored );
@@ -95,7 +103,7 @@ void
 WidgetEntryList::clear()
 {
     m_treestore->clear();
-    Lifeograph::s_elem_dragged = NULL;
+    Lifeograph::end_drag();
 }
 
 void
@@ -298,7 +306,7 @@ select_false( const Glib::RefPtr< Gtk::TreeModel >& model,
 bool
 WidgetEntryList::on_button_press_event( GdkEventButton* event )
 {
-    Lifeograph::s_flag_dragging = false;
+    Lifeograph::end_drag();
 
     Gtk::TreePath path;
     if( get_path_at_pos( ( int ) event->x, ( int ) event->y, path ) )
@@ -308,17 +316,15 @@ WidgetEntryList::on_button_press_event( GdkEventButton* event )
         if( get_selection()->is_selected( path ) && get_selection()->count_selected_rows() > 1 &&
             !( event->state & ( Gdk::CONTROL_MASK | Gdk::SHIFT_MASK ) ) )
         {
-            Lifeograph::s_elem_dragged = &m_multiple_entries;
+            Lifeograph::begin_drag( &m_multiple_entries );
             get_selection()->set_select_function( sigc::ptr_fun( &select_false ) );
         }
         else
         {
-            Lifeograph::s_elem_dragged = row[ ListData::colrec->ptr ];
+            Lifeograph::begin_drag( row[ ListData::colrec->ptr ] );
             get_selection()->set_select_function( sigc::ptr_fun( &select_true ) );
         }
     }
-    else
-        Lifeograph::s_elem_dragged = NULL;
 
     return Gtk::TreeView::on_button_press_event( event );
 }
@@ -328,31 +334,28 @@ WidgetEntryList::on_button_release_event( GdkEventButton* event )
 {
     get_selection()->set_select_function( sigc::ptr_fun( &select_true ) );
 
-    if( ! Lifeograph::s_flag_dragging /*&& event->button != 1*/ )
+    if( Lifeograph::is_drag_in_progress() /*&& event->button != 1*/ &&
+        !( event->state & ( Gdk::CONTROL_MASK | Gdk::SHIFT_MASK ) ) )
     {
-        if( Lifeograph::s_elem_dragged &&
-            !( event->state & ( Gdk::CONTROL_MASK | Gdk::SHIFT_MASK ) ) )
+        if( Lifeograph::get_dragged_elem()->get_type() == DiaryElement::ET_MULTIPLE_ENTRIES )
         {
-            if( Lifeograph::s_elem_dragged->get_type() == DiaryElement::ET_MULTIPLE_ENTRIES )
+            get_selection()->unselect_all();
+
+            Gtk::TreePath path;
+            if( get_path_at_pos( ( int ) event->x, ( int ) event->y, path ) )
             {
-                get_selection()->unselect_all();
-
-                Gtk::TreePath path;
-                if( get_path_at_pos( ( int ) event->x, ( int ) event->y, path ) )
-                {
-                    get_selection()->select( path );
-                    Gtk::TreeRow row = * m_treestore->get_iter( path );
-                    DiaryElement *element = row[ ListData::colrec->ptr ];
-                    element->show();
-                }
+                get_selection()->select( path );
+                Gtk::TreeRow row = * m_treestore->get_iter( path );
+                DiaryElement *element = row[ ListData::colrec->ptr ];
+                element->show();
             }
-            else
-                Lifeograph::s_elem_dragged->show();
         }
-
-        Lifeograph::s_elem_dragged = NULL;
+        else
+            Lifeograph::get_dragged_elem()->show();
     }
 
+    Lifeograph::end_drag();
+
     return Gtk::TreeView::on_button_release_event( event );
 }
 
@@ -382,13 +385,12 @@ WidgetEntryList::on_key_press_event( GdkEventKey* event )
 void
 WidgetEntryList::on_drag_begin( const Glib::RefPtr< Gdk::DragContext >& context )
 {
-    if( Lifeograph::s_elem_dragged )
+    if( Lifeograph::is_drag_in_progress() )
     {
-        Lifeograph::s_flag_dragging = true;
         get_selection()->set_select_function( sigc::ptr_fun( &select_true ) );
 
         //Gtk::TreeView::on_drag_begin( context );
-        context->set_icon( Lifeograph::s_elem_dragged->get_icon32(), 0, 0 );
+        context->set_icon( Lifeograph::get_dragged_elem()->get_icon32(), 0, 0 );
     }
 }
 
@@ -397,25 +399,28 @@ WidgetEntryList::on_drag_end( const Glib::RefPtr< Gdk::DragContext >& context )
 {
     PRINT_DEBUG( "WidgetEntryList::on_drag_end()" );
     Gtk::TreeView::on_drag_end( context );
-    restore_selection();
-    Lifeograph::s_elem_dragged = NULL;
+    Lifeograph::end_drag();
 }
 
 bool
 WidgetEntryList::on_drag_motion( const Glib::RefPtr< Gdk::DragContext >& context,
                                  int x, int y, guint time )
 {
-    m_flag_text_drop = false;
-    if( context->list_targets()[ 0 ] == "GTK_TEXT_BUFFER_CONTENTS" )
-        m_flag_text_drop = true;
-    else if( ! Lifeograph::s_flag_dragging )
-        return false;
-    else if( ! Lifeograph::s_elem_dragged )
+    m_flag_text_drop = ( context->list_targets()[ 0 ] == "GTK_TEXT_BUFFER_CONTENTS" );
+    if( ! m_flag_text_drop && ! Lifeograph::is_drag_in_progress() )
         return false;
 
+    auto drop_target{ Lifeograph::get_drop_target() };
+    if( drop_target != nullptr )
+    {
+        auto row_p = * m_treestore->get_iter( drop_target->m_list_data->treepath );
+        row_p[ ListData::colrec->icon ] = drop_target->get_icon();
+    }
+
     bool retval( false );
     Gdk::DragAction action{ m_flag_text_drop ? Gdk::ACTION_COPY : Gdk::ACTION_MOVE };
     Gtk::TreePath path;
+    Gtk::TreeRow row;
     DiaryElement* element{ nullptr };
 
     // AUTOSCROLL
@@ -445,16 +450,16 @@ WidgetEntryList::on_drag_motion( const Glib::RefPtr< Gdk::DragContext >& context
     // EVALUATE THE HOVERED ELEMENT
     if( get_path_at_pos( x, y, path ) )
     {
-        Gtk::TreeRow row = * m_treestore->get_iter( path );
+        row = * m_treestore->get_iter( path );
         element = row[ ListData::colrec->ptr ];
         if( m_flag_text_drop && element != nullptr )
         {
             if( element->get_type() == DiaryElement::ET_ENTRY )
                 retval = true;
         }
-        else if( element != nullptr && element != Lifeograph::s_elem_dragged )
+        else if( element != nullptr && element != Lifeograph::get_dragged_elem() )
         {
-            switch( Lifeograph::s_elem_dragged->get_type() )
+            switch( Lifeograph::get_dragged_elem()->get_type() )
             {
                 case DiaryElement::ET_ENTRY:
                     if( element->get_type() != DiaryElement::ET_DIARY )
@@ -471,7 +476,7 @@ WidgetEntryList::on_drag_motion( const Glib::RefPtr< Gdk::DragContext >& context
                     if( element->get_type() == DiaryElement::ET_ENTRY &&
                         element->get_date().is_ordinal() == false &&
                         element->get_date().get_pure() !=
-                                Lifeograph::s_elem_dragged->get_date().get_pure() )
+                                Lifeograph::get_dragged_elem()->get_date().get_pure() )
                         retval = true;
                     break;
                 case DiaryElement::ET_TOPIC:
@@ -500,13 +505,10 @@ WidgetEntryList::on_drag_motion( const Glib::RefPtr< Gdk::DragContext >& context
 
     if( retval )
     {
-        m_elem_drop_target = element;
+        Lifeograph::update_drop_target( element );
         context->drag_status( action, time );
-        restore_selection();
-        get_selection()->select( path );
+        row[ ListData::colrec->icon ] = Lifeograph::icons->next_16;
     }
-    else
-        restore_selection();
 
     //return Gtk::TreeView::on_drag_motion( context, x, y, time );
     return retval;
@@ -517,106 +519,106 @@ WidgetEntryList::on_drag_drop( const Glib::RefPtr< Gdk::DragContext >& context,
                                int x, int y, guint time )
 {
     bool retval( false );
+    auto drop_target{ Lifeograph::get_drop_target() };
 
-    if( Lifeograph::s_flag_dragging )
+    if( !drop_target ) { } // do nothing!
+    else if( Lifeograph::is_drag_in_progress() )
     {
-        if( Lifeograph::s_elem_dragged != NULL && m_elem_drop_target != NULL )
+        switch( Lifeograph::get_dragged_elem()->get_type() )
         {
-            switch( Lifeograph::s_elem_dragged->get_type() )
+            case DiaryElement::ET_ENTRY:
             {
-                case DiaryElement::ET_ENTRY:
+                Entry* entry( dynamic_cast< Entry* >( Lifeograph::get_dragged_elem() ) );
+                switch( drop_target->get_type() )
                 {
-                    Entry* entry( dynamic_cast< Entry* >( Lifeograph::s_elem_dragged ) );
-                    switch( m_elem_drop_target->get_type() )
+                    case DiaryElement::ET_CHAPTER:
+                    case DiaryElement::ET_TOPIC:
+                    case DiaryElement::ET_GROUP:
                     {
-                        case DiaryElement::ET_CHAPTER:
-                        case DiaryElement::ET_TOPIC:
-                        case DiaryElement::ET_GROUP:
-                        {
-                            Chapter* chapter( dynamic_cast< Chapter* >( m_elem_drop_target ) );
-                            Diary::d->set_entry_date( entry, chapter->get_free_order() );
-                            AppWindow::p->panel_main->refresh_title();
-                            AppWindow::p->panel_diary->update_entry_list();
-                            retval = true;
-                            break;
-                        }
-                        case DiaryElement::ET_ENTRY:
-                            Diary::d->set_entry_date( entry, m_elem_drop_target->get_date() );
-                            AppWindow::p->panel_main->refresh_title();
-                            AppWindow::p->panel_diary->update_entry_list();
-                            retval = true;
-                            break;
-                        default:
-                            break;
-                    }
-                    break;
-                }
-                case DiaryElement::ET_MULTIPLE_ENTRIES:
-                    if( m_elem_drop_target->get_type() == DiaryElement::ET_CHAPTER ||
-                        m_elem_drop_target->get_type() == DiaryElement::ET_TOPIC ||
-                        m_elem_drop_target->get_type() == DiaryElement::ET_GROUP )
-                    {
-                        Chapter* chapter( dynamic_cast< Chapter* >( m_elem_drop_target ) );
-                        sigc::slot< void, Entry* > s = sigc::bind(
-                                 sigc::mem_fun( this, &WidgetEntryList::move_entry_to_chapter ),
-                                 chapter );
-                        AppWindow::p->panel_diary->do_for_each_selected_entry( s );
-
+                        Chapter* chapter( dynamic_cast< Chapter* >( drop_target ) );
+                        Diary::d->set_entry_date( entry, chapter->get_free_order() );
                         AppWindow::p->panel_main->refresh_title();
                         AppWindow::p->panel_diary->update_entry_list();
+                        retval = true;
+                        break;
                     }
-                    break;
-                case DiaryElement::ET_CHAPTER:
-                {
-                    Chapter* chapter( dynamic_cast< Chapter* >( Lifeograph::s_elem_dragged ) );
-                    Diary::d->get_current_chapter_ctg()->set_chapter_date(
-                            chapter, m_elem_drop_target->get_date_t() );
-                    AppWindow::p->panel_main->refresh_title();
-                    AppWindow::p->panel_diary->update_entry_list();
-                    retval = true;
-                    break;
+                    case DiaryElement::ET_ENTRY:
+                        Diary::d->set_entry_date( entry, drop_target->get_date() );
+                        AppWindow::p->panel_main->refresh_title();
+                        AppWindow::p->panel_diary->update_entry_list();
+                        retval = true;
+                        break;
+                    default:
+                        break;
                 }
-                case DiaryElement::ET_GROUP:
-                case DiaryElement::ET_TOPIC:
-                {
-                    Chapter* chapter( dynamic_cast< Chapter* >( Lifeograph::s_elem_dragged ) );
-                    Diary::d->set_topic_order( chapter, m_elem_drop_target->get_date_t() );
-                    AppWindow::p->panel_main->refresh_title();
-                    AppWindow::p->panel_diary->update_entry_list();
-                    retval = true;
-                    break;
-                }
-                case DiaryElement::ET_TAG:
-                case DiaryElement::ET_UNTAGGED:
-                    if( m_elem_drop_target->get_type() == DiaryElement::ET_ENTRY )
-                    {
-                        Tag* tag( dynamic_cast< Tag* >( Lifeograph::s_elem_dragged ) );
-                        Entry* entry( dynamic_cast< Entry* >( m_elem_drop_target ) );
-                        if( entry->add_tag( tag ) ) // if does not already have the tag
-                        {
-                            if( AppWindow::p->panel_main->is_cur_elem( m_elem_drop_target ) )
-                            {
-                                AppWindow::p->m_entry_view->update_tag_widget();
-                                AppWindow::p->m_entry_view->update_theme();
-                            }
-
-                            retval = true;
-                        }
-                    }
-                    break;
-                default:
-                    break;
+                break;
             }
+            case DiaryElement::ET_MULTIPLE_ENTRIES:
+                if( drop_target->get_type() == DiaryElement::ET_CHAPTER ||
+                    drop_target->get_type() == DiaryElement::ET_TOPIC ||
+                    drop_target->get_type() == DiaryElement::ET_GROUP )
+                {
+                    Chapter* chapter( dynamic_cast< Chapter* >( drop_target ) );
+                    sigc::slot< void, Entry* > s = sigc::bind(
+                             sigc::mem_fun( this, &WidgetEntryList::move_entry_to_chapter ),
+                             chapter );
+                    AppWindow::p->panel_diary->do_for_each_selected_entry( s );
+
+                    AppWindow::p->panel_main->refresh_title();
+                    AppWindow::p->panel_diary->update_entry_list();
+                }
+                break;
+            case DiaryElement::ET_CHAPTER:
+            {
+                Chapter* chapter( dynamic_cast< Chapter* >( Lifeograph::get_dragged_elem() ) );
+                Diary::d->get_current_chapter_ctg()->set_chapter_date(
+                        chapter, drop_target->get_date_t() );
+                AppWindow::p->panel_main->refresh_title();
+                AppWindow::p->panel_diary->update_entry_list();
+                retval = true;
+                break;
+            }
+            case DiaryElement::ET_GROUP:
+            case DiaryElement::ET_TOPIC:
+            {
+                Chapter* chapter( dynamic_cast< Chapter* >( Lifeograph::get_dragged_elem() ) );
+                Diary::d->set_topic_order( chapter, drop_target->get_date_t() );
+                AppWindow::p->panel_main->refresh_title();
+                AppWindow::p->panel_diary->update_entry_list();
+                retval = true;
+                break;
+            }
+            case DiaryElement::ET_TAG:
+            case DiaryElement::ET_UNTAGGED:
+                if( drop_target->get_type() == DiaryElement::ET_ENTRY )
+                {
+                    Tag* tag( dynamic_cast< Tag* >( Lifeograph::get_dragged_elem() ) );
+                    Entry* entry( dynamic_cast< Entry* >( drop_target ) );
+                    if( entry->add_tag( tag ) ) // if does not already have the tag
+                    {
+                        if( AppWindow::p->panel_main->is_cur_elem( drop_target ) )
+                        {
+                            AppWindow::p->m_entry_view->update_tag_widget();
+                            AppWindow::p->m_entry_view->update_theme();
+                        }
+
+                        retval = true;
+                    }
+                }
+                break;
+            default:
+                break;
         }
     }
-    else if( m_flag_text_drop && m_elem_drop_target )
+    else if( m_flag_text_drop )
     {
         if( AppWindow::p->panel_main->get_cur_elem()->get_type() == DiaryElement::ET_ENTRY &&
-            m_elem_drop_target->get_type() == DiaryElement::ET_ENTRY )
+            drop_target->get_type() == DiaryElement::ET_ENTRY )
         {
             PRINT_DEBUG( AppWindow::p->m_entry_view->get_buffer()->get_selected_text() );
-            dynamic_cast< Entry* >( m_elem_drop_target )->append_text(
-                    AppWindow::p->m_entry_view->get_buffer()->get_selected_text() );
+            auto entry{ dynamic_cast< Entry* >( drop_target ) };
+            entry->append_text( AppWindow::p->m_entry_view->get_buffer()->get_selected_text() );
+            AppWindow::p->panel_diary->handle_elem_changed( entry );
             retval = true;
         }
     }
@@ -628,7 +630,7 @@ WidgetEntryList::on_drag_drop( const Glib::RefPtr< Gdk::DragContext >& context,
 void
 WidgetEntryList::handle_selection_changed()
 {
-    if( Lifeograph::s_internaloperation || Lifeograph::s_flag_dragging )
+    if( Lifeograph::s_internaloperation )
         return;
 
     m_multiple_entries.m_entries.clear();
@@ -656,7 +658,7 @@ WidgetEntryList::handle_selection_changed()
     }
 }
 
-void
+/*void
 WidgetEntryList::restore_selection()
 {
     get_selection()->unselect_all();
@@ -665,7 +667,7 @@ WidgetEntryList::restore_selection()
         get_selection()->select( entry->m_list_data->treepath );
 
     // TODO also restore if last selection was another type of element than entry
-}
+}*/
 
 void
 WidgetEntryList::do_for_each_selected_entry( sigc::slot< void, Entry* >& slot )
