@@ -1,6 +1,6 @@
 /***********************************************************************************
 
-    Copyright (C) 2007-2018 Ahmet Öztürk (aoz_2@yahoo.com)
+    Copyright (C) 2007-2019 Ahmet Öztürk (aoz_2@yahoo.com)
 
     This file is part of Lifeograph.
 
@@ -172,6 +172,10 @@ WidgetTagEdit::handle_event_after( GdkEvent* event )
 }
 
 // TAG LIST WIDGET =================================================================================
+Gdk::RGBA                           WidgetEntryTags::s_color_text_default;
+Cairo::RefPtr< Cairo::ToyFontFace > WidgetEntryTags::s_font_system;
+Cairo::RefPtr< Cairo::Context >     WidgetEntryTags::s_image_context;
+
 WidgetEntryTags::WidgetEntryTags( BaseObjectType* cobject, const Glib::RefPtr< Gtk::Builder >& )
 :   DrawingArea( cobject )
 {
@@ -214,13 +218,20 @@ WidgetEntryTags::WidgetEntryTags( BaseObjectType* cobject, const Glib::RefPtr< G
     m_B_tag_remove->signal_clicked().connect(
             sigc::mem_fun( this, &WidgetEntryTags::handle_remove ) );
 
-    // DEFAULT FONT
+    m_image_surface_add = Cairo::ImageSurface::create( Cairo::FORMAT_ARGB32,
+                                                       ICON_SIZE, ICON_SIZE );
+    s_image_context = Cairo::Context::create( m_image_surface_add );
+}
+
+void
+WidgetEntryTags::handle_gtk_theme_changed()
+{
     Gtk::Button* button = new Gtk::Button;
     button->show();
-    m_font_system = Cairo::ToyFontFace::create( button->get_style_context()->get_font().to_string(),
+    s_font_system = Cairo::ToyFontFace::create( button->get_style_context()->get_font().to_string(),
                                                 Cairo::FONT_SLANT_NORMAL,
                                                 Cairo::FONT_WEIGHT_NORMAL );
-    m_color_text_default = button->get_style_context()->get_color( Gtk::STATE_FLAG_ACTIVE );
+    s_color_text_default = button->get_style_context()->get_color( Gtk::STATE_FLAG_ACTIVE );
     delete button;
 
     // TAG ICON
@@ -233,14 +244,12 @@ WidgetEntryTags::WidgetEntryTags( BaseObjectType* cobject, const Glib::RefPtr< G
     Gdk::Cairo::set_source_pixbuf( m_image_context, Lifeograph::icons->tag_theme_16, 0.0, 0.0 );
     m_image_context->paint();*/
 
-    m_image_surface_add = Cairo::ImageSurface::create( Cairo::FORMAT_ARGB32,
-                                                       ICON_SIZE, ICON_SIZE );
-    m_image_context = Cairo::Context::create( m_image_surface_add );
+    // TODO: we need to switch the icon for dark theme but we could not find a way
     Gdk::Cairo::set_source_pixbuf(
-            m_image_context,
-            Gtk::IconTheme::get_default()->load_icon( "list-add", Gtk::ICON_SIZE_MENU ),
+            s_image_context,
+            Gtk::IconTheme::get_default()->load_icon( "list-add-symbolic", ICON_SIZE ),
             0.0, 0.0 );
-    m_image_context->paint();
+    s_image_context->paint();
 }
 
 void
@@ -386,7 +395,7 @@ WidgetEntryTags::start_editing()
         m_hovered_tag = &m_add_tag_item;
 
     if( AppWindow::p->panel_main->get_cur_elem_type() == DiaryElement::ET_ENTRY &&
-        !Diary::d->is_read_only() )
+        Diary::d->is_editing_enabled() )
     {
         auto iter{ m_items.find( m_hovered_tag ) };
         if( iter == m_items.end() )
@@ -561,7 +570,7 @@ WidgetEntryTags::add_item( const Cairo::RefPtr< Cairo::Context >& cr, const Tag*
         cr->set_font_face( m_font_theme );
     }
     else
-        cr->set_font_face( m_font_system );
+        cr->set_font_face( s_font_system );
 
     cr->get_text_extents( label, te );
     if( m_pos_x + ICON_SIZE + LABEL_OFFSET + te.width + MARGIN > m_width )
@@ -615,7 +624,7 @@ WidgetEntryTags::add_item( const Cairo::RefPtr< Cairo::Context >& cr, const Tag*
         }
         else
         {
-            Gdk::Cairo::set_source_rgba( cr, m_color_text_default );
+            Gdk::Cairo::set_source_rgba( cr, s_color_text_default );
             cr->stroke();
         }
 
@@ -639,7 +648,7 @@ WidgetEntryTags::add_item( const Cairo::RefPtr< Cairo::Context >& cr, const Tag*
     if( tag && tag->get_has_own_theme() )
         Gdk::Cairo::set_source_rgba( cr, tag->get_theme()->color_text );
     else
-        Gdk::Cairo::set_source_rgba( cr, m_color_text_default );
+        Gdk::Cairo::set_source_rgba( cr, s_color_text_default );
 
     cr->move_to( m_pos_x + 0.5, m_pos_y + TEXT_HEIGHT + 0.5 );
     cr->show_text( label );
@@ -675,8 +684,8 @@ WidgetEntryTags::on_draw( const Cairo::RefPtr< Cairo::Context >& cr )
 
     if( tags.empty() && !m_flag_editable )
     {
-        cr->set_font_face( m_font_system );
-        Gdk::Cairo::set_source_rgba( cr, m_color_text_default );
+        cr->set_font_face( s_font_system );
+        Gdk::Cairo::set_source_rgba( cr, s_color_text_default );
         cr->move_to( m_pos_x, m_pos_y + TEXT_HEIGHT );
         cr->show_text( _( "Not tagged" ) );
     }
@@ -691,3 +700,4 @@ WidgetEntryTags::on_draw( const Cairo::RefPtr< Cairo::Context >& cr )
 
     return true;
 }
+
