@@ -1,6 +1,6 @@
 /***********************************************************************************
 
-    Copyright (C) 2007-2018 Ahmet Öztürk (aoz_2@yahoo.com)
+    Copyright (C) 2007-2019 Ahmet Öztürk (aoz_2@yahoo.com)
 
     This file is part of Lifeograph.
 
@@ -61,7 +61,7 @@ class UndoEdit : public Undoable
 {
     public:
                             UndoEdit( int position,
-                                      const Glib::ustring& text,
+                                      const Ustring& text,
                                       UndoableType type )
         : Undoable( "placeholder", type ), m_position( position ), m_text( text ) { }
         virtual             ~UndoEdit() {}   //needed because of virtual methods
@@ -71,7 +71,7 @@ class UndoEdit : public Undoable
             return m_position;
         }
 
-        Glib::ustring       get_text() const
+        Ustring             get_text() const
         {
             return m_text;
         }
@@ -93,7 +93,7 @@ class UndoEdit : public Undoable
         void                insert();
 
         unsigned int        m_position;
-        Glib::ustring       m_text;
+        Ustring             m_text;
 
     private:
         static Gtk::TextBuffer*     m_ptr2buffer;
@@ -104,7 +104,7 @@ class UndoEdit : public Undoable
 class UndoInsert :public UndoEdit
 {
     public:
-                            UndoInsert( int position, const Glib::ustring& text )
+                            UndoInsert( int position, const Ustring& text )
         : UndoEdit( position, text, UT_INSERT_TEXT ) { }
 
         bool                can_merge( const Undoable* action ) const
@@ -136,7 +136,7 @@ class UndoInsert :public UndoEdit
 class UndoErase : public UndoEdit
 {
     public:
-                            UndoErase( int position, const Glib::ustring& text )
+                            UndoErase( int position, const Ustring& text )
         : UndoEdit( position, text, UT_ERASE_TEXT ) { }
 
         bool                can_merge( const Undoable* action ) const
@@ -179,7 +179,7 @@ class Link
     public:
         enum LinkType
         {
-            LT_NONE, LT_ID, LT_ENTRY, LT_URI, LT_CHECK, LT_IMAGE
+            LT_NONE, LT_ID, LT_ENTRY, LT_TAG, LT_URI, LT_CHECK, LT_IMAGE
         };
 
         virtual                     ~Link();
@@ -233,6 +233,18 @@ class LinkEntry : public Link
         static Gtk::Menu*           menu_link;
 };
 
+class LinkTag : public Link
+{
+    public:
+                                    LinkTag( const Glib::RefPtr< Gtk::TextMark >&,
+                                             const Glib::RefPtr< Gtk::TextMark >&,
+                                             Tag* );
+        void                        go();
+
+    protected:
+        Tag*                        m_tag;
+};
+
 class LinkUri : public Link
 {
     public:
@@ -254,7 +266,7 @@ class LinkCheck : public Link
         void                        go();
         static void                 set_check( int );
 
-        static const Glib::ustring  s_boxes;
+        static const Ustring        s_boxes;
         static int                  s_pos_mark;
 
         unsigned int                m_state_index;
@@ -270,38 +282,39 @@ class PredicateNL
         }
 };
 
+class TextviewDiary;    // forward declaration
 // TEXTBUFFER ======================================================================================
 class TextbufferDiary : public Gtk::TextBuffer, public EntryParser
 {
     public:
         static const int            LEFT_MARGIN = 10;
 
-
-        typedef void ( TextbufferDiary::*ParsingApplierFn )();
-
 //      typedef sigc::signal< LinkStatus, Date >
 //                                  Signal_LinkStatus_Date;
-        typedef sigc::signal< void, const Glib::ustring& >
+        typedef sigc::signal< void, const Ustring& >
                                     Signal_void_ustring;
 
                                     TextbufferDiary();
-        void                        set_textview( Gtk::TextView* ptr2textview )
+        void                        set_textview( TextviewDiary* ptr2textview )
         { m_ptr2textview = ptr2textview; }
-        void                        handle_logout();
+
         void                        handle_login();
+        void                        handle_edit_enabled();
+        void                        handle_logout();
 
         Link*                       get_link( int ) const;
         Link*                       get_link( const Gtk::TextIter& ) const;
 
         void                        set_richtext( Entry* );
-        Glib::ustring               get_selected_text() const;
+        Ustring                     get_text_to_save();
+        Ustring                     get_selected_text() const;
 
-        void                        set_search_str( const Glib::ustring& );
+        void                        set_search_str( const Ustring& );
         bool                        select_searchstr_previous();
         bool                        select_searchstr_next();
 
-        void                        toggle_format(  Glib::RefPtr< Tag >,
-                                                    const Glib::ustring& );
+        void                        toggle_format( Glib::RefPtr< Tag >,
+                                                   const Ustring& );
         void                        toggle_bold();
         void                        toggle_italic();
         void                        toggle_highlight();
@@ -313,7 +326,7 @@ class TextbufferDiary : public Gtk::TextBuffer, public EntryParser
 
         void                        calculate_para_bounds( Gtk::TextIter&, Gtk::TextIter& );
         bool                        calculate_multi_para_bounds( Gtk::TextIter&,
-                                                                          Gtk::TextIter& );
+                                                                 Gtk::TextIter& );
         void                        handle_indent();
         void                        handle_unindent();
         void                        set_list_item_mark( char );
@@ -335,13 +348,12 @@ class TextbufferDiary : public Gtk::TextBuffer, public EntryParser
 
     protected:
         void                        on_insert( const Gtk::TextIter&,
-                                               const Glib::ustring&,
-                                               int );
+                                               const Ustring&, int ) override;
         void                        on_erase( const Gtk::TextIter&,
-                                              const Gtk::TextIter& );
+                                              const Gtk::TextIter& ) override;
 
         void                        on_mark_set( const TextBuffer::iterator&,
-                                                 const Glib::RefPtr< TextBuffer::Mark >& );
+                                                 const Glib::RefPtr< TextBuffer::Mark >& ) override;
 
         void                        handle_space();
         bool                        handle_minus();
@@ -355,52 +367,55 @@ class TextbufferDiary : public Gtk::TextBuffer, public EntryParser
         void                        apply_hidden_link_tags( Gtk::TextIter&,
                                             const Glib::RefPtr< Tag >& );
 
-        void                        apply_heading( bool );
-        void                        apply_subheading();
-        void                        apply_bold();
-        void                        apply_italic();
-        void                        apply_strikethrough();
-        void                        apply_highlight();
-        void                        apply_comment();
-        void                        apply_ignore();
-        void                        apply_link();
-        void                        apply_link_date();
-        void                        apply_link_id();
+        void                        apply_heading( bool ) override;
+        void                        apply_subheading() override;
+        void                        apply_subsubheading() override;
+        void                        apply_bold() override;
+        void                        apply_italic() override;
+        void                        apply_strikethrough() override;
+        void                        apply_highlight() override;
+        void                        apply_comment() override;
+        void                        apply_ignore() override;
+        void                        apply_link_hidden() override;
+        void                        apply_link() override;
+        void                        apply_date() override;
         void                        apply_check( Glib::RefPtr< Gtk::TextTag >*,
                                                  Glib::RefPtr< Gtk::TextTag >*,
                                                  int );
-        void                        apply_check_ccl();
-        void                        apply_check_unf();
-        void                        apply_check_prg();
-        void                        apply_check_fin();
-        void                        apply_match();
-        void                        apply_indent();
+        void                        apply_check_ccl() override;
+        void                        apply_check_unf() override;
+        void                        apply_check_prg() override;
+        void                        apply_check_fin() override;
+        void                        apply_chart() override;
+
+        void                        apply_match() override;
+        void                        apply_indent() override;
 
         void                        apply_markup( const Glib::RefPtr< Tag >& );
 
         void                        update_todo_status();
 
-        Wchar                       get_char_at( int );
+        Wchar                       get_char_at( int ) override;
 
         bool                        check_cursor_is_in_para();
 
         int                         get_text_width( const Pango::FontDescription&,
-                                                    const Glib::ustring& );
+                                                    const Ustring& );
 
-        void                        handle_word();
+        void                        check_word() override;
 
         void                        handle_spellcheck_toggled();
         void                        handle_language_changed( const std::string&,
                                                              const Gtk::RadioMenuItem* );
         Gtk::Menu*                  build_languages_menu();
-        void                        add_suggestion_menus( const Glib::ustring&, Gtk::Menu* );
-        void                        ignore_misspelled_word( const Glib::ustring& );
-        void                        replace_misspelled_word( const Glib::ustring& );
-        void                        add_word_to_dictionary( const Glib::ustring& );
+        void                        add_suggestion_menus( const Ustring&, Gtk::Menu* );
+        void                        ignore_misspelled_word( const Ustring& );
+        void                        replace_misspelled_word( const Ustring& );
+        void                        add_word_to_dictionary( const Ustring& );
 
-        int                         m_parser_pos_cur_para_begin{ 0 };
-        int                         m_parser_pos_cur_para_end{ 0 };
-        int                         m_pos_cursor{ 0 };
+        Ustring::size_type          m_parser_pos_cur_para_begin{ 0 };
+        Ustring::size_type          m_parser_pos_cur_para_end{ 0 };
+        Ustring::size_type          m_pos_cursor{ 0 };
 
         static PredicateNL          s_predicate_nl;
 
@@ -418,6 +433,7 @@ class TextbufferDiary : public Gtk::TextBuffer, public EntryParser
         // ...there may be other instances of TextbufferDiary in the future
         Glib::RefPtr< Tag >         m_tag_heading;
         Glib::RefPtr< Tag >         m_tag_subheading;
+        Glib::RefPtr< Tag >         m_tag_subsubheading;
         Glib::RefPtr< Tag >         m_tag_match;
         Glib::RefPtr< Tag >         m_tag_markup;
         Glib::RefPtr< Tag >         m_tag_hidden;
@@ -448,7 +464,7 @@ class TextbufferDiary : public Gtk::TextBuffer, public EntryParser
         bool                        m_flag_updating_search_text{ false };
         bool                        m_flag_outstanding_change{ false };
         Entry*                      m_ptr2entry{ nullptr };
-        Gtk::TextView*              m_ptr2textview{ nullptr };
+        TextviewDiary*              m_ptr2textview{ nullptr };
 
         typedef std::list< Link* >  ListLinks;
         ListLinks                   m_list_links;
@@ -473,6 +489,8 @@ class TextviewDiary : public Gtk::TextView
                                                    const Glib::RefPtr<Gtk::Builder>& );
 
         void                        set_richtext( Entry* );
+        void                        clear_link()
+        { m_link_hovered = nullptr; }
 
         TextbufferDiary*            m_buffer;
 
